#include <stdio.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include "spct6100.h"

#define DDR_DATA_FILE "./bin/ddr_config_128_sync_16bit.data.bin"
#define DDR_INST_FILE "./bin/ddr_config_128_sync_16bit.text.bin"

#define DATA_FILE "./bin/ST6100_TABLE.bin"
#define INST_FILE "./bin/dsp_stub_main.text.bin"
#define TimerISR_INST_FILE "./bin/TimerISR.text.bin"

static int user_force_stop = 0;
static void sig_handler(int sig, siginfo_t *sig_info, void *unused)
{
    user_force_stop = 1;
}

int main(int argc, char *argv[])
{
    void *mmap_addr;
    struct pollfd pfd;
    int i;
    FILE *fp[4], *fp_audio[4], *fp_live[4];
    int fd;
    int channel;
    struct spct6100_property property;
#if 0
    struct tm T;
    time_t t;
#endif
    struct sigaction sig_act, old_act[3];
    char *device_name;
    char file_name[512];
    char prefix_name[256];
    int got_I_frame[4];
    int width[4], height[4];
    int mw[4], mh[4];
    char *sub_str;

    int standard[4];
    int resolution[4];
    int preview_enable[4];
    int encoder_enable[4];
    int audio_enable[4];
    int vlost_enable[4];
    int mdect_enable[4];

    if (argc != 2) {
        printf("Usage: %s /dev/pacdmp_dev[0-9]\n", argv[0]);
        return -1;
    }
    device_name = argv[1];

    printf("SDK %s\n\n", spct6100_sdk_version());

    fd  = spct6100_open_device(device_name);

    if (fd < 0) {
        printf("spct6100_open_device failed\n");
        return -1;
    }

    if (spct6100_init_dsp(fd, DDR_DATA_FILE, DDR_INST_FILE) != 0) {
        printf("spct6100_init_dsp failed\n");
        return -1;
    }

    if (spct6100_init_system(fd, DATA_FILE, INST_FILE, TimerISR_INST_FILE) != 0) {
        printf("spct6100_init_system failed\n");
        return -1;
    }

    for( channel = 0; channel < 4; channel++ ) {
        standard[channel] = STD_PAL;

        if (channel == 0) {
            resolution[channel] = R_4CIF;
            preview_enable[channel] = 1;
            encoder_enable[channel] = 1;
            audio_enable[channel] = 0;
            vlost_enable[channel] = 0;
            mdect_enable[channel] = 0;
        } else {
            resolution[channel] = R_CIF;
            preview_enable[channel] = 0;
            encoder_enable[channel] = 0;
            audio_enable[channel] = 0;
            vlost_enable[channel] = 0;
            mdect_enable[channel] = 0;
        }
        switch (resolution[channel]) {
            case R_4CIF:
                width[channel] = 352*2;
                height[channel] = (standard[channel] == STD_PAL) ? 288*2 : 240*2;
                break;

            case R_CIF:
                width[channel] = 352;
                height[channel] = (standard[channel] == STD_PAL) ? 288 : 240;
                break;

            case R_QCIF:
                width[channel] = 352/2;
                height[channel] = (standard[channel] == STD_PAL) ? 288/2 : 240/2;
                break;

            case R_2CIF:
                width[channel] = 352*2;
                height[channel] = (standard[channel] == STD_PAL) ? 288 : 240;
                break;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_STANDARD;
        property.channel = channel;
        property.context.value = standard[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    property.type = CARD_TIME_CTRL;
#if 0
    /* 2011-1-8 16:23:56 */
    T.tm_year = 2011 - 1900;
    T.tm_mon = 1 - 1;
    T.tm_mday = 8;
    T.tm_hour = 16;
    T.tm_min = 23;
    T.tm_sec = 56;
    t = mktime (&T);

    property.context.time_ctrl.ctrl_type = 0;  // obsolute time
    property.context.time_ctrl.time = t;
#else
    /* UTC ----> CST(+8 hours) */
    property.context.time_ctrl.ctrl_type = 1;  // relative time: now(UTC) + time_diff
    property.context.time_ctrl.time = 8*3600;  // time_diff = 8*3600
#endif
    if( spct6100_set_property(fd, &property) != 0 ) {
        return -1;
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_RESOLUTION;
        property.channel = channel;
        property.context.value = resolution[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_PREVIEW_ENABLE;
        property.channel = channel;
        property.context.value = preview_enable[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_ENCODER_ENABLE;
        property.channel = channel;
        property.context.value = encoder_enable[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_AUDIO_ENABLE;
        property.channel = channel;
        property.context.value = audio_enable[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_VLOST_ENABLE;
        property.channel = channel;
        property.context.value = vlost_enable[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

#if 0
    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_RATE_CTRL;
        property.channel = channel;
        property.context.rate_ctrl.ctrl_type = 0;
        property.context.rate_ctrl.bitrate_max = 256;
        property.context.rate_ctrl.bitrate_min = 64;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }
#else
    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_RATE_CTRL;
        property.channel = channel;
        property.context.rate_ctrl.ctrl_type = 1;
        property.context.rate_ctrl.qpi = 29;
        property.context.rate_ctrl.qpp = 30;
        property.context.rate_ctrl.qpb = 31;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }
#endif

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_GOP;
        property.channel = channel;
        property.context.gop.gopp = 30;
        property.context.gop.gopb = 2;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_MDECT;
        property.channel = channel;
        property.context.mdect.mthreshold = 0x2000;
        property.context.mdect.minterval = 2;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_MDECT_ENABLE;
        property.channel = channel;
        property.context.value = mdect_enable[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_FPS;
        property.channel = channel;
        property.context.value = 30;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_COLOR;
        property.channel = channel;
        property.context.color.hue = 0x80;
        property.context.color.contrast = 0x80;
        property.context.color.saturation = 0x80;
        property.context.color.brightness = 0x80;

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_OSD;
        property.channel = channel;
        property.context.osd.color_index = 1;
        property.context.osd.osd0_enable = 1;
        property.context.osd.osd0_x = 32;
        property.context.osd.osd0_y = 32;
        strcpy(property.context.osd.osd0_str,
                "\217\216\215\214/\213\212/\211\210 \205\204:\203\202:\201\200");
        property.context.osd.osd1_enable = 1;
        property.context.osd.osd1_x = 100;
        property.context.osd.osd1_y = 100;
        sprintf(property.context.osd.osd1_str, "Channel-%d: Hello, SPCT6100", channel);

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        if (channel != 0) continue;
        property.type = CHANNEL_MASK;
        property.channel = channel;
        property.context.mask.area_num = 2;
        property.context.mask.area[0].enable = 1;
        property.context.mask.area[0].x0 = 0;
        property.context.mask.area[0].y0 = 0;
        property.context.mask.area[0].x1 = width[channel]/2;
        property.context.mask.area[0].y1 = height[channel]/2;

        property.context.mask.area[1].enable = 1;
        property.context.mask.area[1].x0 = property.context.mask.area[0].x1;
        property.context.mask.area[1].y0 = property.context.mask.area[0].y1;
        property.context.mask.area[1].x1 = width[channel];
        property.context.mask.area[1].y1 = height[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    for( channel = 0; channel < 4; channel++ ) {
        property.type = CHANNEL_POSITION;
        property.channel = channel;
        property.context.position.enable = 0;
        property.context.position.x = 2;
        property.context.position.y = 2;
        property.context.position.w = width[channel];
        property.context.position.h = height[channel];

        if( spct6100_set_property(fd, &property) != 0 ) {
            printf("channel %d spct6100_set_property failed at line %d\n",
                   channel, __LINE__);
            return -1;
        }
    }

    if( spct6100_system_start(fd) != 0 ) {
        printf ("spct6100_system_start failed\n");
        goto err_out;
    }

    mmap_addr = spct6100_mmap(fd);

    spct6100_streamon(fd);

    pfd.fd = fd;
    pfd.events = POLLIN;

    sub_str = strstr(device_name, "pacdmp_dev");
    if (sub_str) {
        strcpy(prefix_name, sub_str + sizeof("pacdmp_") - 1);
    } else {
        prefix_name[0] = '\0';
    }
    for (i = 0; prefix_name[i] != '\0'; i++) {
        if (prefix_name[i] == '/' || prefix_name[i] == '\\') {
            prefix_name[i] = '_';
        }
    }
    for (channel = 0; channel < 4; channel++) {
        sprintf(file_name, "%s_video%d.264", prefix_name, channel);
        fp[channel] = fopen(file_name, "wb");

        sprintf(file_name, "%s_audio%d.723", prefix_name, channel);
        fp_audio[channel] = fopen(file_name, "wb");

        sprintf(file_name, "%s_video%d.yuv", prefix_name, channel);
        fp_live[channel] = fopen(file_name, "wb");
    }

    /* Setup signal handler. */
    sigemptyset(&sig_act.sa_mask);
    sig_act.sa_sigaction = sig_handler;
    sig_act.sa_flags = 0;

    sigaction(SIGHUP, &sig_act, &old_act[0]);
    sigaction(SIGINT, &sig_act, &old_act[1]);
    sigaction(SIGTERM, &sig_act, &old_act[2]);

    // Every motion block is 32X32.
    // Motion buffer is a bitmap.
    //
    // | <---------------- 32 bit ------------------>|
    //
    // | <----------- <mw> bits   ----------->|
    // | <----------- <mw> blocks ----------->|
    // | IMAGE Block "32X32"
    // |---------------------------------------  ---
    // |       |       |       |      |       |   ^
    // | 32X32 | 32X32 | 32X32 | ...  | 32X32 |   |
    // |       |       |       |      |       |   |
    // |---------------------------------------   |
    // |       |       |       |      |       |   |
    // | 32X32 | 32X32 | 32X32 | ...  | 32X32 |   |
    // |       |       |       |      |       |   |
    // |---------------------------------------   |
    // |       |       |       |      |       |  <mh> lines
    // | 32X32 | 32X32 | 32X32 | ...  | 32X32 |   |
    // |       |       |       |      |       |   |
    // |---------------------------------------   |
    // |       |       |       |      |       |   |
    // | ...   | ...   | ...   | ...  | ...   |   |
    // |       |       |       |      |       |   |
    // |---------------------------------------   |
    // |       |       |       |      |       |   |
    // | 32X32 | 32X32 | 32X32 | ...  | 32X32 |   |
    // |       |       |       |      |       |   v
    // |---------------------------------------  ---
    //
    // Motion buffer uses 32-bit to represent mw blocks motion information.
    // So, buffer size is 32*mh

    for (channel = 0; channel < 4; channel++) {
        mw[channel] = (width[channel] + 31) / 32;
        mh[channel] = (height[channel] + 31) / 32;
    }

    memset (got_I_frame, 0, sizeof (got_I_frame));
    while (!user_force_stop) {
        struct spct6100_buffer buf;
        int n = spct6100_poll(&pfd, 1, 1000);

        if (n < 0) {
            printf("spct6100_poll error\n");
            break;
        } else if (n == 0) {
            printf("spct6100_poll timeout\n");
            continue;
        }

        if (spct6100_get_data(fd, mmap_addr, &buf) != 0) {
            printf("spct6100_get_data error\n");
            break;
        }

        for (i = 0; i < buf.n_buffer; i++) {
            if (buf.buffer[i].has_video) {
                if (got_I_frame[buf.channel]) {
                    //printf("channel %d has video len = %d\n", buf.channel, buf.buffer[i].video_len);
                    fwrite(buf.buffer[i].video_buffer, 1, buf.buffer[i].video_len, fp[buf.channel]);
                } else if (buf.buffer[i].video_frame_type == 'I') {
                    printf ("%s channel %d has got an I frame\n", device_name, buf.channel) ;
                    got_I_frame[buf.channel] = 1;
                    //printf("channel %d has video len = %d\n", buf.channel, buf.buffer[i].video_len);
                    fwrite(buf.buffer[i].video_buffer, 1, buf.buffer[i].video_len, fp[buf.channel]);
                }
            }
            if (buf.buffer[i].has_audio) {
                //printf("channel %d has audio len = %d\n", buf.channel, buf.buffer[i].audio_len);
                fwrite(buf.buffer[i].audio_buffer, 1, buf.buffer[i].audio_len, fp_audio[buf.channel]);
            }
            if (buf.buffer[i].has_motion) {
                int x, y;
                //printf("channel %d has motion len = %d\n", buf.channel, buf.buffer[i].motion_len);
                for (y = 0; y < mh[buf.channel]; y++) {
                    unsigned int motion;
                    motion = *(unsigned int *)&buf.buffer[i].motion_buffer[y * 4];

                    for (x = 0; x < mw[buf.channel]; x++, motion >>= 1) {
                        if (motion & 1) {
                            printf("%s channel %d 32X32 block (%d, %d) motion detected!\n",
                                    device_name, buf.channel, x, y);
                        }
                    }
                }
            }
            if (buf.buffer[i].has_vlost) {
                //printf("channel %d has vlost = %d\n", buf.channel, buf.buffer[i].vlost);
            }
            if (buf.buffer[i].has_yuv) {
                //printf("channel %d has yuv len = %d\n", buf.channel, buf.buffer[i].yuv_len);
                fwrite(buf.buffer[i].yuv_buffer, 1, buf.buffer[i].yuv_len, fp_live[buf.channel]);
            }
        }

        if (spct6100_free_data(fd) != 0) {
            printf("spct6100_free_data error\n");
            break;
        }
    }

    sigaction(SIGHUP, &old_act[0], NULL);
    sigaction(SIGINT, &old_act[1], NULL);
    sigaction(SIGTERM, &old_act[2], NULL);

    for (channel = 0; channel < 4; channel++) {
        fclose(fp[channel]);
        fclose(fp_audio[channel]);
        fclose(fp_live[channel]);
    }

    spct6100_streamoff(fd);

    spct6100_munmap(mmap_addr);

err_out:
    spct6100_system_stop(fd);

    spct6100_close_device(fd);

    return 0;
}
