#ifndef _PACDMP_PCI_H_
#define _PACDMP_PCI_H_

#define PACDMP_PCI_VERSION "1.0"
#define MODULENAME "pacdmp_pci"

#define PACDMP_PCI_VENDOR_ID 0x0216
#define PACDMP_PCI_DEVICE_ID 0x6100

#define PACDMP_NAME "pacdmp_dev"

struct pacdmp_core {
    /* device structures */
    struct pci_dev    *pci;

    /* device configuration */
    char              name[32];
    unsigned int      nr;
};

struct pacdmp_dev
{
    struct pacdmp_core core;
    struct cdev        cdev;

    /* pci device & i/o  */
    void  __iomem    *mmio_vaddr;    /* kernel virtual addresss pointer to mmio */
    u32               mmio_size;     /* size of mmio */

    /* locking info */
    spinlock_t        irq_lock;
    spinlock_t        reg_lock;
    struct mutex      m_lock;
    struct mutex      file_lock;
    struct mutex      queue_lock;

    /* bus mastering window */
    unsigned long     master_free_pages;
    u8               *master_buf;
    dma_addr_t        master_buf_phy;

    /* status */
    unsigned int      irq_total;
    unsigned int      irq_self;
    unsigned int      irq_flag;

    unsigned int      users;

    /* data queue */
    unsigned int      hst_rptr;
    unsigned int      hst_len;
    unsigned int      hst_num;

    /* dsp command queue */
    unsigned int      cmdque_base;
    unsigned int      cmdque_len;
    unsigned int      cmdque_num;
    unsigned int      cmdque_wptr;

    /* vic2 data control */
    int               streamon;
    int               wakeup;
    int               delay_count;

    /* vic1 data control */
    int               live_streamon;
    int               live_wakeup;
    unsigned int      live_data;
};

#endif /* _PACDMP_PCI_H_ */
