#ifndef _PACDMP_TYPES_H_
#define _PACDMP_TYPES_H_

#include "spct6100-types.h"

#define u8      unsigned char
#define u16     unsigned short
#define u32     unsigned int

#define PACDMP_MMIO_SIZE  0x1000000 // 16MB

#define PACDMP_IOC_MAGIC  0xEF

typedef struct pacdmp_reg32_t
{
    unsigned int value;
    unsigned int base; // unit: byte
    unsigned int offset;
} REG32;

typedef struct pacdmp_reg16_t
{
    unsigned short value;
    unsigned int base;
    unsigned int offset;
} REG16;

typedef struct pacdmp_reg8_t
{
    unsigned char value;
    unsigned int base;
    unsigned int offset;
} REG8;

typedef struct pacdmp_queue_t
{
    unsigned int base;
    unsigned int len;
    unsigned int num;
} QUEUE;

typedef struct pacdmp_property_t
{
    struct spct6100_property *prop;
    unsigned char *buffer;
    unsigned int len;
    unsigned int retry;
} PACDMP_PROPERTY;

typedef enum pacdmp_rw_cmd_type_e
{
    RW_IOCTL = 1,
    RW_MEMCPY
} PACDMP_RW_CMD_TYPE;

typedef enum pacdmp_boot_dsp_mode_e
{
    BOOT_DIRECT = 0x3064,
    BOOT_ICACHE_DONE = 0x3068
} PACDMP_BOOT_MODE;

#define PACDMP_CTL_SET_WINDOW_BASE  _IOW (PACDMP_IOC_MAGIC,  0, unsigned int)
#define PACDMP_CTL_READ_REG32       _IOWR(PACDMP_IOC_MAGIC,  1, unsigned int)
#define PACDMP_CTL_WRITE_REG32      _IOWR(PACDMP_IOC_MAGIC,  2, unsigned int)
#define PACDMP_CTL_READ_REG16       _IOWR(PACDMP_IOC_MAGIC,  3, unsigned int)
#define PACDMP_CTL_WRITE_REG16      _IOWR(PACDMP_IOC_MAGIC,  4, unsigned int)
#define PACDMP_CTL_READ_REG8        _IOWR(PACDMP_IOC_MAGIC,  5, unsigned int)
#define PACDMP_CTL_WRITE_REG8       _IOWR(PACDMP_IOC_MAGIC,  6, unsigned int)
#define PACDMP_CTL_GET_PHYBAR       _IOR (PACDMP_IOC_MAGIC,  7, unsigned int)
#define PACDMP_CTL_GET_ISRFLG       _IOR (PACDMP_IOC_MAGIC,  8, unsigned int)
#define PACDMP_CTL_QUERYBUF         _IOR (PACDMP_IOC_MAGIC,  9, unsigned int)
#define PACDMP_CTL_STREAMON         _IO  (PACDMP_IOC_MAGIC, 10)
#define PACDMP_CTL_STREAMOFF        _IO  (PACDMP_IOC_MAGIC, 11)
#define PACDMP_CTL_QBUF             _IOW (PACDMP_IOC_MAGIC, 12, unsigned int)
#define PACDMP_CTL_DQBUF            _IOR (PACDMP_IOC_MAGIC, 13, unsigned int)
#define PACDMP_CTL_LIVE_STREAMON    _IO  (PACDMP_IOC_MAGIC, 14)
#define PACDMP_CTL_LIVE_STREAMOFF   _IO  (PACDMP_IOC_MAGIC, 15)
#define PACDMP_CTL_LIVE_QBUF        _IOW (PACDMP_IOC_MAGIC, 16, unsigned int)
#define PACDMP_CTL_LIVE_DQBUF       _IOR (PACDMP_IOC_MAGIC, 17, unsigned int)
#define PACDMP_CTL_INIT_QUEUE       _IOW (PACDMP_IOC_MAGIC, 18, unsigned int)
#define PACDMP_CTL_SET_PROPERTY     _IOW (PACDMP_IOC_MAGIC, 19, unsigned int)
#define PACDMP_CTL_GET_PROPERTY     _IOR (PACDMP_IOC_MAGIC, 20, unsigned int)

#endif /* _PACDMP_TYPES_H_ */
