#ifndef SPCT6100_TYPES_H
#define SPCT6100_TYPES_H

/* Stadared: 0:NTSC, 1:PAL */
enum {
    STD_NTSC = 0,
    STD_PAL = 1,
};
#define DEFAULT_STANDARD  STD_NTSC

/* Resolution: 0:4CIF, 1:CIF, 2:QCIF, 3:2CIF */
enum {
    R_4CIF = 0,
    R_CIF = 1,
    R_QCIF = 2,
    R_2CIF = 3,
};
#define DEFAULT_RESOLUTION  R_CIF

/* GPIO Direction: 0:Out, 1:In */
enum {
    GPIO_OUT = 0,
    GPIO_IN = 1,
};

enum {
    CARD_TIME_CTRL,
    CARD_GPIO_DIRECTION,
    CARD_GPIO_DATA,
    CHANNEL_STANDARD,
    CHANNEL_RESOLUTION,
    CHANNEL_PREVIEW_ENABLE,
    CHANNEL_ENCODER_ENABLE,
    CHANNEL_AUDIO_ENABLE,
    CHANNEL_VLOST_ENABLE,
    CHANNEL_RATE_CTRL,
    CHANNEL_GOP,
    CHANNEL_MDECT,
    CHANNEL_MDECT_ENABLE,
    CHANNEL_FPS,
    CHANNEL_COLOR,
    CHANNEL_OSD,
    CHANNEL_MASK,
    CHANNEL_POSITION,
};

typedef struct time_ctrl_t {
    int ctrl_type;
    int time;
} TIME_CTRL;

typedef struct rate_ctrl_t {
    int ctrl_type;
    int max_bitrate;
    int min_bitrate;
    int qpi;
    int qpp;
    int qpb;
} RATE_CTRL;

typedef struct gop_t {
    int gopp;
    int gopb;
} GOP;

typedef struct mdect_t {
    int mthreshold;
    int minterval;
} MDECT;

typedef struct color_t {
    unsigned char hue;
    unsigned char contrast;
    unsigned char saturation;
    unsigned char brightness;
} COLOR;

#define MAX_OSD_STRING_LENGTH  (40)
typedef struct osd_t {
    int color_index;
    int osd0_enable;
    unsigned short osd0_x;
    unsigned short osd0_y;
    char osd0_str[MAX_OSD_STRING_LENGTH];
    int osd1_enable;
    unsigned short osd1_x;
    unsigned short osd1_y;
    char osd1_str[MAX_OSD_STRING_LENGTH];
} OSD;

typedef struct mask_t {
    int area_num;
    struct area_t {
        int enable;
        unsigned short x0;
        unsigned short y0;
        unsigned short x1;
        unsigned short y1;
    } area[6];
} MASK;

typedef struct position_t {
    int enable;
    unsigned short x;
    unsigned short y;
    unsigned short w;
    unsigned short h;
} POSITION;

typedef struct gpio_t {
    unsigned short pin;
    unsigned short data;
} GPIO;

struct spct6100_property {
    int type;
    int channel;
    union {
        int value;
        TIME_CTRL time_ctrl;
        RATE_CTRL rate_ctrl;
        GOP gop;
        MDECT mdect;
        COLOR color;
        OSD osd;
        MASK mask;
        POSITION position;
        GPIO gpio;
    } context;
};

struct spct6100_buffer {
    int channel;
    int n_buffer;
    struct buffer_t {
        unsigned char  has_video;
        char           video_frame_type;
        char          *video_buffer;
        int            video_len;
        unsigned int   video_timestamp;

        unsigned char  has_audio;
        char          *audio_buffer;
        int            audio_len;
        unsigned int   audio_timestamp;

        unsigned char  has_vlost;
        int            vlost;

        unsigned char  has_motion;
        char          *motion_buffer;
        int            motion_len;

        unsigned char  has_yuv;
        char          *yuv_buffer;
        int            yuv_len;
    } buffer[4];
};

#endif /* SPCT6100_TYPES_H */
