/*
 * =====================================================================================
 *       Copyright (C), 2010-2020, Bridge Team
 *       Filename:  ip_cam_dev.c
 *       Compiler:  gcc
 *
 *    Description:  implement ip cam device 
 *         Others:  none
 *        History:
 *
 *        Version:  1.0
 *        Created:  05/20/10 17:07:23
 *         Author:  liukai , liukai@sunnorth.com.cn
 *        Company:  Sunplus Core Technology
 *
 *   Modification:
 *
 *
 * =====================================================================================
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/poll.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <unistd.h>


#include <unistd.h>
#include <sys/mman.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <time.h>
#include <pthread.h>
#include <fcntl.h>

#include "spct_type.h"
#include "spct6100_dev.h"
#include "ip_cam_dev.h"

#define CAM_STATUS_RUN	0x1234
#define CAM_STATUS_INVAL 0
#define CAM_STATUS_FREE 0xabcd

#ifndef ENABLE
#define ENABLE 1
#endif

#ifndef DISABLE
#define DISABLE 0
#endif

#define PAL_WIDTH_SIZE		352
#define PAL_HEIGHT_SIZE		288
#define NTSC_WIDTH_SIZE		352
#define NTSC_HEIGHT_SIZE	240


typedef struct query_buf_res
{
	void * start;
	int length;
} Query_Buf_Res;


#define EXIT(x)                                                         \
    do{                                                                 \
        fprintf(stderr, "%s:%d failed: %s\n", __FILE__, __LINE__, x);   \
        exit(1);                                             \
    }while(0);

struct av_packet_header{
	unsigned int header_size;      //0x0
	unsigned int packet_size;      //0x04
	unsigned int pad0;             //0x08
	unsigned int frame_type;       //0x0c
	unsigned int video_timestamp;  //0x10
	unsigned int video_length;     //0x14
	unsigned int audio_length;     //0x18
	unsigned int reserved0;        //0x1c bsps
	unsigned int video_offset;     //0x20
	unsigned int audio_offset;     //0x24
	unsigned int reserved1;        //0x28 channel id
	char videolost_enable;         //0x2c
	char motion_enable;            //0x2d
	char encode_enable;            //0x2e
	char audio_enable;             //0x2f
	unsigned int bvideolost;       //0x30
	unsigned int motion_length;    //0x34
	unsigned int audio_timestamp;  //0x38
	unsigned char pad1[88 - 0x3c]; //0x3c
} __attribute__((packed));

struct packet_header{
	unsigned int video_flag;
	unsigned int video_type;
	char * video_addr;
	unsigned int video_length;
	unsigned int video_timestamp;

	unsigned int audio_flag;
	char * audio_addr;
	unsigned int audio_length;
	unsigned int audio_timestamp;

	unsigned int motion_flag;
	char * motion_addr;
	unsigned int motion_length;

	unsigned int videolost_flag;
};

/* buffer used between ap layer and v4l2method layer*/
struct buffer_av{
    unsigned int buf_index;
    unsigned int input;
    unsigned int flags;
    unsigned int length;
    unsigned int offset;
    struct timeval timestamp;
	unsigned int bytesused;
};

int streaming_off(int fd) {
    enum v4l2_buf_type type;
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    return ioctl(fd, VIDIOC_STREAMOFF, &type);
}



static int request_buffer(int fd) {
    struct v4l2_requestbuffers req;

    memset (&(req), 0, sizeof (req));
    req.count = 12;
    req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    req.memory = V4L2_MEMORY_MMAP;
    if( ioctl(fd, VIDIOC_REQBUFS, &req) != 0) {
        fprintf (stderr, "SPCT6100 V4L2 Driver does not support memory mapping\n");
        exit (EXIT_FAILURE);
    }

    return req.count; 
}

static int query_buffer(int fd, int index, unsigned int *length, unsigned int *offset ) {
    struct v4l2_buffer buf;

    memset(&(buf), 0, sizeof(buf));
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;
    buf.index = index;
    if(ioctl(fd, VIDIOC_QUERYBUF, &buf) !=0) {
        EXIT ("VIDIOC_QUERYBUF");
    }
    *length = buf.length;
    *offset = buf.m.offset;

    return 0;
}

static int queue_buffer(int fd, int index) {
    struct v4l2_buffer buf;

    memset (&(buf), 0, sizeof (buf));
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;
    buf.index = index;
    return ioctl(fd, VIDIOC_QBUF, &buf);
}

static int dequeue_buffer(int fd, struct buffer_av *buf_av) {
    struct v4l2_buffer buf;

    memset (&(buf), 0, sizeof (buf));
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;

    if(ioctl(fd, VIDIOC_DQBUF, &buf) !=0) {
        EXIT ("VIDIOC_DQBUF");
    }
    buf_av->buf_index = buf.index;
    buf_av->input = buf.input;
    buf_av->flags = buf.flags;
    buf_av->length = buf.length;
    buf_av->offset = buf.m.offset;
    buf_av->timestamp = buf.timestamp;
    buf_av->bytesused = buf.bytesused;

    return 0;
}

static int streaming_on(int fd) {
    enum v4l2_buf_type type;

    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    return ioctl(fd, VIDIOC_STREAMON, &type);
}



static void parse_packet(char ** packet_addr, struct packet_header * pkt_header)
{
	struct av_packet_header * av_pktp;

	av_pktp = (struct av_packet_header *)*packet_addr;

	if (av_pktp->encode_enable != 0 && av_pktp->video_length > 0){
		pkt_header->video_flag = 1;
		if(av_pktp->frame_type == 0)
			pkt_header->video_type = DATA_VIDEO_I;
		else if(av_pktp->frame_type == 2)
			pkt_header->video_type = DATA_VIDEO_P;
		else if(av_pktp->frame_type == 3)
			pkt_header->video_type = DATA_VIDEO_B;
		pkt_header->video_addr = (char *)av_pktp + av_pktp->video_offset;
		pkt_header->video_length = av_pktp->video_length;
		pkt_header->video_timestamp = av_pktp->video_timestamp;
	}
	else{
		pkt_header->video_flag = 0;
	}

	if (av_pktp->audio_enable != 0 && av_pktp->audio_length > 0){
		pkt_header->audio_flag = 1;
		pkt_header->audio_addr = (char *)av_pktp + av_pktp->audio_offset;
		pkt_header->audio_length = av_pktp->audio_length;
		pkt_header->audio_timestamp = av_pktp->audio_timestamp;
	}
	else{
		pkt_header->audio_flag = 0;
	}

	if (av_pktp->motion_enable != 0 && av_pktp->motion_length > 0){
		pkt_header->motion_flag = 1;
		pkt_header->motion_addr = (char *)av_pktp + av_pktp->header_size;
		pkt_header->motion_length = av_pktp->motion_length;
	}
	else{
		pkt_header->motion_flag = 0;
	}

	if (av_pktp->videolost_enable != 0 && av_pktp->bvideolost != 0){
	//if (av_pktp->bvideolost != 0){
		pkt_header->videolost_flag = 1;
	}
	else{
		pkt_header->videolost_flag = 0;
	}

	*packet_addr = (char*)*packet_addr + av_pktp->packet_size;
	return;
}
#define CHS_PER_CARD 4
static void * comp_proc(void * para)
{
	int i, j;
	int fd;
	int channel;
	int n_buffers = 0;
	unsigned int length, offset;
	int index, num;
	struct buffer_av buf_av;
	int result;
	struct pollfd pfd;
	unsigned int buf_index;
	unsigned int buf_num;
	char * packet_addr;
	struct packet_header pkt_header;
	Query_Buf_Res * buffers = NULL;
	Spct_Data comp_data;

	int got_video[CHS_PER_CARD];
	int got_audio[CHS_PER_CARD];
	struct timeval v_time_stamp[CHS_PER_CARD];
	struct timeval a_time_stamp[CHS_PER_CARD];
	unsigned int last_vtstamp[CHS_PER_CARD];
	unsigned int last_atstamp[CHS_PER_CARD];
	unsigned int duration;

	printf("thread compress\n");

	Ip_Cam_Device * ipcam = (Ip_Cam_Device *) para;

	memset(&buf_av, 0, sizeof(buf_av));

	memset(&got_video, 0, sizeof(got_video));
	memset(&got_audio, 0, sizeof(got_audio));
	memset(&last_vtstamp, 0, sizeof(last_vtstamp));
	memset(&last_atstamp, 0, sizeof(last_atstamp));


	fd = ipcam->comp_fd;

	n_buffers = request_buffer(fd);

	buffers = calloc(n_buffers, sizeof(* buffers));
	if(!buffers)
		EXIT("Out of memory.");

	for(i = 0; i < n_buffers; ++i)
	{
		if(query_buffer(fd, i, &length, &offset) != 0)
			EXIT("VIDIOC_QUERYBUF");
		buffers[i].length = length;
		buffers[i].start = mmap(NULL, length, PROT_READ | PROT_WRITE, MAP_SHARED, fd, offset);
		printf("buffers[%d].start = 0x%x\n",i, buffers[i].start);
		if(MAP_FAILED == buffers[i].start)
			EXIT("mmap");
	}

	for(index = 0; index < n_buffers; index++)
		if(queue_buffer(fd, index) != 0)
			EXIT("VIDIOC_QBUF");

	if(streaming_on(fd) != 0)
		EXIT("VIDIOC_STREAMON");
	printf("card stream on================================\n");
	pfd.fd = fd;
	pfd.events = POLLIN;

	while(ipcam->status == CAM_STATUS_RUN)
	{
		result = poll(&pfd, 1, 15000);

		if(result < 0)
			printf("pool ing errro ==================\n");
		if(result == 0)
		{
			printf("pool ing time out --------------\n");
			exit(1);
		}

		if(result < 0)
			continue;
		if(result == 0)
			continue;
		dequeue_buffer(fd, &buf_av);
		buf_index = buf_av.buf_index;
		buf_num = buf_av.length;
		for(i = 0; i < CHS_PER_CARD; i++)
		{
			index = buf_index & 0xff;
			num = buf_num & 0xff;

			buf_index >>= 8;
			buf_num >>= 8;
			if(index != 0xff)
			{
				/*DATA PACKET*/
				channel = i;

				for(j = 0, packet_addr = buffers[index].start; j < num; j++)
				{
					parse_packet(&packet_addr, &pkt_header);

					//pkt_header.motion_addr, 4 * 9
					if(pkt_header.motion_flag)
					{
						//printf("receive a motion\n\n\n\n");
					}

					if (pkt_header.videolost_flag)
					{
						// printf("video lost\n");
					}

					if(pkt_header.audio_flag)
					{//length
						if(!got_audio[channel])
						{
							got_audio[channel] = 1;
							//a_time_stamp[channel] = buf_av.timestamp;
							a_time_stamp[channel].tv_usec = ((pkt_header.audio_timestamp % 32768) * 1000ULL * 1000ULL) >> 15;
							a_time_stamp[channel].tv_sec = (pkt_header.audio_timestamp >> 15) + (a_time_stamp[channel].tv_usec / 1000000);
							a_time_stamp[channel].tv_usec %= 1000000;
						}
						else
						{
							duration = pkt_header.audio_timestamp - last_atstamp[channel];
							a_time_stamp[channel].tv_usec += ((duration % 32768) * 1000ULL * 1000ULL) >> 15;
							a_time_stamp[channel].tv_sec += (duration >> 15) + (a_time_stamp[channel].tv_usec / 1000000);
							a_time_stamp[channel].tv_usec %= 1000000;

						}
						last_atstamp[channel] = pkt_header.audio_timestamp;

						//printf("audio frame\n");
						comp_data.channel = channel;
						comp_data.type = DATA_AUDIO;
						comp_data.flags = 0;
						comp_data.timestamp = a_time_stamp[channel];
						comp_data.size = pkt_header.audio_length;
						comp_data.data = pkt_header.audio_addr;
						ipcam->fun(ipcam->datahandler, &comp_data);
					}

					if(pkt_header.video_flag)
					{

						if(!got_video[channel])
						{
							got_video[channel] = 1;
							//v_time_stamp[channel] = buf_av.timestamp;
							v_time_stamp[channel].tv_usec = ((pkt_header.video_timestamp % 32768) * 1000ULL * 1000ULL) >> 15;
							v_time_stamp[channel].tv_sec = (pkt_header.video_timestamp >> 15) + (v_time_stamp[channel].tv_usec / 1000000);
							v_time_stamp[channel].tv_usec %= 1000000;
						}
						else
						{
							duration = pkt_header.video_timestamp - last_vtstamp[channel];
							v_time_stamp[channel].tv_usec += ((duration % 32768) * 1000ULL * 1000ULL) >> 15;
							v_time_stamp[channel].tv_sec += (duration >> 15) + (v_time_stamp[channel].tv_usec / 1000000);
							v_time_stamp[channel].tv_usec %= 1000000;
						}
						last_vtstamp[channel] = pkt_header.video_timestamp;

						comp_data.channel = channel;
						comp_data.type = DATA_VIDEO;
						comp_data.flags = pkt_header.video_type;
						comp_data.timestamp = v_time_stamp[channel];
						comp_data.size = pkt_header.video_length;
						comp_data.data = pkt_header.video_addr;
						ipcam->fun(ipcam->datahandler, &comp_data);
					}
				}
			}
		}
		queue_buffer(fd, buf_av.buf_index);
	}

	if(streaming_off(fd) != 0)
		EXIT("VIDIOC_STREAMOFF");

	for(i = 0; i < n_buffers; ++i)
		munmap(buffers[i].start, buffers[i].length);

	if(buffers)
		free(buffers);

	return NULL;
}



static int ip_cam_open_device(Ip_Cam_Device * ipcam, char * devname)
{
	int i;
	int fd;
	int ch_num;
	int ch_opt = 0;
	ch_num = 4;
	fd = open(devname, O_RDWR);
	if (fd < 0)
		return -1;
	if (ch_num <= 0)
		return -1;

	ipcam->comp_fd = fd;

	return 0;
}

int ip_cam_construct(Ip_Cam_Device * ipcam, char * devname)
{
	int res ;
	if(ipcam == NULL)
		return -1;
	if(ip_cam_open_device(ipcam, devname))
		return -1;
	ipcam->cmptid = 0;
	return 0;
}

int ip_cam_destruct(Ip_Cam_Device * ipcam)
{
	ipcam->status = CAM_STATUS_FREE;
	if(ipcam->cmptid)
		pthread_join(ipcam->cmptid, NULL);
	if(ipcam && ipcam->comp_fd)
		close(ipcam->comp_fd);
	return 0;
}

int cam_start_work(Ip_Cam_Device * ipcam)
{
	if(pthread_create(&ipcam->cmptid, NULL, comp_proc, (void*)ipcam))
	{
		return -1;
	}

	//dev stream on
	ipcam->status = CAM_STATUS_RUN;

	return 0;
}


