#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/mman.h>
#include <errno.h>
#include <fcntl.h>
#include "spct6100-v4l2-types.h"

#define CLEAR(x) memset (&(x), 0, sizeof (x))

int fd;
int stop;
int got_sps;
int totalframes;
struct buffer
{
    void *start;
    size_t length;
};
struct buffer *buffers = NULL;
FILE *output[4];
void save_one_frame_pcm(struct v4l2_buffer buf);
void process_data (void);

    static void
errno_exit (const char *s)
{
    fprintf (stderr, "%s error %d, %s\n", s, errno, strerror (errno));
    exit (EXIT_FAILURE);
}

void poked(int sig)
{
    stop = 1;
}

    int
main (int argc, char **argv)
{
    int index;
    int n = 0;
    unsigned int i;
    struct pollfd pfd;
    enum v4l2_buf_type type;
    struct v4l2_requestbuffers reqbuf;

    signal(SIGHUP, poked);
    signal(SIGINT, poked);
    signal(SIGTERM, poked);

    fd = open ("/dev/video1", O_RDWR | O_SYNC);

    if (fd < 0) {

        printf ("Open device failed\n");
    }

    /*** Step 1 Start ***/
    memset (&reqbuf, 0, sizeof (reqbuf));
    reqbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    reqbuf.memory = V4L2_MEMORY_MMAP;
    reqbuf.count = 12;
    if (-1 == ioctl (fd, VIDIOC_REQBUFS, &reqbuf)) {

        if (errno == EINVAL)
            printf ("Video capturing or mmap-streaming is not supported\n");
        else
            perror ("VIDIOC_REQBUFS");
        exit (EXIT_FAILURE);
    }
    /*** Step 1 End ***/

    /*** Step 2 Start ***/
    buffers = calloc (reqbuf.count, sizeof (*buffers));
    for (i = 0; i < reqbuf.count; i++) {

        struct v4l2_buffer buffer;
        memset (&buffer, 0, sizeof (buffer));
        buffer.type = reqbuf.type;
        buffer.memory = V4L2_MEMORY_MMAP;
        buffer.index = i;
        if (-1 == ioctl (fd, VIDIOC_QUERYBUF, &buffer)) {

            perror ("VIDIOC_QUERYBUF");
            exit (EXIT_FAILURE);
        }
        buffers[i].length = buffer.length;
        buffers[i].start = mmap (NULL,
                buffer.length,
                PROT_READ | PROT_WRITE, /* recommended */
                MAP_SHARED, /* recommended */
                fd,
                buffer.m.offset);
        if (MAP_FAILED == buffers[i].start) {

            perror ("mmap");
            exit (EXIT_FAILURE);
        }
    }

    // Initialize all buffers ***/
    for (index = 0; index < reqbuf.count; index++) {

        struct v4l2_buffer buf;
        CLEAR (buf);
        buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        buf.memory = V4L2_MEMORY_MMAP;
        buf.index = index;
        if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
            errno_exit ("VIDIOC_QBUF");
    }
    /*** Step 2 End ***/

    /*** Step 3 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMON, &type))
        errno_exit ("VIDIOC_STREAMON");
    /*** Step 3 End ***/

    output[0] = fopen ("audio0.723", "wb");
    output[1] = fopen ("audio1.723", "wb");
    output[2] = fopen ("audio2.723", "wb");
    output[3] = fopen ("audio3.723", "wb");

    /*** Step 4 Start ***/
    pfd.fd = fd;
    pfd.events = POLLIN;
    while (!stop) {
        n = poll (&pfd, 1, -1);
        if (n < 0) {
            break;
        } else {
            process_data ();
        }
    }
    /*** Step 4 Stop ***/

    /*** Step 5 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMOFF, &type))
        errno_exit ("VIDIOC_STREAMOFF");
    /*** Step 5 Stop ***/

    fclose (output[0]);
    fclose (output[1]);
    fclose (output[2]);
    fclose (output[3]);
    close (fd);
    return 0;
}                // End of main

    void
process_data (void)
{
    struct v4l2_buffer buf;
    CLEAR (buf);
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;
    if (-1 == ioctl (fd, VIDIOC_DQBUF, &buf))
        errno_exit ("VIDIOC_DQBUF");
    save_one_frame_pcm(buf);
    if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
        errno_exit ("VIDIOC_QBUF");
}

void save_one_frame_pcm(struct v4l2_buffer buf)
{
    unsigned int buf_index = buf.index;
    unsigned int buf_len = buf.length;
    unsigned int index, subpacket_num, channel_index, subpacket_index;
    unsigned char *cur_addr = NULL;

    for (channel_index = 0; channel_index < 4; channel_index++) {
        index = buf_index & 0xff;
        subpacket_num = buf_len & 0xff;
        buf_index >>= 8;
        buf_len >>= 8;

        /* We only save audio data from channel 2 */
        if (index != 0xff) {
            cur_addr = buffers[index].start;
            for (subpacket_index = 0; subpacket_index < subpacket_num; subpacket_index++) {
                if (PACKET_WITH_AUDIO(cur_addr)) {
                    printf("Save pcm frame %d\n", totalframes);
                    fwrite(cur_addr + AV_AOFFSET(cur_addr), AV_ALEN(cur_addr), 1, output[channel_index]);
                    totalframes++;
                }
                cur_addr+=AV_TOTALSIZE(cur_addr);
            }
        }
    }
}
