/*
 * This programe demos how to change the brightness.
 * We first get the current value of the brightness of channel 0,
 * then we reset it to the value specified at command line.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "spct6100-v4l2-types.h"

#define CLEAR(x) memset (&(x), 0, sizeof (x))

    static void
errno_exit(const char *s)
{
    fprintf(stderr, "%s error %d, %s\n", s, errno, strerror(errno));
    exit(EXIT_FAILURE);
}

void validate_value(int new_brightness_value)
{
    if (new_brightness_value < 0
            || new_brightness_value > 255) {
        printf("Brightness should be within 0 - 255\n");
        exit(1);
    }
}

int main (int argc, char *argv[])
{
    int fd = -1;
    int channelnum = 0;
    int new_brightness_value = 0;
    struct v4l2_control brightness_control;

    if (argc != 2) {

        printf("Usage: programe-name [new brightness value]\n");
        exit(1);
    }

    fd = open( "/dev/video0", O_RDWR);
    if (fd < 0) {

        printf( "Can not open SPCT6100 PCI device....\n" );
        exit(0);
    }

    if (-1 == ioctl(fd, VIDIOC_S_INPUT, &channelnum))
        errno_exit("VIDIOC_S_INPUT");

    new_brightness_value = atoi(argv[1]);
    validate_value(new_brightness_value);
    CLEAR(brightness_control);
    brightness_control.id = V4L2_CID_BRIGHTNESS;
    if (-1 == ioctl(fd, VIDIOC_G_CTRL, &brightness_control))
        errno_exit("VIDIOC_G_CTRL");
    else
        printf("The current brightness of channel 0 is: %d\n", brightness_control.value);


    brightness_control.id = V4L2_CID_BRIGHTNESS;
    brightness_control.value = new_brightness_value;
    if (-1 == ioctl(fd, VIDIOC_S_CTRL, &brightness_control))
        errno_exit("VIDIOC_S_CTRL");
    else
        printf("Changing brightness of channel 0 success!\n");

    brightness_control.id = V4L2_CID_BRIGHTNESS;
    if (-1 == ioctl(fd, VIDIOC_G_CTRL, &brightness_control))
        errno_exit("VIDIOC_G_CTRL");
    else
        printf("Now, the current brightness of channel 0 is: %d\n",
                brightness_control.value);

    return 0;
}
