#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/mman.h>
#include <errno.h>
#include <fcntl.h>
#include "spct6100-v4l2-types.h"

#define CLEAR(x) memset (&(x), 0, sizeof (x))

int fd;
int got_sps;
int totalframes;
struct buffer
{
    void *start;
    size_t length;
};
struct buffer *buffers = NULL;
FILE *output = NULL;
void save_one_frame_H264(struct v4l2_buffer buf);
void process_data (void);

    static void
errno_exit (const char *s)
{
    fprintf (stderr, "%s error %d, %s\n", s, errno, strerror (errno));
    exit (EXIT_FAILURE);
}

    int
main (int argc, char **argv)
{
    int index;
    int n = 0;
    unsigned int i;
    struct pollfd pfd;
    enum v4l2_buf_type type;
    struct v4l2_control change_mode;
    struct v4l2_requestbuffers reqbuf;
    struct v4l2_control change_time;
    struct tm T;
    time_t t;

    fd = open ("/dev/video1", O_RDWR | O_SYNC);
    if (fd < 0) {

        printf ("Open device failed\n");
    }

    for (index = 0; index < 4; index++) {
        if (-1 == ioctl (fd, VIDIOC_S_INPUT, &index)) {

            perror ("VIDIOC_S_INPUT");
            exit (EXIT_FAILURE);
        }

        change_mode.id = V4L2_CID_MODE;
        change_mode.value = 1;    // 0: NTSC, 1: PAL
        if (-1 == ioctl (fd, VIDIOC_S_CTRL, &change_mode)) {

            perror ("VIDIOC_S_CTRL");
            exit (EXIT_FAILURE);
        }
    }

#if 0
    /* Set time: 2011-1-8 16:23:56 */
    T.tm_year = 2011 - 1900;
    T.tm_mon = 1 - 1;
    T.tm_mday = 8;
    T.tm_hour = 16;
    T.tm_min = 23;
    T.tm_sec = 56;

    t = mktime (&T);
#else
    time (&t);

    /* UTC --> CST(+8 hours) */
    t += 8*3600;
#endif

    /* Note:
     * "TIMESET" ONLY set "today" and "tomorrow".
     *  So, it need be called every day.
     *  Otherwise, at "the day after tomorrow", OSD still shows "tomorrow".
     */
    change_time.id = V4L2_CID_TIMESET;
    change_time.value = t;
    if (-1 == ioctl (fd, VIDIOC_S_CTRL, &change_time)) {
        perror ("VIDIOC_S_CTRL");
        exit (EXIT_FAILURE);
    }

    /*** Step 1 Start ***/
    memset (&reqbuf, 0, sizeof (reqbuf));
    reqbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    reqbuf.memory = V4L2_MEMORY_MMAP;
    reqbuf.count = 12;
    if (-1 == ioctl (fd, VIDIOC_REQBUFS, &reqbuf)) {

        if (errno == EINVAL)
            printf ("Video capturing or mmap-streaming is not supported\n");
        else
            perror ("VIDIOC_REQBUFS");
        exit (EXIT_FAILURE);
    }
    /*** Step 1 End ***/

    /*** Step 2 Start ***/
    buffers = calloc (reqbuf.count, sizeof (*buffers));
    for (i = 0; i < reqbuf.count; i++) {

        struct v4l2_buffer buffer;
        memset (&buffer, 0, sizeof (buffer));
        buffer.type = reqbuf.type;
        buffer.memory = V4L2_MEMORY_MMAP;
        buffer.index = i;
        if (-1 == ioctl (fd, VIDIOC_QUERYBUF, &buffer)) {

            perror ("VIDIOC_QUERYBUF");
            exit (EXIT_FAILURE);
        }
        buffers[i].length = buffer.length;
        buffers[i].start = mmap (NULL,
                buffer.length,
                PROT_READ | PROT_WRITE, /* recommended */
                MAP_SHARED, /* recommended */
                fd,
                buffer.m.offset);
        if (MAP_FAILED == buffers[i].start) {

            perror ("mmap");
            exit (EXIT_FAILURE);
        }
    }

    // Initialize all buffers ***/
    for (index = 0; index < reqbuf.count; index++) {

        struct v4l2_buffer buf;
        CLEAR (buf);
        buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        buf.memory = V4L2_MEMORY_MMAP;
        buf.index = index;
        if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
            errno_exit ("VIDIOC_QBUF");
    }
    /*** Step 2 End ***/

    /*** Step 3 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMON, &type))
        errno_exit ("VIDIOC_STREAMON");
    /*** Step 3 End ***/

    output = fopen ("data.264", "wb");
    /*** Step 4 Start ***/
    pfd.fd = fd;
    pfd.events = POLLIN;
    while (1) {
        n = poll (&pfd, 1, -1);
        if (n < 0) {
            break;
        } else {

            process_data ();
            if (totalframes >= 300)
                break;
        }
    }
    /*** Step 4 Stop ***/

    /*** Step 5 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMOFF, &type))
        errno_exit ("VIDIOC_STREAMOFF");
    /*** Step 5 Stop ***/

    fclose (output);
    close (fd);
    return 0;
}                // End of main

    void
process_data (void)
{
    struct v4l2_buffer buf;
    CLEAR (buf);
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;
    if (-1 == ioctl (fd, VIDIOC_DQBUF, &buf))
        errno_exit ("VIDIOC_DQBUF");
    save_one_frame_H264(buf);
    if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
        errno_exit ("VIDIOC_QBUF");
}


void save_one_frame_H264(struct v4l2_buffer buf)
{
    int buf_index = buf.index;
    int buf_len = buf.length;
    int index, subpacket_num, channel_index, subpacket_index;
    unsigned char *cur_addr = NULL;
    for (channel_index = 0; channel_index < 4; channel_index++) {
        index = buf_index & 0xff;
        subpacket_num = buf_len & 0xff;
        buf_index >>= 8;
        buf_len >>= 8;

        if (index == 0xff) {
            continue;
        }

        /* We only process data from channel 0 */
        if (channel_index == 0) {
            cur_addr = buffers[index].start;
            for (subpacket_index = 0; subpacket_index < subpacket_num; subpacket_index++) {
                if (PACKET_WITH_VIDEO(cur_addr)) {
                    if (PACKET_IS_ISPSFRAME(cur_addr)) {
                        got_sps = 1;
                    }
                    if (AV_VSPSLEN(cur_addr)) {
                        printf("spslen = 0x%x\n", AV_VSPSLEN(cur_addr));
                    }

                    if (got_sps) {
                        printf("Save video frame %d\n", totalframes);
                        fwrite(cur_addr + AV_VOFFSET(cur_addr), AV_VLEN(cur_addr), 1, output);
                        totalframes++;
                    }
                }
                cur_addr+=AV_TOTALSIZE(cur_addr);
            }
        }
    }
}
