#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/mman.h>
#include <errno.h>
#include <fcntl.h>
#include "spct6100-v4l2-types.h"

#define CLEAR(x) memset (&(x), 0, sizeof (x))

int fd;
int got_sps;
int totalframes;
struct buffer
{
    void *start;
    size_t length;
};
struct buffer *buffers = NULL;
FILE *output = NULL;
void save_one_frame_yuv(struct v4l2_buffer buf);
void process_data (void);

    static void
errno_exit (const char *s)
{
    fprintf (stderr, "%s error %d, %s\n", s, errno, strerror (errno));
    exit (EXIT_FAILURE);
}

    int
main (int argc, char **argv)
{
    int index;
    int n = 0;
    unsigned int i;
    struct pollfd pfd;
    enum v4l2_buf_type type;
    struct v4l2_control change_mode;
    struct v4l2_requestbuffers reqbuf;
    v4l2_std_id std_id;

    fd = open ("/dev/video0", O_RDWR | O_SYNC);
    if (fd < 0) {

        printf ("Open device failed\n");
    }

    for (index = 0; index < 4; index++) {
        if (-1 == ioctl (fd, VIDIOC_S_INPUT, &index)) {

            perror ("VIDIOC_S_INPUT");
            exit (EXIT_FAILURE);
        }

        change_mode.id = V4L2_CID_MODE;
        change_mode.value = 1;    // 0: NTSC, 1: PAL
        if (-1 == ioctl (fd, VIDIOC_S_CTRL, &change_mode)) {

            perror ("VIDIOC_S_CTRL");
            exit (EXIT_FAILURE);
        }
    }

    /*** Step 1 Start ***/
    memset (&reqbuf, 0, sizeof (reqbuf));
    reqbuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    reqbuf.memory = V4L2_MEMORY_MMAP;
    reqbuf.count = 12;
    if (-1 == ioctl (fd, VIDIOC_REQBUFS, &reqbuf)) {

        if (errno == EINVAL)
            printf ("Video capturing or mmap-streaming is not supported\n");
        else
            perror ("VIDIOC_REQBUFS");
        exit (EXIT_FAILURE);
    }
    /*** Step 1 End ***/

    /*** Step 2 Start ***/
    buffers = calloc (reqbuf.count, sizeof (*buffers));
    for (i = 0; i < reqbuf.count; i++) {

        struct v4l2_buffer buffer;
        memset (&buffer, 0, sizeof (buffer));
        buffer.type = reqbuf.type;
        buffer.memory = V4L2_MEMORY_MMAP;
        buffer.index = i;
        if (-1 == ioctl (fd, VIDIOC_QUERYBUF, &buffer)) {

            perror ("VIDIOC_QUERYBUF");
            exit (EXIT_FAILURE);
        }
        buffers[i].length = buffer.length;
        buffers[i].start = mmap (NULL,
                buffer.length,
                PROT_READ | PROT_WRITE, /* recommended */
                MAP_SHARED, /* recommended */
                fd,
                buffer.m.offset);
        if (MAP_FAILED == buffers[i].start) {

            perror ("mmap");
            exit (EXIT_FAILURE);
        }
    }

    // Initialize all buffers ***/
    for (index = 0; index < reqbuf.count; index++) {

        struct v4l2_buffer buf;
        CLEAR (buf);
        buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
        buf.memory = V4L2_MEMORY_MMAP;
        buf.index = index;
        if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
            errno_exit ("VIDIOC_QBUF");
    }
    /*** Step 2 End ***/

    std_id = V4L2_STD_PAL_M;
    ioctl(fd, VIDIOC_S_STD, &std_id);

    /*** Step 3 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMON, &type))
        errno_exit ("VIDIOC_STREAMON");
    /*** Step 3 End ***/

    output = fopen ("data.yuv", "wb");
    /*** Step 4 Start ***/
    pfd.fd = fd;
    pfd.events = POLLIN;
    while (1) {
        n = poll (&pfd, 1, -1);
        if (n < 0) {
            break;
        } else {

            process_data ();
            if (totalframes >= 100)
                break;
        }
    }
    /*** Step 4 Stop ***/

    /*** Step 5 Start ***/
    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    if (-1 == ioctl (fd, VIDIOC_STREAMOFF, &type))
        errno_exit ("VIDIOC_STREAMOFF");
    /*** Step 5 Stop ***/

    fclose (output);
    close (fd);
    return 0;
}

    void
process_data (void)
{
    struct v4l2_buffer buf;
    CLEAR (buf);
    buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
    buf.memory = V4L2_MEMORY_MMAP;
    if (-1 == ioctl (fd, VIDIOC_DQBUF, &buf))
        errno_exit ("VIDIOC_DQBUF");
    save_one_frame_yuv(buf);
    if (-1 == ioctl (fd, VIDIOC_QBUF, &buf))
        errno_exit ("VIDIOC_QBUF");
}

void save_one_frame_yuv(struct v4l2_buffer buf)
{
    int buf_index, ch;
    for (ch = 0; ch < 4; ch++) {
        buf_index = (buf.index >> (ch*8)) & 0xFF;
        if (ch != 0 || buf_index == 0xFF) {
            continue;
        }
        fwrite(buffers[buf_index].start, buf.length, 1, output);
        totalframes++;
    }
}
