#ifndef DEV_H
#define DEV_H

#include <sys/time.h>
#include <time.h>
#include <pthread.h>
#include "lib.h"

enum DeviceType {
    UnknownDev = 0,
    LiveDev = 1,
    CompressDev = 2
};

struct buffer {
  void *start;
  size_t length;
};

struct DeviceInfo {
    char dev_name[32];
    enum DeviceType dev_type;
    int dev_id;
    int fd;
    struct buffer *buffers;
    int n_buffers;
    pthread_t thread_id;
    int thread_run;
    struct timeval start_time;
    struct timeval end_time;
    unsigned int frames[4];
    unsigned long long bytes[4];
    unsigned int video_save;
    unsigned int video_save_count[4];
    unsigned int audio_save;
    unsigned int audio_save_count[4];

    int has_buddy;
    int buddy_index;
    int motion_area[4][18][22];
};

extern struct DeviceInfo g_LiveDev[4];
extern int g_nLiveCount;

extern struct DeviceInfo g_CompressDev[4];
extern int g_nCompressCount;

void open_all_devices (int argc, char *argv[]);
void close_all_devices (void);
void start_live_thread (int id);
void start_compress_thread (int id);
void start_all_live_threads (void);
void start_all_compress_threads (void);
void stop_live_thread (int id);
void stop_compress_thread (int id);
void stop_all_live_threads (void);
void stop_all_compress_threads (void);
void set_live_save (int id, int val);
void set_all_live_save (int val);
void set_compress_video_save (int id, int val);
void set_all_compress_video_save (int val);
void set_compress_audio_save (int id, int val);
void set_all_compress_audio_save (int val);
void resize_window (int width, int height);

#endif /* DEV_H */
