#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <SDL/SDL.h>
#include "test.h"

extern void resize_window (int width, int height);

int done = 0;

static void signal_handler (int sig)
{
    done = 1;
}

int main (int argc, char *argv[])
{
    int i, rc;

    signal (SIGHUP, signal_handler);
    signal (SIGINT, signal_handler);
    signal (SIGTERM, signal_handler);

    rc = test_begin (argc, argv);
    if (rc != 0) {
        fprintf (stderr, "test_begin failed: rc = %d\n", rc);
        goto error_out;
    }
    
    for (i = 0; i < test_array_size; i++) {
        rc = test_array[i] ();
        if (rc != 0) {
            fprintf (stderr, "test_arrary[%d] failed: rc = %d\n", i, rc);
            done = 1;
            break;
        }
    }

    printf ("Press Ctrl+C to exit......\n");
    while (!done) {
        SDL_Event event;

        while (SDL_PollEvent (&event)) {
            switch (event.type) {
            case SDL_QUIT:
                done = 1;
                break;
            case SDL_VIDEORESIZE:
                resize_window (event.resize.w, event.resize.h);
                break;
            }
            if (done) {
                break;
            }
        }
    }

error_out:
    test_end ();
    return rc;
}
