#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "test.h"
#include "dev.h"

extern int done;

static int show_performance (void);

static int show_device_info (void)
{
    int i;
    struct DeviceInfo *pInfo;

    for (i = 0; i < g_nLiveCount; i++) {
        pInfo = &g_LiveDev[i];
        fprintf (stderr, "%s: Live Device\n", pInfo->dev_name);
        fprintf (stderr, "\tdev_id = %d, ", pInfo->dev_id);
        fprintf (stderr, "n_buffers = %d, ", pInfo->n_buffers);
        fprintf (stderr, "video_save = %u, ", pInfo->video_save);
        fprintf (stderr, "audio_save = %u\n", pInfo->audio_save);
        if (pInfo->has_buddy) {
            fprintf (stderr, "\tHas buddy: %s\n",
                     g_CompressDev[pInfo->buddy_index].dev_name);
        }
    }
    for (i = 0; i < g_nCompressCount; i++) {
        pInfo = &g_CompressDev[i];
        fprintf (stderr, "%s: Compress Device\n", pInfo->dev_name);
        fprintf (stderr, "\tdev_id = %d, ", pInfo->dev_id);
        fprintf (stderr, "n_buffers = %d, ", pInfo->n_buffers);
        fprintf (stderr, "video_save = %u, ", pInfo->video_save);
        fprintf (stderr, "audio_save = %u\n", pInfo->audio_save);
        if (pInfo->has_buddy) {
            fprintf (stderr, "\tHas buddy: %s\n",
                     g_LiveDev[pInfo->buddy_index].dev_name);
        }
    }
    return 0;
}

static int set_device_propery (void)
{
    int i, chan;
    struct DeviceInfo *pInfo;

    for (i = 0; i < g_nCompressCount; i++) {
        pInfo = &g_CompressDev[i];
        for (chan = 0; chan < 4; chan++) {
            s_input (pInfo->fd, chan);
            s_std (pInfo->fd, "PAL");
            s_ctrl_AUDIO (pInfo->fd, 1);
            s_ctrl_MOTIONDETECT (pInfo->fd, 1);
            s_ctrl_VLOSTEN (pInfo->fd, 1);
            s_fmt (pInfo->fd, 352, 288, "YUV420");
        }
    }
    return 0;
}

static int start_all_threads (void)
{
    int value;

    set_all_live_save (0);
    set_all_compress_video_save (100000);
    set_all_compress_audio_save (0);
    show_device_info ();
    start_all_live_threads ();
    start_all_compress_threads ();

    for (value = 0; value < 256; value += 10) {
#if 0
        for (i = 0; i < g_nLiveCount; i++) {
            pInfo = &g_LiveDev[i];
            for (chan = 0; chan < 4; chan++) {
                s_input (pInfo->fd, chan);
                s_ctrl_BRIGHTNESS (pInfo->fd, value);
            }
        }
        for (i = 0; i < g_nCompressCount; i++) {
            pInfo = &g_CompressDev[i];
            for (chan = 0; chan < 4; chan++) {
                s_input (pInfo->fd, chan);
                s_ctrl_BRIGHTNESS (pInfo->fd, value);
            }
        }
        sleep (1);
#endif
    }
    return 0;
}

static int test_osd (void)
{
    int iteration;

    fprintf (stderr, "\nEnter function: %s\n", __FUNCTION__);

    for (iteration = 0; iteration < 6; iteration++) {
        int i, chan;
        struct DeviceInfo *pInfo;
        struct spct6100_osd osd;

        if (done) {
            break;
        }

        sleep (1);
        fprintf (stderr, "%s iteration %d:\n", __FUNCTION__, iteration);

        for (i = 0; i < g_nCompressCount; i++) {
            pInfo = &g_CompressDev[i];
            for (chan = 0; chan < 1; chan++) {
                s_input (pInfo->fd, chan);

                osd.tran = 1;
                osd.acolor = 1;
                osd.vscale = 0;
                osd.hscale = 0;
                osd.timer_enable = 1;
                osd.osd_enable = 1;
                osd.osd_pos_x = 100;
                osd.osd_pos_y = 120;
                if ((iteration % 3) == 0) {
                    sprintf (osd.str, "0123456789");
                } else if ((iteration % 3) == 1) {
                    sprintf (osd.str, "56789xb");
                } else if ((iteration % 3) == 2) {
                    sprintf (osd.str, "iaushdfkhsb");
                }

                s_ctrl_OSD (pInfo->fd, &osd);
                sleep (1);
            }
        }
    }
    fprintf (stderr, "Leave function: %s\n\n", __FUNCTION__);
    return 0;
}

//====================================
TEST_FUNC test_array[] =
{
    set_device_propery,
    start_all_threads,
    test_osd,
};
int test_array_size = 1000;

int test_begin (int argc, char *argv[])
{
    int size;
    char *local_argv[10] = {
        NULL,
        "/dev/video0", "/dev/video1",
        "/dev/video2", "/dev/video3",
        "/dev/video4", "/dev/video5",
        "/dev/video6", "/dev/video7",
        NULL};

    size = sizeof (test_array) / sizeof (test_array[0]);
    if (size < test_array_size) {
        test_array_size = size;
    }

    putenv("SDL_VIDEO_YUV_HWACCEL=0");
    open_all_devices (9, local_argv);
    return 0;
}

static int show_performance (void)
{
    int i, chan;
    double duration, fps, bit_rate;
    struct DeviceInfo *pInfo;

    if (g_nLiveCount > 0) {
        printf ("\nLive device:\n");
    }
    for (i = 0; i < g_nLiveCount; i++) {
        pInfo = &(g_LiveDev[i]);
        duration = pInfo->end_time.tv_sec - pInfo->start_time.tv_sec
                   + (pInfo->end_time.tv_usec - pInfo->start_time.tv_usec)
                     / 1000000.;
        printf ("%s, duration = %fs\n", pInfo->dev_name, duration);
        for (chan = 0; chan < 4; chan++) {
            fps = pInfo->frames[chan] / duration;
            bit_rate = (8. * pInfo->bytes[chan]) / (duration * 1000.);
            printf ("channel %d:\n", chan);
            printf ("\tframes = %d, fps = %f\n", pInfo->frames[chan], fps);
            printf ("\tbytes = %lld, bit_rate = %fKb/s\n",
                    pInfo->bytes[chan], bit_rate);
        }
    }

    if (g_nCompressCount > 0) {
        printf ("\nCompress device:\n");
    }
    for (i = 0; i < g_nCompressCount; i++) {
        pInfo = &(g_CompressDev[i]);
        duration = pInfo->end_time.tv_sec - pInfo->start_time.tv_sec
                   + (pInfo->end_time.tv_usec - pInfo->start_time.tv_usec)
                     / 1000000.;
        printf ("%s, duration = %fs\n", pInfo->dev_name, duration);
        for (chan = 0; chan < 4; chan++) {
            fps = pInfo->frames[chan] / duration;
            bit_rate = (8. * pInfo->bytes[chan]) / (duration * 1000.);
            printf ("channel %d:\n", chan);
            printf ("\tframes = %d, fps = %f\n", pInfo->frames[chan], fps);
            printf ("\tbytes = %lld, bit_rate = %fKb/s\n",
                    pInfo->bytes[chan], bit_rate);
        }
    }
    return 0;
}

int test_end ()
{
    stop_all_live_threads ();
    stop_all_compress_threads ();
    show_performance ();
    close_all_devices ();
    return 0;
}
