#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <audio_enable>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  audio_enable               0(disable), 1(enable)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int audio, new_audio;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_audio);

    if (new_audio < 0 || new_audio > 1) {
        printf ("audio_enable(%d) is out of range [0, 1]\n", new_audio);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_AUDIO (fd, &audio) != 0) {
            printf ("g_ctrl_AUDIO failed\n");
            break;
        }

        printf ("current audio: %d(%s)\n", audio,
                audio ? "enable" : "disable");

        rc = s_ctrl_AUDIO (fd, new_audio);
        printf ("reset audio to %d(%s) %s\n", new_audio,
                new_audio ? "enable" : "disable",
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
