#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <contrast>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  contrast                   0~255\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int contrast, new_contrast;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_contrast);

    if (new_contrast < 0 || new_contrast > 255) {
        printf ("contrast(%d) is out of range [0, 255]\n", new_contrast);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_CONTRAST (fd, &contrast) != 0) {
            printf ("g_ctrl_CONTRAST failed\n");
            break;
        }

        printf ("current contrast: %d\n", contrast);

        rc = s_ctrl_CONTRAST (fd, new_contrast);
        printf ("reset contrast to %d %s\n", new_contrast,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
