#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <start_x> <start_y>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  start_x                    0~255\n");
    printf ("  start_y                    0~255\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int mode;
    struct spct6100_cropset cropset;
    int start_x, start_y;

    if (argc != 4) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &start_x);
    sscanf (argv[3], "%d", &start_y);

    if (start_x < 0 || start_x > 255) {
        printf ("start_x(%d) is out of range [0, 255]\n", start_x);
        show_usage (argc, argv);
        return -1;
    }
    if (start_y < 0 || start_y > 255) {
        printf ("start_y(%d) is out of range [0, 255]\n", start_y);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MODE (fd, &mode) != 0) {
            printf ("g_ctrl_MODE failed\n");
            break;
        }

        switch (mode) {
        case MODE_NTSC:
            cropset.width = 720;
            cropset.height = 240;
            break;

        case MODE_PAL:
            cropset.width = 720;
            cropset.height = 288;
            break;

        default:
            printf ("unknown mode: %d\n", mode);
            goto err_out;
        }
        cropset.start_x = start_x;
        cropset.start_y = start_y;

        rc = s_ctrl_CROPSET (fd, &cropset);
        printf ("reset cropset %s\n",
                rc ? "failed" : "succeeded");
    }

err_out:
    close (fd);
    return 0;
}
