#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <compress show size>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  compress show size         0(4CIF), 1(CIF)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int cshowsize, new_cshowsize;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_cshowsize);

    if (new_cshowsize < 0 || new_cshowsize > 1) {
        printf ("cshowsize(%d) is out of range [0, 1]\n", new_cshowsize);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_CSHOWSIZE (fd, &cshowsize) != 0) {
            printf ("g_ctrl_LSHOWSIZE failed\n");
            break;
        }

        switch (cshowsize) {
        case R_4CIF:
            printf ("current compress show size: %d(4CIF)\n", cshowsize);
            break;

        case R_CIF:
            printf ("current compress show size: %d(CIF)\n", cshowsize);
            break;

        default:
            printf ("unknown compress show size %d\n", cshowsize);
            goto err_out;
        }

        rc = s_ctrl_CSHOWSIZE (fd, new_cshowsize);
        switch (new_cshowsize) {
        case R_4CIF:
            printf ("reset compress show size to %d(4CIF) %s\n", new_cshowsize,
                    rc ? "failed" : "succeeded");
            break;

        case R_CIF:
            printf ("reset compress show size to %d(CIF) %s\n", new_cshowsize,
                    rc ? "failed" : "succeeded");
            break;

        default:
            printf ("unknown compress show size %d\n", new_cshowsize);
            goto err_out;
        }
    }

err_out:
    close (fd);
    return 0;
}
