#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <encode speed>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  encode speed               1~30\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int enspeed, new_enspeed;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_enspeed);

    if (new_enspeed < 1 || new_enspeed > 30) {
        printf ("encode speed(%d) is out of range [1, 30]\n", new_enspeed);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_EENSPEED (fd, &enspeed) != 0) {
            printf ("g_ctrl_EENSPEED failed\n");
            break;
        }

        printf ("current enspeed(%d)\n", enspeed);

        rc = s_ctrl_EENSPEED (fd, new_enspeed);
        printf ("reset enspeed(%d) %s\n", new_enspeed,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
