#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    struct spct6100_equality equality, new_equality;

    if (argc != 2) {
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_EQUALITY (fd, &equality) != 0) {
            printf ("g_ctrl_EQUALITY failed\n");
            break;
        }

        printf ("current equality:\n");
        switch (equality.type) {
        case TYPE_CBR:
            printf ("  type: %d(CBR)\n", equality.type);
            printf ("  minbitrate: %d\n", equality.minbitrate);
            printf ("  maxbitrate: %d\n", equality.maxbitrate);
            break;

        case TYPE_VBR:
            printf ("  type: %d(VBR)\n", equality.type);
            printf ("  qpi: %d\n", equality.qpi);
            printf ("  qpp: %d\n", equality.qpp);
            printf ("  qpb: %d\n", equality.qpb);
            break;

        default:
            printf ("  type: %d(unknown), error!\n", equality.type);
            goto err_out;
        }

        new_equality.type = TYPE_CBR;
        new_equality.minbitrate = 64;
        new_equality.maxbitrate = 256;

        rc = s_ctrl_EQUALITY (fd, &new_equality);
        printf ("reset EQUALITY %s:\n",
                rc ? "failed" : "succeeded");
        switch (new_equality.type) {
        case TYPE_CBR:
            printf ("  type: %d(CBR)\n", new_equality.type);
            printf ("  minbitrate: %d\n", new_equality.minbitrate);
            printf ("  maxbitrate: %d\n", new_equality.maxbitrate);
            break;

        case TYPE_VBR:
            printf ("  type: %d(VBR)\n", new_equality.type);
            printf ("  qpi: %d\n", new_equality.qpi);
            printf ("  qpp: %d\n", new_equality.qpp);
            printf ("  qpb: %d\n", new_equality.qpb);
            break;
        }
    }

err_out:
    close (fd);
    return 0;
}
