#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <gopp> <gopb>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  gopp                       P_Frame(>= 0)\n");
    printf ("  gopb                       B_Frame(>= 0)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int new_gopp, new_gopb;
    struct spct6100_gop gop;

    if (argc != 4) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_gopp);
    sscanf (argv[3], "%d", &new_gopb);

    if (new_gopp < 0) {
        printf ("gopp(%d) should be positive\n", new_gopp);
        show_usage (argc, argv);
        return -1;
    }
    if (new_gopb < 0) {
        printf ("gopb(%d) should be positive\n", new_gopb);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_GOP (fd, &gop) != 0) {
            printf ("g_ctrl_GOP failed\n");
            break;
        }

        printf ("current <gopp, gopb>: <%d, %d>\n", gop.gopp, gop.gopb);

        gop.gopp = new_gopp;
        gop.gopb = new_gopb;

        rc = s_ctrl_GOP (fd, &gop);
        printf ("reset <gopp, gopb> to <%d, %d> %s\n", new_gopp, new_gopb,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
