#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <pin> <direction>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  pin                        %d~%d\n", GPIO_MINPIN, GPIO_MAXPIN);
    printf ("  direction                  0(OUT), 1(IN)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, rc;
    int pin, direction;
    struct spct6100_gpio gpio, new_gpio;
    int i;

    if (argc != 4) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &pin);
    sscanf (argv[3], "%d", &direction);

    if (pin < GPIO_MINPIN || pin > GPIO_MAXPIN) {
        printf ("pin(%d) is out of range [%d, %d]\n", pin, GPIO_MINPIN, GPIO_MAXPIN);
        show_usage (argc, argv);
        return -1;
    }
    if (direction < 0 || direction > 1) {
        printf ("direction(%d) is out of range [0, 1]\n", direction);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (i = GPIO_MINPIN; i <= GPIO_MAXPIN; i++) {
        gpio.pinid = i;
        if (g_ctrl_GPIOD (fd, &gpio) != 0) {
            printf ("g_ctrl_GPIOD failed\n");
            goto err_out;
        }
        printf ("pin(%d) current direction: %d\n", gpio.pinid, gpio.value);
    }

    new_gpio.pinid = pin;
    new_gpio.value = direction;

    rc = s_ctrl_GPIOD (fd, &new_gpio);
    printf ("reset pin(%d) GPIOD %d(%s) %s\n", pin, direction,
            (direction == GPIO_OUT) ? "OUT" : "IN",
            rc ? "failed" : "succeeded");

err_out:
    close (fd);
    return 0;
}
