#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <live show size>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  live show size             0(4CIF), 1(CIF)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int lshowsize, new_lshowsize;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_lshowsize);

    if (new_lshowsize < 0 || new_lshowsize > 1) {
        printf ("lshowsize(%d) is out of range [0, 1]\n", new_lshowsize);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_LSHOWSIZE (fd, &lshowsize) != 0) {
            printf ("g_ctrl_LSHOWSIZE failed\n");
            break;
        }

        switch (lshowsize) {
        case R_4CIF:
            printf ("current live show size: %d(4CIF)\n", lshowsize);
            break;

        case R_CIF:
            printf ("current live show size: %d(CIF)\n", lshowsize);
            break;

        default:
            printf ("unknown live show size %d\n", lshowsize);
            goto err_out;
        }

        rc = s_ctrl_LSHOWSIZE (fd, new_lshowsize);
        switch (new_lshowsize) {
        case R_4CIF:
            printf ("reset live show size to %d(4CIF) %s\n", new_lshowsize,
                    rc ? "failed" : "succeeded");
            break;

        case R_CIF:
            printf ("reset live show size to %d(CIF) %s\n", new_lshowsize,
                    rc ? "failed" : "succeeded");
            break;

        default:
            printf ("unknown live show size %d\n", new_lshowsize);
            goto err_out;
        }
    }

err_out:
    close (fd);
    return 0;
}
