#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <mode>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  mode                       0~1\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int mode, new_mode;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_mode);

    if (new_mode < 0 || new_mode > 1) {
        printf ("mode(%d) is out of range [0, 1]\n", new_mode);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MODE (fd, &mode) != 0) {
            printf ("g_ctrl_MODE failed\n");
            break;
        }

        printf ("current mode: %d(%s)\n", mode,
                (mode == MODE_NTSC) ? "MODE_NTSC" : "MODE_PAL");

        rc = s_ctrl_MODE (fd, new_mode);
        printf ("reset mode to %d(%s) %s\n", new_mode,
                (new_mode == MODE_NTSC) ? "MODE_NTSC" : "MODE_PAL",
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
