#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    struct spct6100_mosaic mosaic, new_mosaic;
    int i;

    if (argc != 2) {
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MOSAIC (fd, &mosaic) != 0) {
            printf ("g_ctrl_MOSAIC failed\n");
            break;
        }

        printf ("current mosaic num = %d:\n", mosaic.num);
        for (i = 0; i < mosaic.num; i++) {
            printf ("  <x%d, y%d, w%d, h%d> = <%d, %d, %d, %d>\n",
                    i, i, i, i, mosaic.x[i], mosaic.y[i], mosaic.w[i], mosaic.h[i]);
        }

        memset (&new_mosaic, 0, sizeof (new_mosaic));
        new_mosaic.num = channel + 2;
        for (i = 0; i < new_mosaic.num; i++) {
            new_mosaic.x[i] = 16 * i * 2;
            new_mosaic.y[i] = 20 * i * 2;
            new_mosaic.w[i] = 16 * 2;
            new_mosaic.h[i] = 20 * 2;
        }

        rc = s_ctrl_MOSAIC (fd, &new_mosaic);
        printf ("reset MOSAIC num = %d %s:\n", new_mosaic.num,
                rc ? "failed" : "succeeded");
        for (i = 0; i < new_mosaic.num; i++) {
            printf ("  <x%d, y%d, w%d, h%d> = <%d, %d, %d, %d>\n",
                    i, i, i, i,
                    new_mosaic.x[i], new_mosaic.y[i], new_mosaic.w[i], new_mosaic.h[i]);
        }
    }

    close (fd);
    return 0;
}
