#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <motion detect enable>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  motion detect enable       0~1\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int mdetect, new_mdetect;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_mdetect);

    if (new_mdetect < 0 || new_mdetect > 1) {
        printf ("motion detect enable(%d) is out of range [0, 1]\n", new_mdetect);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MOTIONDETECT (fd, &mdetect) != 0) {
            printf ("g_ctrl_MOTIONDETECT failed\n");
            break;
        }

        printf ("current motion detect enable: %d\n", mdetect);

        rc = s_ctrl_MOTIONDETECT (fd, new_mdetect);
        printf ("reset motion detect enable to %d %s\n", new_mdetect,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
