#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <motion interval>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  motion interval            1,2,4,8,16\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int minterval, new_minterval;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_minterval);

    if (new_minterval != 1
        && new_minterval != 2
        && new_minterval != 4
        && new_minterval != 8
        && new_minterval != 16) {
        printf ("motion interval(%d) is out of range 1|2|4|8|16\n", new_minterval);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MINTERVAL (fd, &minterval) != 0) {
            printf ("g_ctrl_MOTIONDETECT failed\n");
            break;
        }

        printf ("current motion interval: %d\n", minterval);

        rc = s_ctrl_MINTERVAL (fd, new_minterval);
        printf ("reset motion interval to %d %s\n", new_minterval,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
