#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <motion threshold>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  motion threshold           1~66585600\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int mthreshold, new_mthreshold;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_mthreshold);

    if (new_mthreshold < 1 || new_mthreshold > 66585600) {
        printf ("motion threshold(%d) is out of range [1, 66585600]\n", new_mthreshold);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_MTHRESHOLD (fd, &mthreshold) != 0) {
            printf ("g_ctrl_MOTIONDETECT failed\n");
            break;
        }

        printf ("current motion threshold: %d\n", mthreshold);

        rc = s_ctrl_MTHRESHOLD (fd, new_mthreshold);
        printf ("reset motion threshold to %d %s\n", new_mthreshold,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
