#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    struct spct6100_osd osd;

    if (argc != 2) {
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_OSD (fd, &osd) != 0) {
            printf ("g_ctrl_OSD failed\n");
            break;
        }

        printf ("current OSD:\n");
        printf ("  osd.tran         = %d\n", osd.tran);
        printf ("  osd.acolor       = %d\n", osd.acolor);
        printf ("  osd.vscale       = %d\n", osd.vscale);
        printf ("  osd.hscale       = %d\n", osd.hscale);
        printf ("  osd.timer_enable = %d\n", osd.timer_enable);
        printf ("  osd.osd_enable   = %d\n", osd.osd_enable);
        printf ("  osd.osd_pos_x    = %d\n", osd.osd_pos_x);
        printf ("  osd.osd_pos_y    = %d\n", osd.osd_pos_y);
        printf ("  osd.str          = %s\n", osd.str);

        memset (&osd, 0, sizeof (osd));
        osd.acolor = 15;
        osd.timer_enable = 1;
        osd.osd_enable = 1;
        osd.osd_pos_x = 100;
        osd.osd_pos_y = 120;
        strcpy (osd.str, "Hello, SPCT6100!");

        rc = s_ctrl_OSD (fd, &osd);
        printf ("reset OSD %s:\n",
                rc ? "failed" : "succeeded");
        printf ("  osd.tran         = %d\n", osd.tran);
        printf ("  osd.acolor       = %d\n", osd.acolor);
        printf ("  osd.vscale       = %d\n", osd.vscale);
        printf ("  osd.hscale       = %d\n", osd.hscale);
        printf ("  osd.timer_enable = %d\n", osd.timer_enable);
        printf ("  osd.osd_enable   = %d\n", osd.osd_enable);
        printf ("  osd.osd_pos_x    = %d\n", osd.osd_pos_x);
        printf ("  osd.osd_pos_y    = %d\n", osd.osd_pos_y);
        printf ("  osd.str          = %s\n", osd.str);
    }

    close (fd);
    return 0;
}
