#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <preview_path>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  preview_path               0(DDR), 1(PCI)\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int preview, new_preview;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_preview);

    if (new_preview < 0 || new_preview > 1) {
        printf ("preview_path(%d) is out of range [0, 1]\n", new_preview);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_PREVIEW (fd, &preview) != 0) {
            printf ("g_ctrl_PREVIEW failed\n");
            break;
        }

        printf ("current preview path: %d(%s)\n", preview,
                (preview == PREVIEW_PATH_DDR) ? "DDR" : "PCI");

        rc = s_ctrl_PREVIEW (fd, new_preview);
        printf ("reset preview path to %d(%s) %s\n", new_preview,
                (new_preview == PREVIEW_PATH_DDR) ? "DDR" : "PCI",
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
