#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <channel>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  channel                    0~3\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &channel);

    if (channel < 0 || channel > 3) {
        printf ("channel(%d) is out of range [0, 3]\n", channel);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    printf ("\nchannel %d:\n", channel);

    rc = s_ctrl_REQIFRAME (fd, channel);
    printf ("request I-Frame %s\n",
            rc ? "failed" : "succeeded");

    close (fd);
    return 0;
}
