#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <pin>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  pin                        %d~%d\n", GPIO_MINPIN, GPIO_MAXPIN);
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, rc;
    int pin;
    struct spct6100_gpio gpio;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &pin);

    if (pin < GPIO_MINPIN || pin > GPIO_MAXPIN) {
        printf ("pin(%d) is out of range [%d, %d]\n", pin, GPIO_MINPIN, GPIO_MAXPIN);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    gpio.pinid = pin;
    gpio.value = GPIO_IN;

    rc = s_ctrl_GPIOD (fd, &gpio);
    printf ("set pin(%d) GPIOD %d(%s) %s\n", pin, GPIO_IN, "IN",
            rc ? "failed" : "succeeded");

    gpio.pinid = pin;

    rc = s_ctrl_RGPIO (fd, &gpio);
    printf ("read from pin(%d) %s: value = %d\n", pin,
            rc ? "failed" : "succeeded", gpio.value);

    close (fd);
    return 0;
}
