#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <saturation>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  saturation                 0~255\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int saturation, new_saturation;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_saturation);

    if (new_saturation < 0 || new_saturation > 255) {
        printf ("saturation(%d) is out of range [0, 255]\n", new_saturation);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_SATURATION (fd, &saturation) != 0) {
            printf ("g_ctrl_SATURATION failed\n");
            break;
        }

        printf ("current saturation: %d\n", saturation);

        rc = s_ctrl_SATURATION (fd, new_saturation);
        printf ("reset saturation to %d %s\n", new_saturation,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
