#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <live off/on> <compress off/on>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  live off/on                0/1\n");
    printf ("  compress off/on            0/1\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int status, new_status;
    int live, compress;

    if (argc != 4) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &live);
    sscanf (argv[3], "%d", &compress);

    if (live < 0 || live > 1) {
        printf ("live off/on(%d) is out of range [0, 1]\n", live);
        show_usage (argc, argv);
        return -1;
    }
    if (compress < 0 || compress > 1) {
        printf ("compress off/on(%d) is out of range [0, 1]\n", compress);
        show_usage (argc, argv);
        return -1;
    }
    new_status = 0;
    if (live) {
        new_status |= STATUS_LIVE;
    }
    if (compress) {
        new_status |= STATUS_COMPRESS;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_STATUS (fd, &status) != 0) {
            printf ("g_ctrl_STATUS failed\n");
            break;
        }

        printf ("current status live(%s), compress(%s)\n",
                (status & STATUS_LIVE) ? "on" : "off",
                (status & STATUS_COMPRESS) ? "on" : "off");

        rc = s_ctrl_STATUS (fd, new_status);
        printf ("reset status live(%s), compress(%s) %s\n",
                (new_status & STATUS_LIVE) ? "on" : "off",
                (new_status & STATUS_COMPRESS) ? "on" : "off",
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
