#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <timeout>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  timeout                    integer >= 1\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int timeout, new_timeout;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_timeout);

    if (new_timeout <= 0) {
        printf ("timeout(%d) should be positive\n", new_timeout);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_TIMEOUT (fd, &timeout) != 0) {
            printf ("g_ctrl_TIMEOUT failed\n");
            break;
        }

        printf ("current timeout: %d(ms)\n", timeout);

        rc = s_ctrl_TIMEOUT (fd, new_timeout);
        printf ("reset timeout to %d(ms) %s\n", new_timeout,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
