#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <video lost enable>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  video lost enable          0~1\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, channel, rc;
    int vlosten, new_vlosten;

    if (argc != 3) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &new_vlosten);

    if (new_vlosten < 0 || new_vlosten > 1) {
        printf ("vlosten(%d) is out of range [0, 1]\n", new_vlosten);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    for (channel = 0; channel < 4; channel++) {

        printf ("\nchannel %d:\n", channel);
        if (s_input (fd, channel) != 0) {
            printf ("s_input failed\n");
            break;
        }

        if (g_ctrl_VLOSTEN (fd, &vlosten) != 0) {
            printf ("g_ctrl_VLOSTEN failed\n");
            break;
        }

        printf ("current video lost enable: %d\n", vlosten);

        rc = s_ctrl_VLOSTEN (fd, new_vlosten);
        printf ("reset vlosten to %d %s\n", new_vlosten,
                rc ? "failed" : "succeeded");
    }

    close (fd);
    return 0;
}
