#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "lib.h"

static void show_usage (int argc, char *argv[])
{
    printf ("Usage: %s <device_name> <pin> <value>\n", argv[0]);
    printf ("  device_name                /dev/video[0~7]\n");
    printf ("  pin                        %d~%d\n", GPIO_MINPIN, GPIO_MAXPIN);
    printf ("  value                      0 or 1, data to write out\n");
}

int main (int argc, char *argv[])
{
    const char *device_name;
    int fd, rc;
    int pin, value;
    struct spct6100_gpio gpio;

    if (argc != 4) {
        show_usage (argc, argv);
        return -1;
    }

    sscanf (argv[2], "%d", &pin);
    sscanf (argv[3], "%d", &value);

    if (pin < GPIO_MINPIN || pin > GPIO_MAXPIN) {
        printf ("pin(%d) is out of range [%d, %d]\n", pin, GPIO_MINPIN, GPIO_MAXPIN);
        show_usage (argc, argv);
        return -1;
    }
    if (value < 0 || value > 1) {
        printf ("value(%d) is out of range [0, 1]\n", value);
        show_usage (argc, argv);
        return -1;
    }

    device_name = argv[1];
    fd = open (device_name, O_RDWR | O_SYNC);
    if (fd < 0) {
        printf ("Cannot open device %s\n", device_name);
        show_usage (argc, argv);
        return -1;
    }

    gpio.pinid = pin;
    gpio.value = GPIO_OUT;

    rc = s_ctrl_GPIOD (fd, &gpio);
    printf ("set pin(%d) GPIOD %d(%s) %s\n", pin, GPIO_OUT, "OUT",
            rc ? "failed" : "succeeded");

    gpio.pinid = pin;
    gpio.value = value;
    rc = s_ctrl_WGPIO (fd, &gpio);
    printf ("write %d to pin(%d) %s\n", value, pin,
            rc ? "failed" : "succeeded");

    close (fd);
    return 0;
}
