#ifndef SPCT6100_V4L2_TYPES_H
#define SPCT6100_V4L2_TYPES_H

#include <stdint.h>

#ifdef __arm__
typedef __signed__ char __s8;
typedef unsigned char __u8;

typedef __signed__ short __s16;
typedef unsigned short __u16;

typedef __signed__ int __s32;
typedef unsigned int __u32;

#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
typedef __signed__ long long __s64;
typedef unsigned long long __u64;
#endif
#endif

/* V4L2_CID_BRIGHTNESS, V4L2_CID_CONTRAST, V4L2_CID_SATURATION, V4L2_CID_HUE
   are defined in <linux/videodev2.h>
*/    
#include <linux/videodev2.h>

#define V4L2_CID_REQIFRAME            (V4L2_CID_PRIVATE_BASE + 0)
#define V4L2_CID_INITENV              (V4L2_CID_PRIVATE_BASE + 1)
#define V4L2_CID_MODE                 (V4L2_CID_PRIVATE_BASE + 6)
#define V4L2_CID_STATUS               (V4L2_CID_PRIVATE_BASE + 7)
#define V4L2_CID_EQUALITY             (V4L2_CID_PRIVATE_BASE + 9)
#define V4L2_CID_LSHOWSIZE            (V4L2_CID_PRIVATE_BASE + 10)
#define V4L2_CID_EENSPEED             (V4L2_CID_PRIVATE_BASE + 11)
#define V4L2_CID_MOSAIC               (V4L2_CID_PRIVATE_BASE + 15)
#define V4L2_CID_MOTIONDETECT         (V4L2_CID_PRIVATE_BASE + 16)
#define V4L2_CID_DOWNLOAD             (V4L2_CID_PRIVATE_BASE + 18)
#define V4L2_CID_CARDNUM              (V4L2_CID_PRIVATE_BASE + 19)
#define V4L2_CID_DSPSTATUS            (V4L2_CID_PRIVATE_BASE + 20)
#define V4L2_CID_UPLOAD               (V4L2_CID_PRIVATE_BASE + 22)
#define V4L2_CID_RUNCODE              (V4L2_CID_PRIVATE_BASE + 24)
#define V4L2_CID_TIMEOUT              (V4L2_CID_PRIVATE_BASE + 26)
#define V4L2_CID_MINTERVAL            (V4L2_CID_PRIVATE_BASE + 27)
#define V4L2_CID_MTHRESHOLD           (V4L2_CID_PRIVATE_BASE + 28)
#define V4L2_CID_GPIOD                (V4L2_CID_PRIVATE_BASE + 33)
#define V4L2_CID_WGPIO                (V4L2_CID_PRIVATE_BASE + 34)
#define V4L2_CID_VLOSTEN              (V4L2_CID_PRIVATE_BASE + 35)
#define V4L2_CID_OSDSET               (V4L2_CID_PRIVATE_BASE + 36)
#define V4L2_CID_GOPSET               (V4L2_CID_PRIVATE_BASE + 37)
#define V4L2_CID_REQISEQ              (V4L2_CID_PRIVATE_BASE + 38)
#define V4L2_CID_RGPIO                (V4L2_CID_PRIVATE_BASE + 40)
#define V4L2_CID_CSHOWSIZE            (V4L2_CID_PRIVATE_BASE + 43)
#define V4L2_CID_PREVIEW              (V4L2_CID_PRIVATE_BASE + 44)
#define V4L2_CID_AUDIO                (V4L2_CID_PRIVATE_BASE + 45)
#define V4L2_CID_TIMESET              (V4L2_CID_PRIVATE_BASE + 46)
#define V4L2_CID_CROPSET              (V4L2_CID_PRIVATE_BASE + 47)

/*
Disabled:
#define V4L2_CID_FLAG                 (V4L2_CID_PRIVATE_BASE + 8)
#define V4L2_CID_LFRAMESIZE           (V4L2_CID_PRIVATE_BASE + 12)
#define V4L2_CID_LBUFBAR              (V4L2_CID_PRIVATE_BASE + 13)
#define V4L2_CID_LPATHSCALE           (V4L2_CID_PRIVATE_BASE + 14)
#define V4L2_CID_OSDTIMESYNC          (V4L2_CID_PRIVATE_BASE + 17)
#define V4L2_CID_ALLMODE              (V4L2_CID_PRIVATE_BASE + 21)
#define V4L2_CID_DISABLEMOSAIC        (V4L2_CID_PRIVATE_BASE + 23)
#define V4L2_CID_STOPBOARD            (V4L2_CID_PRIVATE_BASE + 25)
#define V4L2_CID_DROPFRAME            (V4L2_CID_PRIVATE_BASE + 29)
#define V4L2_CID_DSPPING              (V4L2_CID_PRIVATE_BASE + 30)
#define V4L2_CID_WI2C                 (V4L2_CID_PRIVATE_BASE + 31)
#define V4L2_CID_RI2C                 (V4L2_CID_PRIVATE_BASE + 32)
#define V4L2_CID_GETCLR               (V4L2_CID_PRIVATE_BASE + 39)
#define V4L2_CID_WATCHEN              (V4L2_CID_PRIVATE_BASE + 41)
#define V4L2_CID_WATCHSET             (V4L2_CID_PRIVATE_BASE + 42)
*/

typedef struct {
                                  // offset
  uint32_t AVH_uReserved0;        // 0x00
  uint32_t AVH_uTotalSize;        // 0x04
  uint32_t AVH_uReserved1;        // 0x08
  uint32_t AVH_uVideoFrameType;   // 0x0C
  uint32_t AVH_uVideoTimeStamp;   // 0x10
  uint32_t AVH_uVideoDataLen;     // 0x14
  uint32_t AVH_uAudioDataLen;     // 0x18
  uint32_t AVH_uVideoSpsLen;      // 0x1C
  uint32_t AVH_uVideoDataOffset;  // 0x20
  uint32_t AVH_uAudioDataOffset;  // 0x24
  uint32_t AVH_uReserved2;        // 0x28
  uint8_t  AVH_uVlost_En;         // 0x2C
  uint8_t  AVH_uMotion_En;        // 0x2D
  uint8_t  AVH_uVideo_En;         // 0x2E
  uint8_t  AVH_uAudio_En;         // 0x2F
  uint32_t AVH_uVideoLost;        // 0x30
  uint32_t AVH_uMotionDataLen;    // 0x34
  uint32_t AVH_uAudioTimeStamp;   // 0x38
  uint8_t  AVH_uReserved3[88 - 0x3c];
} __attribute__((packed)) DMAVHEADER, *PDMAVHEADER;

#define AV_TOTALSIZE(x)          (((PDMAVHEADER)(x))->AVH_uTotalSize)

/* Video data */
#define AV_VENCODEN(x)           (((PDMAVHEADER)(x))->AVH_uVideo_En)
#define AV_VLEN(x)               (((PDMAVHEADER)(x))->AVH_uVideoDataLen)
#define AV_VOFFSET(x)            (((PDMAVHEADER)(x))->AVH_uVideoDataOffset)
#define AV_VTSTAMP(x)            (((PDMAVHEADER)(x))->AVH_uVideoTimeStamp)
#define AV_VFRAMETYPE(x)         (((PDMAVHEADER)(x))->AVH_uVideoFrameType)
#define AV_VSPSLEN(x)            (((PDMAVHEADER)(x))->AVH_uVideoSpsLen)

/* Audio data */
#define AV_AENCODEN(x)           (((PDMAVHEADER)(x))->AVH_uAudio_En)
#define AV_ALEN(x)               (((PDMAVHEADER)(x))->AVH_uAudioDataLen)
#define AV_AOFFSET(x)            (((PDMAVHEADER)(x))->AVH_uAudioDataOffset)
#define AV_ATSTAMP(x)            (((PDMAVHEADER)(x))->AVH_uAudioTimeStamp)

/* Motion Detect data */
#define AV_MDEN(x)               (((PDMAVHEADER)(x))->AVH_uMotion_En)
#define AV_MDLEN(x)              (((PDMAVHEADER)(x))->AVH_uMotionDataLen)
#define AV_MDOFFSET(x)           (88)

/* Video Lost data */
#define AV_VLOSTEN(x)            (((PDMAVHEADER)(x))->AVH_uVlost_En)
#define AV_VLOSTV(x)             (((PDMAVHEADER)(x))->AVH_uVideoLost)

#define PACKET_WITH_VIDEO(x)     (AV_VENCODEN(x) && AV_VLEN(x))
#define PACKET_WITH_AUDIO(x)     (AV_AENCODEN(x) && AV_ALEN(x))
#define PACKET_WITH_MOTION(x)    (AV_MDEN(x) && AV_MDLEN(x))
#define PACKET_WITH_VIDEOLOST(x) (AV_VLOSTEN(x) && AV_VLOSTV(x))
#define PACKET_IS_IFRAME(x)      (AV_VFRAMETYPE(x) == 0)
#define PACKET_IS_PFRAME(x)      (AV_VFRAMETYPE(x) == 2)
#define PACKET_IS_BFRAME(x)      (AV_VFRAMETYPE(x) == 3)
#define PACKET_IS_SPSFRAME(x)    (AV_VSPSLEN(x))
#define PACKET_IS_ISPSFRAME(x)   (PACKET_IS_IFRAME(x) && PACKET_IS_SPSFRAME(x))

#define MODE_NTSC  0
#define MODE_PAL   1

#define STATUS_COMPRESS  0x00000001
#define STATUS_LIVE      0x00000002

#define R_4CIF  0
#define R_CIF   1

#define PREVIEW_PATH_DDR  0
#define PREVIEW_PATH_PCI  1

#define TYPE_CBR  0
#define TYPE_VBR  2

#define GPIO_MINPIN  0
#define GPIO_MAXPIN  9
#define GPIO_OUT     0
#define GPIO_IN      1

struct spct6100_gop {
    unsigned int gopp;
    unsigned int gopb;
};

struct spct6100_equality {
    unsigned int type;
    unsigned int minbitrate;
    unsigned int maxbitrate;
    unsigned int qpi;
    unsigned int qpb;
    unsigned int qpp;
};

struct spct6100_osd {
    unsigned int tran;   // not used
    unsigned int acolor; // 1 <= acolor <= 40
    unsigned int vscale; // not used
    unsigned int hscale; // not used
    unsigned int timer_enable;
    unsigned int osd_enable;
    unsigned int osd_pos_x;
    unsigned int osd_pos_y;
    char str[16];
};

struct spct6100_mosaic {
    unsigned int num;
    unsigned int x[6];
    unsigned int y[6];
    unsigned int w[6];
    unsigned int h[6];
};

struct spct6100_cropset {
    unsigned short start_x;
    unsigned short start_y;
    unsigned short width;
    unsigned short height;
};

struct spct6100_gpio {
    unsigned int pinid;
    unsigned int value;
};

struct spct6100_memory {
    unsigned int dsp_addr; // DSP address
    char *buf;             // Host buffer
    unsigned int buf_size; // Host buffer size
};

/*
struct spct6100_eeprom {
    unsigned int deviceid;
    unsigned int addr;
    unsigned int length;
    void *buf;
};
*/

#endif /* SPCT6100_V4L2_TYPES_H */
