/********************************************\
cci.h:
	DSP communication with HOST via PCI
\********************************************/
#ifndef _CCI_H_
#define _CCI_H_

#ifdef	_WIN32
typedef ULONG	Uint32;
typedef USHORT	Uint16;
typedef UCHAR	Uint8;

#define	CHT_PAL					0
#define	CHT_NTSC				1

#define	CHT_I_FRAME				0
#define	CHT_N_FRAME				1		//NULL frame, video length = 0
#define	CHT_P_FRAME				2
#define	CHT_B_FRAME				3
#define	CHT_CP_FRAME			4	//Copy P frame, video length = 0
#define	CHT_CB_FRAME			5	//Copy B frame, video length = 0

#define	AVC_MODE_SHIFT			2
#define	AVC_MODE(x)				((x)>>AVC_MODE_SHIFT)
#define	AVC_PAL_2CIF			0
#define	AVC_PAL_CIF				1
#define	AVC_PAL_MCIF			2
#define	AVC_PAL_4CIF			3

#define	AVC_NTSC_2CIF			4
#define	AVC_NTSC_CIF			5
#define	AVC_NTSC_MCIF			6
#define	AVC_NTSC_4CIF			7

#define AVC_I_FRAME 			0
#define AVC_P_FRAME 			1
#define AVC_B_FRAME 			2

#endif

#define	CCI_MAGIC_ID	(Uint32)(('S'<<24)|('Z'<<16)|('N'<<8)|('0'))
#define	CCI_H264_ID		(Uint32)(('H'<<24)|('2'<<16)|('6'<<8)|('4'))
#define	CCI_TEST_ID		(Uint32)(('T'<<24)|('E'<<16)|('S'<<8)|('T'))
#define	CCI_WRITER_ID	(Uint32)(('S'<<24)|('M'<<16)|('J'<<8)|('9'))
#define	CCI_CHECK_ID	(Uint32)(('C'<<24)|('H'<<16)|('C'<<8)|('K'))

/*********************************************************************\
				CCI ENV ADDRESS DEFINE
				
	CCI_ENV_ADDR:	CCI_ENV			
	-0x0100		:	OSDTIME
	-0x0500		:	OSDNAME
	-0x0A00		:	OSDICON
	-0x0E00		:	OSDBOARDNAME
	-0x1600		:	OSDAUXNAME
\*********************************************************************/

#define	CCI_ENV_ADDR	0x0003fe00
#define	CCI_ENV_LEN		0x00000200

#define	CCI_OSDTIME_ADDR	(CCI_ENV_ADDR-0x100)
//#define	CCI_OSDNAME_ADDR	(CCI_ENV_ADDR-0x500)
//#define	CCI_OSDNAME_LEN		0x100

#define	CCI_OSDICON_ADDR		(CCI_OSDTIME_ADDR-0xA00)
#define	CCI_OSDICON_TOTALLEN	0xA00
#define	CCI_OSDICON_LEN			0x200
#define	CCI_OSDICON_ACT_ADDR	(CCI_OSDICON_ADDR)
#define	CCI_OSDICON_PRO_ADDR	(CCI_OSDICON_ADDR+0x200)
#define	CCI_OSDICON_ALA_ADDR	(CCI_OSDICON_ADDR+0x400)
#define	CCI_OSDICON_REC_ADDR	(CCI_OSDICON_ADDR+0x600)
#define	CCI_OSDICON_AUD_ADDR	(CCI_OSDICON_ADDR+0x800)

//#define	CCI_OSDBOARDNAME_ADDR	(CCI_OSDICON_ADDR-0x400)
//#define	CCI_OSDBOARDNAME_LEN	0x100
//#define	CCI_OSDAUXNAME_ADDR		(CCI_OSDBOARDNAME_ADDR-0x800)
//#define	CCI_OSDAUXNAME_LEN		0x200

////////////////////////////////////////////////////////////////////
//about osd text
#define	OSDTEXT_TIME			1
#define	OSDTEXT_NAME			2
#define	OSDTEXT_DATE			4
#define	OSDTEXT_MASK			8

#define		OSDTEXT_MAX		8
#define		OSDTEXT_PIXELS	256
#define		OSDTEXT_LINES	16
#define		OSDTEXT_BYTES	768		//OSDTEXT_PIXELS*3
#define		OSDTEXT_LEN		(OSDTEXT_PIXELS*OSDTEXT_LINES*3)
typedef struct tag_CCI_OSDTEXT
{
	Uint16		flag;
	Uint16		offset;
	Uint32		uAddrText;	
}CCI_OSDTEXT, *PCCI_OSDTEXT;

typedef struct tag_CCI_OSDTIME
{
	Uint16		flag;
	Uint16		offset;
	Uint32		uColor;
}CCI_OSDTIME, *PCCI_OSDTIME;

typedef struct tag_CCI_OSDTEXT_CHAN
{
	Uint32			flag;
	CCI_OSDTIME		osdtime;
	CCI_OSDTEXT		osdname;
	CCI_OSDTEXT		osddate;
	CCI_OSDTEXT		osdtext[OSDTEXT_MAX];
}CCI_OSDTEXT_CHAN, *PCCI_OSDTEXT_CHAN;

typedef struct tag_CCI_OSDTEXT_SYS
{
	Uint32				flag;
	CCI_OSDTEXT			osdtime;
	CCI_OSDTEXT_CHAN	osdchan[4];
}CCI_OSDTEXT_SYS, *PCCI_OSDTEXT_SYS;

#define	CCI_OSDTEXT_SYS_ADDR		(CCI_OSDICON_ADDR-0x400)
#define	CCI_OSDTEXT_SYS_LEN			sizeof(CCI_OSDTEXT_SYS)

#define	CCI_OSDTEXT_SYS_TIMEOFF		(Uint32)(&((PCCI_OSDTEXT_SYS)0)->osdtime)
#define	CCI_OSDTEXT_SYS_TIMEADDR	(CCI_OSDTEXT_SYS_ADDR+CCI_OSDTEXT_SYS_TIMEOFF)

#define	CCI_OSDTEXT_SYS_CHANOFF(x)	(Uint32)(&((PCCI_OSDTEXT_SYS)0)->osdchan[x])
#define	CCI_OSDTEXT_SYS_CHANADDR(x)	(CCI_OSDTEXT_SYS_ADDR+CCI_OSDTEXT_SYS_CHANOFF(x))

#define	CCI_OSDTEXT_CHAN_TIMEOFF	(Uint32)(&((PCCI_OSDTEXT_CHAN)0)->osdtime)
#define	CCI_OSDTEXT_CHAN_NAMEOFF	(Uint32)(&((PCCI_OSDTEXT_CHAN)0)->osdname)
#define	CCI_OSDTEXT_CHAN_DATEOFF	(Uint32)(&((PCCI_OSDTEXT_CHAN)0)->osddate)	
#define	CCI_OSDTEXT_CHAN_TEXTOFF	(Uint32)(&((PCCI_OSDTEXT_CHAN)0)->osdtext[0])
///////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////

#define	CCI_CHAN_PARAM_Q_SET		0x00000001
#define	CCI_CHAN_PARAM_COLOR_SET	0x00000002
#define	CCI_CHAN_PARAM_MASK_SET		0x00000004
#define	CCI_CHAN_PARAM_BITCTRL_SET	0x00000008
#define	CCI_CHAN_PARAM_REQIFRAME	0x00000080
#define	CCI_CHAN_PARAM_SHOWSIZE		0x00000100
#define	CCI_CHAN_PARAM_SPEED		0x00000200
#define	CCI_CHAN_PARAM_REQCAPTURE	0x00000400

#define	CCI_CHAN_PARAM_VLOSTCHK		0x00010000
#define	CCI_CHAN_PARAM_DRAWMASK		0x00020000
#define	CCI_CHAN_PARAM_DRAWTIME		0x00040000
#define	CCI_CHAN_PARAM_DRAWOSD		0x00080000
#define	CCI_CHAN_PARAM_DRAWACTIVE	0x00100000
#define	CCI_CHAN_PARAM_ALARM		0x00200000
#define	CCI_CHAN_PARAM_MOSAIC		0x00400000
#define	CCI_CHAN_PARAM_SETENABLE	0x80000000

#define	CCI_CHAN_PARAM_CLR			0x003f0000		

typedef struct tag_DM_MASK_AREA
{
	Uint32	uMaskRect;
	Uint16	uMaskState;
	Uint16	uMaskMoveCnt;
}DM_MASK_AREA, *PDM_MASK_AREA;

typedef struct tag_CCI_CHAN_PARAM
{
	Uint32	uFlags;
	Uint8	bQ;					//31~24: reserved, must be 0
								//23~16: Qi
								//15~8:	 Qp
								//7~0:	 Qb
	Uint8	bShowSize;			//Show size: CHT_CIF, CHT_2CIF, CHT_QCIF, CHT_4CIF
	Uint8	bReserved0;
	Uint8	bEncoderSpeed;		//Encoder frame rate per second
	Uint32	uMosaicArea;		//31~24: left
								//23~16: top
								//15~8:	 right
								//7~0:	 bottom
								
	Uint32	uColor;				//31~24: bright 
								//23~16: contrast
								//15~8:	 saturation  
								//7~0:	 hue
								
	Uint32	uMotionParam;		//31~16: motion threshold
								//15~8:  motion move frame number
								//7~0:	 motion area number
								
	Uint32	uBitCtrl;			//31:	 bit control enable flag
								//30~8:  bit control threshold
								//7~0:	 frame per second
	DM_MASK_AREA	tMaskArea[5];
}CCI_CHAN_PARAM, *PCCI_CHAN_PARAM;


typedef struct tag_CCI_CHAN_SET
{
	Uint32	uChanID;
	Uint32	uStatus;
	Uint32	uMode;					//byte0: video mode(PAL/NTSC)
	Uint32 	uReserved;
	Uint32	uVideoAddress[2];
	Uint32	uVideoPitch[2];	
	CCI_CHAN_PARAM	tParam;
}CCI_CHAN_SET, *PCCI_CHAN_SET;


typedef struct tag_CCI_DSP2HOST
{
	Uint32			uChanID;
	Uint32			uFlag;		//0:		data buffer flip
								//1:		display buffer flip
								//2:		capture buffer flip
								//3:		capture or not
								//15~8:		show size
	Uint32			uDataLen;
	Uint32			uIOValue;	//31~16:	Input value
								//15~0:		Output value
}CCI_DSP2HOST, *PCCI_DSP2HOST;

typedef struct tag_CCI_EEROM_PARAM
{
	Uint8			bCI2100id;
	Uint8			bLanguage;
	Uint8			bODMid;
	Uint8			bRevid;
	Uint32			uTime;
	Uint16			uWriterID;
	Uint8			uWriterAddr;
	Uint8			uReserved8;
	Uint16			uSubDevId;
	Uint16			uReserved16;
	Uint8			bSerialNum[16];
}CCI_EEROM_PARAM, *PCCI_EEROM_PARAM;

typedef struct tag_CCI_ENV
{
	Uint32			uMagicID;
	Uint32			uDevID;		//31~24: reserved
								//23~16: language id
								//15~8:  odm id
								//7~0:	 odm ext id
	Uint32			uOutputSet;	//31~16:	watchdog
								//15~0:		output value
	//Uint32			uReserved;
	Uint8			uDisplayMode;
	Uint8			uReserved0;
	Uint8			uReserved1;
	Uint8			uReserved2;
	
	Uint32			uDataBuf[2];
	CCI_DSP2HOST	dsp2host;
		
	CCI_CHAN_SET	chanSet[4];
	CCI_EEROM_PARAM	eerom_param;	
	Uint8			bTime[8];
	Uint32			dwIcon[4];
	Uint16			wAudioMask[4];
}CCI_ENV, *PCCI_ENV;


#define	CCI_VERSION_ADDR	((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->uMagicID))
#define	CCI_VERSION_LEN		(2*sizeof(Uint32) )

#define	CCI_EEROM_PARAM_ADDR	((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->eerom_param))
#define	CCI_EEROM_PARAM_LEN		sizeof(CCI_EEROM_PARAM)

#define	CCI_DSP2HOST_ADDR ((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->dsp2host))	
#define	CCI_DSP2HOST_DATABUF_LEN	(64*1024)
#define	CCI_MAX_VIDEODATA_LEN		(60*1024)
#define	CCI_AVDATABUF_LEN			(256*1024)
#define	CCI_MAX_AVDATA_LEN			(250*1024)

#define	CCI_CHAN_SET_ADDR(id) ((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->chanSet[id]))
#define	CCI_CHAN_PARAM_ADDR(id) ((Uint32)(&((PCCI_CHAN_SET)CCI_CHAN_SET_ADDR(id))->tParam))

#define	CCI_OUTPUTSET_ADDR ((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->uOutputSet))

#define	CCI_TIME_ADDR	((Uint32)(&((PCCI_ENV)CCI_ENV_ADDR)->bTime[0]))

#define	CCI_ICONSET_ADDR(id) ((Uint32)(& ((PCCI_ENV)CCI_ENV_ADDR)->dwIcon[id]))

#define	CCI_AUDIOMASK_ADDR 	((Uint32)(& ((PCCI_ENV)CCI_ENV_ADDR)->wAudioMask[0]))

#define	ICON_ACTIVE		0x00000010
#define	ICON_PROGRAM	0x00000008
#define	ICON_ALARM		0x00000004
#define	ICON_RECORD		0x00000002
#define	ICON_AUDIO		0x00000001
/*************************************************\
DMAUXDATA:
	ݵĽṹ
\*************************************************/


/*************************************************\
DMAVHEADER:
	ƵݰͷĽṹ
\*************************************************/
typedef struct tag_DMAVHEADER
{
	//ýṹֽ
	Uint32		uSize;
	//ݰֽ
	Uint32		uTotalDataSize;
	//֡
	Uint32		uFrameIndex;
	//־
	Uint32		uFlags;
	//ʱǩ
	Uint32		uTimeStamp;
	//Ƶݵֽ
	Uint32		uVideoDataLen;
	//Ƶݵֽ
	Uint32		uAudioDataLen;
	//
	Uint32		uReserved0;
	//ƵݿʼƫƵַڸݰͷ
	Uint32		uVideoDataAddrOffset;
	//ƵݿʼƫƵַڸݰͷ
	Uint32		uAudioDataAddrOffset;
	//
	Uint32		uReserved1;
	//
	Uint32		uMaskNum;
	DM_MASK_AREA	tMaskArea[5];
}DMAVHEADER, *PDMAVHEADER;

#define	DMAV_FLAGS_FRAMETYPE_MASK	0x000000ffu
#define	DMAV_FLAGS_LOST				0x00010000u
#define	DMAV_FLAGS_ALARM			0x00020000u
#define	DMAV_FLAGS_AUX				0x00040000u
#define	DMAV_FLAGS_VCODEC_MASK		0x00f00000u
#define	DMAV_FLAGS_ACODEC_MASK		0x0f000000u
#define	DMAV_FLAGS_ALARMPROCESS		0x20000000u
#define	DMAV_FLAGS_RT				0x40000000u
#define	DMAV_FLAGS_AUDIO			0x80000000u
#define	DMAV_FRAMETYPE(p)	( ((PDMAVHEADER)p)->uFlags & DMAV_FLAGS_FRAMETYPE_MASK)


#define	CCI_GET_CHANSET(v, x)	( (PCCI_CHAN_SET)( &((PCCI_ENV)v)->chanSet[x] ) )
#define	CCI_GET_CHANPARAM(v, x) ( (PCCI_CHAN_PARAM)(&(CCI_GET_CHANSET(v, x))->tParam) )
#define	CCI_GET_CHANQ(v,x)		( (Uint32)  CCI_GET_CHANPARAM(v,x)->uQ )
#define	CCI_GET_CHANQI(v, x)	( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uQ>>16)&0x000000ff) )
#define	CCI_GET_CHANQP(v, x)	( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uQ>>8)&0x000000ff) )
#define	CCI_GET_CHANQB(v, x)	( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uQ)&0x000000ff) )
#define	CCI_GET_CHANMOTIONTHRE(v, x)	( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uMotionParam>>16)&0x0000ffff) )
#define	CCI_GET_CHANMOTIONNUM(v, x)		( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uMotionParam)&0x000000ff) )
#define	CCI_GET_CHANMOTIONALARMNUM(v, x) ( (Uint32)( ((CCI_GET_CHANPARAM(v,x))->uMotionParam>>8)&0x000000ff) )


/*********************************************************************************\
					°OSDĲ֣2008/03/29
\*********************************************************************************/
#define	CCIOSD_MAXNUM		8
typedef	struct tag_CCIOSD
{	
	Uint16	flag;
	Uint16	line;		//OSD ʾţʵʾλ32*linesλã4CIFͼ
	Uint16	offsetx;	//ˮƽʼַʵʾʼλ64*offsetxʼ4CIFͼ
	Uint16	len;		//ˮƽĿȣʵʾĿΪ64*len	
	Uint32	color;		//color_y, color_y, color_v, color_u,	if color == 0, this object is invalid
	Uint32	uOSDAddr;	//ʵOSDݴŵλ
}CCIOSD, *PCCIOSD;

typedef struct tag_CCIOSD_CHAN
{
	Uint32	flags;
	CCIOSD	osdTime;
	CCIOSD	osdDate;
	CCIOSD	osdName;	//Channel name	
	CCIOSD	osd[CCIOSD_MAXNUM];
}CCIOSD_CHAN, *PCCIOSD_CHAN;
#define	CCIOSD_CHAN_SHOWTIME	1
#define	CCIOSD_CHAN_SHOWNAME	2
#define	CCIOSD_CHAN_SHOWOSD		4
#define	CCIOSD_CHAN_SHOWDATE	8

typedef struct tag_CCIOSDSYS
{
	CCIOSD		osdTime;
	CCIOSD		osdDate;
	CCIOSD_CHAN	osdchan[4];
}CCIOSD_SYS, *PCCIOSD_SYS;
#define		CCIOSD_SYS_ADDR		(CCI_ENV_ADDR-sizeof(CCIOSD_SYS) )


#endif //_CCI_H_
