#ifndef _PACDMP_DSP_REGS_H_
#define _PACDMP_DSP_REGS_H_

#define PACDMP_DSP_DMEM_START               0x00000     // (RW) data memory: 64KB
#define PACDMP_DSP_DMEM_END                 0x0FFFF     // (RW)
#define PACDMP_DSP_CPSA                     0x50000     // (RW) dsp code start address
#define PACDMP_DSP_CPS                      0x50004     // (--) 0: start directly, 1: start while ICDONE == 1
#define PACDMP_DSP_CBUSY                    0x50008     // (R-) bit0: core is running, bit1: exception, bit2: debugging
#define PACDMP_DSP_CLRINT                   0x5000C     // (-W) clear DSP interrupt signal
#define PACDMP_DSP_ICSA                     0x50010     // (RW) Start Address to ICache
#define PACDMP_DSP_ICPS                     0x50014     // (RW) Initial program size to ICache
#define PACDMP_DSP_ICMS                     0x50018     // (RW) Number of miss cache lines
#define PACDMP_DSP_DBGISR                   0x5001C     // (RW) debugger ISR address
#define PACDMP_DSP_EXCISR                   0x50020     // (RW) exception ISR address
#define PACDMP_DSP_FIQISR                   0x50024     // (RW) FIQ ISR address
#define PACDMP_DSP_IRQISR                   0x50028     // (RW) IRQ ISR address
#define PACDMP_DSP_ICDONE                   0x5002C     // (R-) indicates that dsp initialization is done
#define PACDMP_DSP_DCCR                     0x50030     // (R-) hw contention count
#define PACDMP_DSP_IMCR                     0x50034     // (R-) instruction miss count
#define PACDMP_DSP_SYSBAR                   0x50038     // (RW) dsp base address in system
#define PACDMP_DSP_DMCFGI                   0x5003C     // (RW) data memory interleaving mode, 0: linear, 1: interleaving
#define PACDMP_DSP_DMCFGP                   0x50040     // (RW) data memory priority, 0: Core > BIU > CFU, 1: BIU > Core > CFU
#define PACDMP_DSP_ICFLUSH                  0x50044     // (-W) flush the instruction cache
#define PACDMP_DSP_ICS                      0x50048     // (-W) start cache initialization

#define PACDMP_DSP_EXC                      0x5004C     // (R-) exception status, bit0: scalar overflow,        bit4: undefined CAP, 
                                                        //                        bit5: undefined instruction,  bit6: illegal sequence

#define PACDMP_DSP_CFGWAR                   0x50050     // (R-) warning of improper register setting, 
                                                        //          bit0: improper ICPSA                bit1: ICISIZE
                                                        //          bit2: improper TR_WIDTH             bit3: improper SGR, DSR, or BSZ
                                                        //          bit4: improper SAR or DAR

#define PACDMP_DSP_DMASR                    0x50054     // (R-) dma status register
#define PACDMP_DSP_VERSION                  0x50058     // (R-) version code
#define PACDMP_DSP_DMACLRINT                0x50070     // (-W) clear internal DMA interrupt

// FIXME: TODO: PAC DSP IDMA registers... (internal DMA)


#endif /* _PACDMP_DSP_REGS_H_ */
