#ifndef _PACDMP_SYSDMA_REGS_H_
#define _PACDMP_SYSDMA_REGS_H_

/*****************************************************
 * Per channel reg. definition
 *****************************************************/
#define PACDMP_SYSDMA_CHAN_SAR		0x0000	// (RW)	source address register,
                                            //			[31:0]	: any value

#define PACDMP_SYSDMA_CHAN_DAR		0x0004	// (RW) destination address register,
                                            //			[31:0]	: any value

#define PACDMP_SYSDMA_CHAN_SGR		0x0008	// (RW) source gather register,
                                            //			[31:20] : SGC, source gather count ( in transfer unit )
                                            //			[19:0]	: source gather offset ( in transfer unit ), between two successive start address

#define PACDMP_SYSDMA_CHAN_DSR		0x000C	// (RW)	destination scatter register, 
                                            //			[31:20] : DSC, destination scatter count ( in transfer unit )
                                            //			[19:0]	: DSO, destination scatter offset ( in transfer unit ), between two successive address

#define PACDMP_SYSDMA_CHAN_CTL		0x0010	// (RW) control register,
                                            //			[31:16] : block transfer size
                                            //			[12:12]	: interrupt enable bit, 0: disable, 1: enable
                                            //			[9:9]	: destination scatter enable bit, 0: disable, 1: enable
                                            //			[8:8]	: source gather enable bit, 0:disable, 1: enable
                                            //			[4:4]	: data transfer direction,
                                            //						0: external memory --> internal memory
                                            //						1: internal memory --> external memory
                                            //			[1:0]	: transfer unit,
                                            //						00: byte, 	01: half-word
                                            //						10: word,	11: non-alignment

#define PACDMP_SYSDMA_CHAN_EN		0x0014	// (-W) activate data transfer,
                                            //			[31:0]	: any value to activate

#define PACDMP_SYSDMA_CHAN_CLR		0x0018	// (-W) clear interrupt status,
                                            //			[31:0]	: any vaule to clear

#define PACDMP_SYSDMA_CHAN_PDCTL	0x001C	// (RW) padding mode (for video codec, e.g., H.264, ...) control register,
                                            //			[17:12]	: macroblock height (unit: byte)
                                            //			[11:8]	: padding number (unit: byte)
                                            //			[5:4]	: padding direction,
                                            //						00: up, 	01: down
                                            //						10: left,	11: right
                                            //			[0:0]	: source gather enable bit,
                                            //						0: disable, 	1: enable

#define PACDMP_SYSDMA_CHAN_OFFSET	0x0020	// channel offset

/****************************************************
 * Channel based address 
 ****************************************************/
#define PACDMP_SYSDMA_CHAN0			0x0000
#define PACDMP_SYSDMA_CHAN1			0x0020	// equal to PACDMP_SYSDMA_CHAN0 + PACDMP_SYSDMA_CHAN_OFFSET * 1
#define PACDMP_SYSDMA_CHAN2			0x0040	// equal to PACDMP_SYSDMA_CHAN0 + PACDMP_SYSDMA_CHAN_OFFSET * 2
#define PACDMP_SYSDMA_CHAN3			0x0060	// HU...
#define PACDMP_SYSDMA_CHAN4			0x0080
#define PACDMP_SYSDMA_CHAN5			0x00A0
#define PACDMP_SYSDMA_CHAN6			0x00C0
#define PACDMP_SYSDMA_CHAN7			0x00E0
#define PACDMP_SYSDMA_CHAN8			0x0100
#define PACDMP_SYSDMA_CHAN9			0x0120
#define PACDMP_SYSDMA_CHAN10		0x0140
#define PACDMP_SYSDMA_CHAN11		0x0160
#define PACDMP_SYSDMA_CHAN12		0x0180
#define PACDMP_SYSDMA_CHAN13		0x01A0
#define PACDMP_SYSDMA_CHAN14		0x01C0
#define PACDMP_SYSDMA_CHAN15		0x01E0

// Global status register of all channels
#define PACDMP_SYSDMA_DMASR			0x0200	// (R-) status register for channels,
                                            // 		[x:x]	:  x = 0 ~ 15, for channel 0 ~ channel 15
                                            //			0: done, 1: busy
                                            //		[16:16]	: OR operation of bit0 ~ bit15

#endif /* _PACDMP_SYSDMA_REGS_H_ */
