#ifndef _PACDMP_VIC_REGS_H_
#define _PACDMP_VIC_REGS_H_

/***********************************************************************\
 * Registers per VIC
\***********************************************************************/
#define VIC_IRQ_STATUS              0x0000      // (R-) Shows the status of the interrupts after masking
#define VIC_FIQ_STATUS              0x0004      // (R-) Shows the status of the interrupts after masking
#define VIC_RAW_STATUS              0x0008      // (R-) Shows the status of the interrupts before masking
#define VIC_INT_SELECT              0x000C      // (RW) Selects the type of interrupt,
                                                //          0: IRQ, 1: FIQ (only valid for DSP VIC1, VIC2)
#define VIC_INT_ENABLE              0x0010      // (RW) Enables the interrupt request lines
                                                //			For Write, 1: enable, 0: no-effect for bit0~bit31
#define VIC_INT_ENABLE_CLEAR        0x0014      // (-W) Clears bits in the INT_EN register,
                                                //          1: clear, 0: no-effect for bit0~bit31

#define VIC_INT_SOFT                0x0018      // (RW) Software interrupt register
#define VIC_INT_SOFT_CLEAR          0x001C      // (-W) Software interrupt clear register
#define VIC_PROTECT                 0x0020      // (RW) Protection enable register

#define VIC_VECT_ADDR               0x02C0		// (RW)	Vector-based address register
                                                //			Read : Two things happens:
                                                //                  1. return the address of the current activate ISR and ;
                                                //                  2. DISABLE lower priority interrupts !! ( VIC HW behavior )
                                                //          Write: when current activate ISR is processed, 
                                                //                 write any value to enable the lower priority interrupts !!

#define VIC_DEF_VECT_ADDR           0x0034      // Default vector address register

#define VIC_VECT_ADDR0              0x0100      // Vector address register, 0~15 (32bit register)
#define VIC_VECT_CNTL0              0x0200      // Vector control register, 0~15 (32bit register)
#define VIC_ITCR                    0x0300      // VIC test control register

#define VIC_VECT_CNTL_ENABLE        (1<<5)      // bit5 in Vector control register

// for backward compatibility only !!
#define PACDMP_VIC_INT_EN           VIC_INT_ENABLE
#define PACDMP_VIC_INT_CLN_EN       VIC_INT_ENABLE_CLEAR


/***********************************************************************\
 * VIC pin assignment
\***********************************************************************/
// Host Lv1 VIC
#define PACDMP_VIC_LV1_VCP0_LV              0x00000001  // Video capture port 0 Live Mode
#define PACDMP_VIC_LV1_VCP0_ENC             0x00000002  // Video capture port 0 Encode
#define PACDMP_VIC_LV1_VCP0_EXC             0x00000004  // Video capture port 0 Exception
#define PACDMP_VIC_LV1_VCP0_OSD_H           0x00000008  // Video capture port 0 OSD Hour Update
#define PACDMP_VIC_LV1_VCP0_OSD_D           0x00000010  // Video capture port 0 OSD Daily Update
#define PACDMP_VIC_LV1_VCP1_LV              0x00000020  // Video capture port 1 Live Mode
#define PACDMP_VIC_LV1_VCP1_ENC             0x00000040  // Video capture port 1 Encode
#define PACDMP_VIC_LV1_VCP1_EXC             0x00000080  // Video capture port 1 Exception
#define PACDMP_VIC_LV1_VCP1_OSD_H           0x00000100  // Video capture port 1 OSD Hour Update
#define PACDMP_VIC_LV1_VCP1_OSD_D           0x00000200  // Video capture port 1 OSD Daily Update
#define PACDMP_VIC_LV1_VCP2_LV              0x00000400  // Video capture port 2 Live Mode
#define PACDMP_VIC_LV1_VCP2_ENC             0x00000800  // Video capture port 2 Encode
#define PACDMP_VIC_LV1_VCP2_EXC             0x00001000  // Video capture port 2 Exception
#define PACDMP_VIC_LV1_VCP2_OSD_H           0x00002000  // Video capture port 2 OSD Hour Update
#define PACDMP_VIC_LV1_VCP2_OSD_D           0x00004000  // Video capture port 2 OSD Daily Update
#define PACDMP_VIC_LV1_VCP3_LV              0x00008000  // Video capture port 3 Live Mode
#define PACDMP_VIC_LV1_VCP3_ENC             0x00010000  // Video capture port 3 Encode
#define PACDMP_VIC_LV1_VCP3_EXC             0x00020000  // Video capture port 3 Exception
#define PACDMP_VIC_LV1_VCP3_OSD_H           0x00040000  // Video capture port 3 OSD Hour Update
#define PACDMP_VIC_LV1_VCP3_OSD_D           0x00080000  // Video capture port 3 OSD Daily Update
#define PACDMP_VIC_LV1_VCP4_EXC             0x00100000  // Video display port 4 Exception

#define PACDMP_VIC_LV1_ADIF_TX              0x00200000  // Audio input interface transmission
#define PACDMP_VIC_LV1_ADIF_RX              0x00400000  // Audio input interface received.

#define PACDMP_VIC_LV1_DSP_EXC              0x00800000  // PACDSP exception
#define PACDMP_VIC_LV1_DSP_INT              0x01000000  // PACDSP interrupt
#define PACDMP_VIC_LV1_DSP_SWI              0x02000000  // PACDSP software interrupt

#define PACDMP_VIC_LV1_HP_INT               0x04000000  // FIX ME: Host Processor interrupt (?)
#define PACDMP_VIC_LV1_EXT_INT0             0x08000000  // External interrupt source 0
#define PACDMP_VIC_LV1_EXT_INT1             0x10000000  // External interrupt source 1
#define PACDMP_VIC_LV1_EXT_INT2             0x20000000  // External interrupt source 2
#define PACDMP_VIC_LV1_EXT_INT3             0x40000000  // External interrupt source 3

// Host Lv2 VIC
#define PACDMP_VIC_LV2_SYSDMA_CHAN0         0x00000001  // System DMA Channel 0
#define PACDMP_VIC_LV2_SYSDMA_CHAN1         0x00000002  // System DMA Channel 1
#define PACDMP_VIC_LV2_SYSDMA_CHAN2         0x00000004  // System DMA Channel 2
#define PACDMP_VIC_LV2_SYSDMA_CHAN3         0x00000008  // System DMA Channel 3
#define PACDMP_VIC_LV2_SYSDMA_CHAN4         0x00000010  // System DMA Channel 4
#define PACDMP_VIC_LV2_SYSDMA_CHAN5         0x00000020  // System DMA Channel 5
#define PACDMP_VIC_LV2_SYSDMA_CHAN6         0x00000040  // System DMA Channel 6
#define PACDMP_VIC_LV2_SYSDMA_CHAN7         0x00000080  // System DMA Channel 7
#define PACDMP_VIC_LV2_SYSDMA_CHAN8         0x00000100  // System DMA Channel 8
#define PACDMP_VIC_LV2_SYSDMA_CHAN9         0x00000200  // System DMA Channel 9
#define PACDMP_VIC_LV2_SYSDMA_CHAN10        0x00000400  // System DMA Channel 10
#define PACDMP_VIC_LV2_SYSDMA_CHAN11        0x00000800  // System DMA Channel 11
#define PACDMP_VIC_LV2_SYSDMA_CHAN12        0x00001000  // System DMA Channel 12
#define PACDMP_VIC_LV2_SYSDMA_CHAN13        0x00002000  // System DMA Channel 13
#define PACDMP_VIC_LV2_SYSDMA_CHAN14        0x00004000  // System DMA Channel 14
#define PACDMP_VIC_LV2_SYSDMA_CHAN15        0x00008000  // System DMA Channel 15
#define PACDMP_VIC_LV2_SYSDMA_CHANALL       0x00010000  // All system DMA channel. This interrupt source bit will be triggered if any one
                                                        // system DMA channel issues an interrupt.
#define PACDMP_VIC_LV2_DSPDMA_CHAN0         0x00020000  // DSP DMA Channel 0
#define PACDMP_VIC_LV2_DSPDMA_CHAN1         0x00040000  // DSP DMA Channel 1
#define PACDMP_VIC_LV2_DSPDMA_CHAN2         0x00080000  // DSP DMA Channel 2
#define PACDMP_VIC_LV2_DSPDMA_CHAN3         0x00100000  // DSP DMA Channel 3
#define PACDMP_VIC_LV2_DSPDMA_CHANALL       0x00200000  // All DSP DMA channel. This interrupt source bit will be triggered if any one
                                                        // DSP DMA channel issues an interrupt.
#define PACDMP_VIC_LV2_I2C                  0x00400000  // I2C
#define PACDMP_VIC_LV2_SPI                  0x00800000  // SPI
#define PACDMP_VIC_LV2_UART                 0x01000000  // UART
#define PACDMP_VIC_LV2_TMR0                 0x02000000  // Timer 0
#define PACDMP_VIC_LV2_TMR1                 0x04000000  // Timer 1
#define PACDMP_VIC_LV2_TMR2                 0x08000000  // Timer 2
#define PACDMP_VIC_LV2_TMR3                 0x10000000  // Timer 3
#define PACDMP_VIC_LV2_TMR4                 0x20000000  // Timer 4
#define PACDMP_VIC_LV2_TMR5                 0x40000000  // Timer 5

#endif /* _PACDMP_VIC_REGS_H_ */
