#include <linux/kernel.h>
#include <linux/pci.h>
#include <linux/firmware.h>

#include "spct6100-main.h"
#include "spct6100-io.h"
#include "spct6100-firmware.h"

#ifdef ENABLE_REQUEST_FIRMWARE
struct firmware_properity {
  int run;
  int wait;
  unsigned int addr;
};

static char firmwarename[7][30] = {
  "spct6100_ddr_data.bin",
  "spct6100_ddr_text.bin",
  "spct6100_font.bin",
  "spct6100_osd_text.bin",
  "spct6100_tw2815_data.bin",
  "spct6100_h264codec_data.bin",
  "spct6100_all_text.bin"
};

static struct firmware_properity firmwarepro[7] = {
  {0, 0, 0x1E000000},
  {1, 1, 0x1C000000},
  {0, 0, 0x00100000},
  {1, 1, 0x1C000000},
  {0, 0, 0x00F11000},
  {0, 0, 0x1E008000},
  {0, 0, 0x00000000},
};

static const struct firmware *fw_entry[7];

static void
spct6100_syncdsp(void *dev)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  unsigned int udata32 = 0x8;
#define DSP_RUN 0x1
#define DSP_EXCEPTION 0x2
#define DSP_DEBUGGING 0x4
  while(1) {

    spct6100_get_reg(cur_dev, REG_DSP_CBUSY, (unsigned char *)&udata32, -1);

    if (udata32 == 0x0)
      break;
    msleep(100);
  }
}

int
download_firmware(void *dev)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  struct device *p_dev = (struct device *)&(cur_dev->pci->dev);
  int index, rc = 0;

  for (index = 0; index < 7; index++) {

    rc = request_firmware(&fw_entry[index],
        firmwarename[index],
        p_dev);
    if (rc != 0) {

      ERROR ("Firmware %s reading failed with errno %d\n",
             firmwarename[index],
             rc);
      break;
    } else {
      INFO ("Firmware %s reading successed\n",
            firmwarename[index]);
    }

    spct6100_pacdmp_write_bar3 (cur_dev, firmwarepro[index].addr);
    spct6100_writemem(cur_dev, 0, (unsigned char *)fw_entry[index]->data,
                      fw_entry[index]->size);

    if (firmwarepro[index].run)
      spct6100_enabledevice(cur_dev, firmwarepro[index].addr);
    if (firmwarepro[index].wait)
      spct6100_syncdsp(cur_dev);

    release_firmware(fw_entry[index]);
  }
  return rc;
}
#else /* ENABLE_REQUEST_FIRMWARE */
int
download_firmware(void *dev)
{
  return -1;
}
#endif /* ENABLE_REQUEST_FIRMWARE */
