#ifndef SPCT6100_MAIN_H
#define SPCT6100_MAIN_H

#include <linux/version.h>
#include <linux/interrupt.h>
/* For V4L2 Layer */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
#include <media/v4l2-ioctl.h>
#endif
#include <linux/videodev2.h>
#include <media/v4l2-common.h>

#include "spct6100-packet-define.h"
#include "spct6100-utils.h" // for NUM_V4L2_CID

#define NR_CARDS               8
#define NR_CH                  4

/* The core device structure */
typedef struct SPCT6100_DEV
{
  /* master_buf_logic must be the first field */
  unsigned char            *master_buf_logic;
  unsigned int             index;
  char                     name[30];

  /* pci device & i/o  */
  struct pci_dev           *pci;
  unsigned char            *mmio_vaddr;
  unsigned int             mmio_size;
  unsigned short           id;

  /* For locking */
  struct semaphore         sem;
  struct semaphore         status_read_sem;
  struct semaphore         status_write_sem;
  spinlock_t               reg_lock;

  /* For dma */
  unsigned char            *master_buf_phy;

  /* For spct6100_compress_only */
  unsigned long            master_free_pages;

  /* For V4L2 midware */
  enum v4l2_field          field;
  struct spct6100_fmt*     fmt;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
  v4l2_std_id              current_norm;
#endif

#if 0//shengguo
  int volatile sps_header  [NR_CARDS];
#endif

  struct video_device      *v4l2_live;
  struct video_device      *v4l2_compress;
  unsigned int             live_users;
  unsigned int             compress_users;

  unsigned int status;
  unsigned int stream_status;

  /* for proc system */
  unsigned int l_irq[5];
  unsigned int c_irq[5];
  unsigned int l_lost_irq[5];
  unsigned int c_lost_irq[5];
  struct timeval l_irq_stime[5];
  struct timeval c_irq_stime[5];
  struct timeval l_irq_lost_stime[5];
  struct timeval c_irq_lost_stime[5];
  struct timeval l_irq_lost_htime[5];
  struct timeval c_irq_lost_htime[5];
  unsigned int pci_read_len;
  unsigned int pci_write_len;
  unsigned int pci_read_time;
  unsigned int pci_write_time;
  struct timer_list update_osd;
} spct6100_dev;

extern spct6100_dev g_spct6100_dev[NR_CARDS];
extern int g_device_num;
extern unsigned long spct6100_reserved_addr;

extern unsigned int qctl_current_values[NR_CARDS][NR_CH][NUM_V4L2_CID];
extern struct spct6100_eqset g_eqset[NR_CARDS][NR_CH];
extern struct spct6100_gopset g_gopset[NR_CARDS][NR_CH];
extern struct spct6100_osdset g_osdset[NR_CARDS][NR_CH];
extern struct spct6100_mosaic g_mosaic[NR_CARDS][NR_CH];
extern int user_mosaicset[NR_CARDS][NR_CH];
extern struct spct6100_cropset g_cropset[NR_CARDS][NR_CH];
extern int user_cropset[NR_CARDS][NR_CH];

extern int spct6100_gpio_mode;
extern int spct6100_preview_pci;
#ifdef __arm__
extern int spct6100_irq_shared;
#endif

extern int lstream_toggle[NR_CARDS];
extern int cstream_toggle[NR_CARDS];

#ifdef __arm__
#define SPCT6100_PRIVATE_SPACE  (0)
#define SPCT6100_DVR_RING_SZ    (0x300000)
#define SPCT6100_DVR_ORDER      (10)
#else
#define SPCT6100_PRIVATE_SPACE  (128*1024)
#define SPCT6100_DVR_RING_SZ    (0xC00000)
#define SPCT6100_DVR_ORDER      (12)
#endif
#define SPCT6100_DVR_ELEM_SZ    (0xC6000)
#define START_ADDRESS           0x1E000000
#define BOARD_STOP              0
#define BOARD_RUNNING           1
#define BOARD_STREAMOFF         2
#define BOARD_STREAMON          3

#ifdef __arm__
irqreturn_t
spct6100_interrupt(int irq, void *dev_id, struct pt_regs *regs);
#else
irqreturn_t
spct6100_interrupt(int irq, void *dev_id);
#endif

extern void
set_mosaic_area(void *mosaic, void *file, void *dev, int ch_id);

#endif /* SPCT6100_MAIN_H */
