//===- CommandPacket.h ----------------------*- For C/C++ -*-===//
//
//                     Command Packet Description
//
// This file is distributed under the GPLv3.
//
//===----------------------------------------------------------------------===//
//
// This file defines all command packet id and its detailed payload format.
//
//===----------------------------------------------------------------------===//

#ifndef PACKET_H
#define PACKET_H

/// All PACKET ID
enum packet_id {

  PID_NULL   = 0,     // Special Command
  PID_STATUS,         // Live/Compress open or close
  PID_STANDARD,       // card standard NTSC/PAL
  PID_BUE,            // The following four set the color properity
  PID_CONTRAST,
  PID_SATURATION,
  PID_BRIGHTNESS,
  PID_LSHOWSIZE,      // Live path resolution
  PID_CSHOWSIZE,      // Compress path resolution
  PID_CSPEED,         // Compress path encode speed
  PID_CQUALITY,       // Compress path encode quality
  PID_VLOST,          // Video Lost
  PID_REQIFRAME,      // Request I frame with SPS header
  PID_CQINIT,         // Command Queue init
  PID_START,          // Start board
  PID_STOP,           // Pause the board
  PID_RESTART,        // Restart the board
  PID_END,            // Stop board
  PID_MOSAIC,         // set mosaic area
  PID_MOTION,         // set motion detection
  PID_PING,           // Check if firmware is still working
  PID_CDATA,          // Compress data packet
  PID_ACK,            // Command ack packet
  PID_LPFS,           // Set live ring buffer size
  PID_LPBB,           // Set live ring buffer offset
  PID_LDATA,          // Live data packet
  PID_NUM,
};

// This is the id in Win32
enum win32_packet_id {

 // Data source class
  WIN32_PID_V_INIT = 0,
  WIN32_PID_A_INIT,
  WIN32_PID_V_STD_SET,
  WIN32_PID_V_PRV_SET,
  WIN32_PID_V_CPRS_SET,
  WIN32_PID_V_CLR_SET,
  WIN32_PID_V_GET_CLR,
  WIN32_PID_V_LOST_ENAB,
  WIN32_PID_A_ENAB_SET,
  WIN32_PID_V_CROPPING,

  //Preview video class
  WIN32_PID_P_ENAB = 0x10,

  //Encoder video class
  WIN32_PID_E_V_INIT = 0x20,
  WIN32_PID_E_V_ENAB,
  WIN32_PID_E_V_FPS_SET,
  WIN32_PID_E_V_RSL_SET,
  WIN32_PID_E_V_GOP_SET,
  WIN32_PID_E_V_RATE_CTL_TYPE_SET,
  WIN32_PID_E_V_RATE_CTL_QP_SET,
  WIN32_PID_E_V_RATE_CTL_BR_SET,
  WIN32_PID_E_V_REQ_I,
  WIN32_PID_E_V_REQ_SEQ,

  //Encoder audio class
  WIN32_PID_E_A_INIT = 0x30,
  WIN32_PID_E_A_ENAB,

  //OSD & Mask class
  WIN32_PID_OSD_SET = 0x40,
  WIN32_PID_OSD_ENAB,
  WIN32_PID_MASK_SET,
  WIN32_PID_MASK_ENAB,

  //Motion detection class
  WIN32_PID_MDECT_SET,
  WIN32_PID_MDECT_ENAB,

  //Miscellaneous
  WIN32_PID_WATDOG_SET = 0x50,
  WIN32_PID_WATDOG_ENAB,
  WIN32_PID_PING,
  WIN32_PID_ACK,
  WIN32_PID_GET_DEBUG,
  WIN32_PID_SND_FEBUG,
  WIN32_PID_W_EEP,
  WIN32_PID_R_EEP,
  WIN32_PID_SET_GPIO_D,
  WIN32_PID_W_GPIO,
  WIN32_PID_R_GPIO,

  //Compress class
  WIN32_PID_V_CPRS = 0x80,
  WIN32_PID_A_CPRS,
  WIN32_PID_V_RAW,
  WIN32_PID_A_RAW,

   //Miscellaneous
  WIN32_PID_ACK_FROM_DSP = 0x90,
  WIN32_PID_V_LOST_STATUS,
  WIN32_PID_MDECT_RESULT,

  //System class
  WIN32_PID_S_INIT = 0xF0,
  WIN32_PID_S_START,
  WIN32_PID_S_STOP,
  WIN32_PID_S_END,
  WIN32_PID_S_TIMER,
};


enum ptype {
  PAYLOAD_TYPE_NULL = 0,  // special type
  PAYLOAD_TYPE_1,     // general usage
  PAYLOAD_TYPE_2,     // for mosaic area
  PAYLOAD_TYPE_3,     // for motion detection
};

struct pheader {

  /// used as a mutex between driver and firmware
  /// 0 - host side
  /// 1 - dsp  side
  u16  owner;

  /// serial number
  u16 serialnum;

  /// command packet id
  u16 id;

  /// flag description:
  ///   bit 0 - 1:  read or write or action
  ///   bit 2 - 3: channel or card command
  u16  flag;

  /// Common field, depends on the flag
  u16  common;

  /// reserved for futher expansion
  u16 reserved;

  /// size of payload (byte)
  u32  payloadsize;

};

//#define DATA_OWNER_BASE 0x01C02000  //1E003DB0
//#define FW_RETURN_ADDR  0x1E00ADA4
#define FW_RETURN_ADDR  0x1E00EE00
#define DATA_OWNER_BASE 0x1E003DB0
#define C_OWNER_DRIVER   0
#define C_OWNER_FIRMWARE 1

#define D_OWNER_DRIVER 0
#define D_OWNER_FIRMWARE 1

#define PACKET_FLAG_DEFAULT        0x0
#define PACKET_FLAG_TYPE_BASE      0
#define PACKET_FLAG_TYPE_LENGTH    2
#define PACKET_TYPE_READ           0x1
#define PACKET_TYPE_WRITE          0x2
#define PACKET_TYPE_ACTION         0x3

#define PACKET_FLAG_SCOPE_LENGTH   2
#define PACKET_FLAG_SCOPE_BASE     2
#define PACKET_SCOPE_CARD          0x4
#define PACKET_SCOPE_CHANNEL       0x8

#define ACK_FAIL        -1
#define ACK_OK          0
#define ACK_NOSUPPORT   1
#define ACK_PARTIALFAIL 2

#define PAYLOAD_EMPTY  0
#define PAYLOAD_4BYTES 4
#define PAYLOAD_8BYTES 8
#define PAYLOAD_48BYTES 48

#define PAYLOAD_VARSIZE -1

#define DEFAULT_FLAG   0x0

#define NONE_V4L2_INDEX -1

#define NONE_REG_INDEX -1

#define COMMAND_WITH_ACK 1
#define COMMAND_WITHOUT_ACK 0

#define TEST_PATTERN_HAND_GEN 0
#define TEST_PATTERN_AUTO_GEN 1

#define SHOULD_CHECK 1
#define SHOULD_NOT_CHECK 0

#define AV_HSIZE()    88

#define AV_SIZE(x)     (*(unsigned int *)((unsigned long)(x) + 0x0))
#define AV_TOTAL(x)    (*(unsigned int *)((unsigned long)(x) + 0x4))
#define AV_FLAGS(x)    (*(unsigned int *)((unsigned long)(x) + 0xC))
#define AV_VTSTAMP(x)  (*(unsigned int *)((unsigned long)(x) + 0x10))
///EXPOSE
#define AV_VLEN(x)     (*(unsigned int *)((unsigned long)(x) + 0x14))
#define AV_ALEN(x)     (*(unsigned int *)((unsigned long)(x) + 0x18))
///ENDEXPOSE
#define AV_RESERV0(x)  (*(unsigned int *)((unsigned long)(x) + 0x1C))
///EXPOSE
#define AV_VOFFSET(x)  (*(unsigned int *)((unsigned long)(x) + 0x20))
#define AV_AOFFSET(x)  (*(unsigned int *)((unsigned long)(x) + 0x24))
///ENDEXPOSE
#define AV_RESERV1(x)  (*(unsigned int *)((unsigned long)(x) + 0x28))
#define AV_VLOSTEN(x)  (*(unsigned char *)((unsigned long)(x) + 0x2C))
#define AV_MDEN(x)     (*(unsigned char *)((unsigned long)(x) + 0x2D))
#define AV_VENCODEN(x) (*(unsigned char *)((unsigned long)(x) + 0x2E))
#define AV_AENCODEN(x) (*(unsigned char *)((unsigned long)(x) + 0x2F))
#define AV_VLOSTV(x)   (*(unsigned int *)((unsigned long)(x) + 0x30))
#define AV_MDLEN(x)    (*(unsigned int *)((unsigned long)(x) + 0x34))
#define AV_ATSTAMP(x)  (*(unsigned int *)((unsigned long)(x) + 0x38))

struct geometry {
  u16 left;
  u16 top;
  u16 width;
  u16 height;
};


/// Type 1 description:
///    General usage
struct payload_type_1 {
  u32 value;
};

/// Type 2 description:
///    This type of command is for mosaic area setting only
struct payload_type_2 {
  struct geometry area;
};

/// Type 3 description:
///    This type of command is for motion detection setting only
struct payload_type_3 {
  u32 areanum;
  u16 sensitive;
  u16 threshold;
  struct geometry area[5];
};


#define DWORD unsigned int
#define SHORT unsigned short

typedef struct _SPCT_PKt_DATA {
  union {
    /*
    for test version
    struct {
        SHORT PaSetting_Type;
          SHORT PaFormat;
        DWORD Adc_Table_Base;
    } Board_Init;

    struct {
        DWORD VConetSet_Base;
        SHORT VConet_Count;
    } Video_Connect_Init;

    struct {
        DWORD VCmsSet_Base;
        SHORT VCms_Count;
    } Video_Compress_Init;

    struct {
        DWORD VEnCod_Base;
        SHORT VEnCod_count;
    } Video_Encoder_Init;
    */
    //[pen 09.10.20]
    struct {
        DWORD ADC_Table_Size;
        DWORD ADC_Table_Base;
    }S_INIT;

    struct {
        DWORD Standard;
    }V_INIT;

    struct {
        DWORD Resolution;
    }V_CPRS_SET;

    struct {
        SHORT endian;
        SHORT resolution;
        SHORT DMA_threshold;
        SHORT path;
        DWORD ringbuffer_offset;
        DWORD ringbuffer_size;
    }V_PRV_SET;

    struct {
        DWORD enable;
    }P_ENAB;

    struct {
        DWORD Settings_Base;
        SHORT paValidVideoEncoderCount;
        SHORT pnumctrl;
    }E_V_INIT;

    struct{
        DWORD enable;
    }E_V_ENAB;

    struct{
        DWORD typesetting;// 0:CBR,1:VBR-R,2:VBR-Q
    }E_V_RATE_CTL_TYPE_SET;

    struct{
        DWORD maxbitrate;// Max. Bit Rate
        DWORD minbitrate;// Min. Bit Rate
    }E_V_RATE_CTL_BR_SET;

    struct{
        DWORD QP_I;
        DWORD QP_P;
        DWORD QP_B;
    }E_V_RATE_CTL_QP_SET;

    struct{
        DWORD GOP;// high part : GOPB, low part : GOPP (for test, changeable)
    }E_V_GOP_SET;

    struct{
       DWORD I_Counter;
    }E_V_REQ_I;

    struct{
       DWORD enable;
    }E_A_ENAB;

    struct{
       DWORD enable;
    }A_ENAB_SET;

    struct{
       DWORD MD_thold;
       DWORD MD_interval;
    }MDECT_SET;

    struct{
       DWORD enable;
    }MDECT_ENAB;

    struct{
        DWORD TotalMaskNum;
        // See the following field as a array
        // could accelerate process
        DWORD UpL_Mask0;
        DWORD DownR_Mask0;
        DWORD UpL_Mask1;
        DWORD DownR_Mask1;
        DWORD UpL_Mask2;
        DWORD DownR_Mask2;
        DWORD UpL_Mask3;
        DWORD DownR_Mask3;
        DWORD UpL_Mask4;
        DWORD DownR_Mask4;
        DWORD UpL_Mask5;
        DWORD DownR_Mask5;
    }MASK_SET;

    struct{
        DWORD MaskEnable[6];
    }MASK_ENAB;

    struct{
        DWORD Osd0Enable;
        DWORD Osd1Enable;
    }OSD_ENAB;

    struct {
        DWORD translucent;
        DWORD Osd0position;
        char  str[20];
        DWORD Osd1position;
        DWORD year;
        DWORD month;
        DWORD secday;
        DWORD minsec;
        DWORD hr;
    }OSD_SET;

    struct{
        SHORT hue;
        SHORT contrast;
        SHORT saturation;
        SHORT brightness;
    }V_CLR_SET;

    struct{
        SHORT chan;
        SHORT reserved;
        SHORT contrast;
        SHORT saturation;
        SHORT brightness;
    }V_GET_CLR;

    struct {
      DWORD format;
      DWORD ADCTableBaseAddr;
    }V_STD_SET;

    struct{
        DWORD enable;
    }V_LOST_ENAB;

    struct{
        DWORD timeout;
    }WATDOG_SET;

    struct{
        DWORD enable;
    }WATDOG_ENAB;

    struct {
      DWORD md_thold;
      DWORD md_inter;
    }MD_SETTING;

    struct {
      DWORD enable;
    }MD_ENABLE;

    struct {
      DWORD ymd;
      DWORD hms;
    }S_TIMER;

    struct {
      DWORD count;
    }REQ_ISEQ;

    struct {
      SHORT crop;
      SHORT v_delay;
      SHORT v_active;
      SHORT h_delay;
      SHORT h_active;
    }V_CROP;

    struct {
      DWORD fps;
    }E_V_FPS;
  };
} SPCT_PAYLOAD, *PSPCT_PAYLOAD;

typedef struct _SPCT_PKt_DATA_EXTERN {
  union {
    SPCT_PAYLOAD payload;
#if 1
    struct{
        DWORD data[16];
    }W_EEP;

    struct{
        DWORD src;
        DWORD len;
    }R_EEP;
#else

    struct {
      SHORT deviceid;
      SHORT dst;
      DWORD len;
      char data[16];
    }W_EEP;

    struct{
        SHORT deviceid;
        SHORT reserver0;
        DWORD reserver1;
        DWORD src;
        DWORD len;
    }R_EEP;
#endif

    struct{
        SHORT pin;
        SHORT dir;
    }SET_GPIO_D;

    struct{
        SHORT pin;
        SHORT value;
    }W_GPIO;

    struct{
        DWORD pin;
    }R_GPIO;

    struct{
        DWORD reserved1;//0x14
        DWORD reserved2;//0x14
        DWORD reserved3;//0x14
        DWORD reserved4;//0x14

        SHORT I2CdeviceID;//0x10
        SHORT reserved5;//0x12
        DWORD reserved6;//0x14
       DWORD src;//0x18
        DWORD len;//0x1C
    }R_I2C;

    struct{
        SHORT I2CdeviceID;
        SHORT dst;
        SHORT len;
        DWORD data1;
        DWORD data2;
        DWORD data3;
        DWORD data4;
    }W_I2C;
  };
} SPCT_PAYLOAD_EXTERN, *PSPCT_PAYLOAD_EXTERN;


///EXPOSE
//some ioctl structure type
struct spct6100_eeprom {
  unsigned int deviceid;
  unsigned int addr;
  unsigned int length;
  void *buf;
};

struct spct6100_gpio {
  unsigned int pinid;
  unsigned int value;
};

struct spct6100_osdset {
  unsigned int tran;
  unsigned int acolor;
  unsigned int vscale;
  unsigned int hscale;
  unsigned int timer_enable;
  unsigned int osd_enable;
  unsigned int osd_pos_x;
  unsigned int osd_pos_y;
  char str[16];
};

struct spct6100_gopset {
  unsigned int gopp;
  unsigned int gopb;
};

struct spct6100_watchset {
  unsigned int timeout;
};

struct spct6100_mosaic {
  unsigned int num;
  unsigned int x[6];
  unsigned int y[6];
  unsigned int w[6];
  unsigned int h[6];
};

struct spct6100_eqset {
  unsigned int type;
  unsigned int minbitrate;
  unsigned int maxbitrate;
  unsigned int qpi;
  unsigned int qpb;
  unsigned int qpp;
};

struct spct6100_cropset {
  unsigned short start_x;
  unsigned short start_y;
  unsigned short width;
  unsigned short height;
};

///ENDEXPOSE
#endif /* PACKET_H */
