/*
 * =====================================================================================
 *
 *       Filename:  PacketProperity.h
 *
 *    Description:  All Packet Properity
 *
 *        Version:  1.0
 *        Created:  05/22/09 15:09:14 CST
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Guang-Yuan Wang (), wangjiong@sunnorth.com.cn
 *        Company:  Sunplus Core Technology
 *
 * =====================================================================================
 */

/// FORMAT:
///(PID,
///    PayloadType,    ;; see spct6100-packet-define.h for detailed type define
///    PayloadSize,    ;; size of the payload
///    DefaultFlag,    ;; command packet flag, is it per channel or something else
///    V4L2 index,     ;; corresponding V4L2 index for this command packet
///    Reg_index,      ;; corresponding Register index for this command packet
///    Ack,            ;; whether firmware will give ack for this command packet
///    AutoPattern,    ;; test pattern should be auto-generated or hand-written
///    Min,            ;; if it is auto-generated pattern, then the minimum test value
///    Max,            ;; if it is auto-generated pattern, then the maximum test value
///    Step,           ;; if it is auto-generated pattern, then the step value
///    Desciption,     ;; ascii description of this command packet
///    Support check   ;; when driver start, whether this command should be check for support
///    Fake Result     ;; fake firmware testing result
///)

PACKET_INFO
(PID_NULL,
             PAYLOAD_TYPE_NULL,
             PAYLOAD_EMPTY,
             DEFAULT_FLAG,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_NULL",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_STATUS,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             STATUS_INDEX,
             REG_USTATUS,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             3,
             1,
             "PID_STATUS",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_STANDARD,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             MODE_INDEX,
             REG_UMODE,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             1,
             1,
             "PID_STANDARD",
             SHOULD_CHECK,
             ACK_FAIL
)

PACKET_INFO
(PID_BUE,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             HUE_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             255,
             50,
             "PID_BUE",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_CONTRAST,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             CONTRAST_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             255,
             50,
             "PID_CONTRAST",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_SATURATION,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             SATURATION_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             255,
             50,
             "PID_SATURATION",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_BRIGHTNESS,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             BRIGHT_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             255,
             50,
             "PID_BRIGHTNESS",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_LSHOWSIZE,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             3,
             1,
             "PID_LSHOWSIZE",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_CSHOWSIZE,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             CSHOWSIZE_INDEX,
             REG_USHOWSIZE,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             3,
             1,
             "PID_CSHOWSIZE",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_CSPEED,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             EENSPEED_INDEX,
             REG_UENCODERSPEED,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             30,
             10,
             "PID_CSPEED",
             SHOULD_CHECK,
             ACK_NOSUPPORT
)

PACKET_INFO
(PID_CQUALITY,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             EQUALITY_INDEX,
             REG_UQUALITY,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             10,
             3,
             "PID_CQUALITY",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_VLOST,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_VLOST",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_REQIFRAME,
             PAYLOAD_TYPE_1,
             PAYLOAD_EMPTY,
             PACKET_SCOPE_CARD,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_REQIFRAME",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_CQINIT,
             PAYLOAD_TYPE_1,
             PAYLOAD_EMPTY,
             PACKET_SCOPE_CARD,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_CQINIT",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_RESTART,
             PAYLOAD_TYPE_1,
             PAYLOAD_EMPTY,
             (PACKET_SCOPE_CARD|PACKET_TYPE_ACTION),
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_RESTART",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_START,
             PAYLOAD_TYPE_1,
             PAYLOAD_EMPTY,
             (PACKET_SCOPE_CARD|PACKET_TYPE_ACTION),
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_START",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_STOP,
             PAYLOAD_TYPE_1,
             PAYLOAD_EMPTY,
             (PACKET_SCOPE_CARD|PACKET_TYPE_ACTION),
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_STOP",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_END,
             PAYLAOD_TYPE_1,
             PAYLOAD_EMPTY,
             (PACKET_SCOPE_CARD|PACKET_TYPE_ACTION),
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_END",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_MOSAIC,
             PAYLOAD_TYPE_2,
             PAYLOAD_8BYTES,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_MOSAIC",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_MOTION,
             PAYLOAD_TYPE_3,
             PAYLOAD_48BYTES,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_MOTION",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_PING,
             PAYLOAD_TYPE_1,
             PAYLOAD_4BYTES,
             PACKET_SCOPE_CARD,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITH_ACK,
             TEST_PATTERN_AUTO_GEN,
             0,
             0,
             0,
             "PID_PING",
             SHOULD_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_CDATA,
             PAYLOAD_TYPE_1,
             PAYLOAD_VARSIZE,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_CDATA",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_ACK,
             PAYLOAD_TYPE_1,
             PAYLOAD_VARSIZE,
             DEFAULT_FLAG,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_ACK",
             SHOULD_NOT_CHECK,
             ACK_OK
)

PACKET_INFO
(PID_LPFS,
            PAYLOAD_TYPE_1,
            PAYLOAD_4BYTES,
            PACKET_SCOPE_CHANNEL,
            LFRAMESIZE_INDEX,
            REG_ULPFS,
            COMMAND_WITH_ACK,
            TEST_PATTERN_HAND_GEN,
            0,
            0,
            0,
            "PID_LPFS",
            SHOULD_NOT_CHECK,
            ACK_OK
)

PACKET_INFO
(PID_LPBB,
            PAYLOAD_TYPE_1,
            PAYLOAD_4BYTES,
            PACKET_SCOPE_CHANNEL,
            LBUFBAR_INDEX,
            REG_ULPBB,
            COMMAND_WITH_ACK,
            TEST_PATTERN_HAND_GEN,
            0,
            0,
            0,
            "PID_LPBB",
            SHOULD_NOT_CHECK,
            ACK_OK
)

PACKET_INFO
(PID_LDATA,
             PAYLOAD_TYPE_1,
             PAYLOAD_VARSIZE,
             PACKET_SCOPE_CHANNEL,
             NONE_V4L2_INDEX,
             NONE_REG_INDEX,
             COMMAND_WITHOUT_ACK,
             TEST_PATTERN_HAND_GEN,
             0,
             0,
             0,
             "PID_LDATA",
             SHOULD_NOT_CHECK,
             ACK_OK
)



#undef PACKET_INFO
