#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <asm/irq.h>
#include <linux/pci.h>
#include "spctcommon/pacdmp-types.h"
#include "spctcommon/pacdmp-regs.h"
#include "spctcommon/spct6100cci.h"
#include "spct6100-trace.h"
#include "spct6100-packet-define.h"
#include "spct6100-utils.h"
#include "spct6100-main.h"
#include "spct6100-io.h"

#ifndef IRQF_SHARED
#define IRQF_SHARED SA_SHIRQ
#endif
#ifdef __arm__
#undef IRQF_SHARED
#define IRQF_SHARED SA_INTERRUPT
#endif

#define CHANNEL_ALL 4
#define CLEAR(x)  memset(&(x), 0, sizeof (x))
#define PING 0
#define PONG 1
#define PING_PONG_BUFFER_SIZE  (64*1024)
#define QCIF_NTSC_FRAME_SIZE   (176*120*2)
#define QCIF_PAL_FRAME_SIZE    (176*144*2)
#define CIF_NTSC_FRAME_SIZE    (352*240*2)
#define CIF_PAL_FRAME_SIZE     (352*288*2)
#define CIF4_NTSC_FRAME_SIZE   (704*480*2)
#define CIF4_PAL_FRAME_SIZE    (704*576*2)

#define SPCT6100_VP_BASE          0x1E000000
#define SPCT6100_VP_PORT0         0x1000
#define SPCT6100_VP_PORT1         0x2000
#define SPCT6100_VP_PORT2         0x3000
#define SPCT6100_VP_PORT3         0x4000
#define SPCT6100_VP_DPELIVBASE    0x50
#define SPCT6100_VP_DPELIVFRMSZ   0x54
#define SPCT6100_VP_DPEBUFPTR     0x58
#define SPCT6100_VP_DPEBUFUSG     0x5C
#define SPCT6100_VP_DPECTRL       0x40
#define SPCT6100_VP_FTHRESHOLD    0x84
#define SPCT6100_VP_LTHRESHOLD    0x88
#define SPCT6100_VP_CTHRESHOLD    0x8C
#define SPCT6100_VP_GTHRESHOLD    0x80
#define SPCT6100_VP_DTHRESHOLD    0x90

#define PACDMP_VPE_INT_EN           0x1C
#define PACDMP_PORT_CTRL_BASE       0x1000
#define OSD_HMS_SETTING_REG         0x110
#define OSD_TIMER_ADJUSTMENT_REG    0x104
#define OSD_HMS_READBLOCK_COUNTER   0x118
#define OSD_YMD_SETTING_MODE        0x120
#define OSD_YMD_SETTING_REG         0X124
#define OSD_HMS_INT                 0x140
#define OSD_YMD_INT                 0x160
#define OSD_WINDOW_0_COLOR_REG      0x40
#define OSD_WINDOW_1_COLOR_REG      0x50
#define OSD_WINDOW_1_SIZE_REG       0x54
#define OSD_WINDOW_1_POSITION_REG   0x58
#define OSD_WINDOW_1_FONT_REG       0x5C
#define OSD_ENABLE_CONTROL_REG      0x00


unsigned int osd_color_table[NR_COLOR] = {
  0xFFFFFFFF,
  0x00000400,
  0x00000500,
  0x00000600,
  0x00000700,
  0x00000800,
  0x00000900,
  0x00000A00,
  0x00000B00,
  0x00000C00,
  0x00000D00,
  0x00000E00,
  0x00000F00,
  0x00001000,
  0x00001100,
  0x00001200,
  0x00001300,
  0x00001400,
  0x00001500,
  0x00001600,
  0x00001700,
  0x00001800,
  0x00001900,
  0x00001A00,
  0x00001B00,
  0x00001C00,
  0x00001D00,
  0x00001E00,
  0x00001F00,
  0x00002000,
  0x00002100,
  0x00002200,
  0x00002300,
  0x00002400,
  0x00002500,
  0x00002600,
  0x00002700,
  0x00002800,
  0x00002900,
  0x00002A00,
  0x00002B00,
};

struct spct6100_cmd_node {
  enum packet_id id;
  int serialnum;
  int recordvalue;
  int channel;
  int flag;
  struct spct6100_cmd_node *next;
};

extern struct queue_context fip[NR_CARDS];

extern int spct6100_cpacket_num;
static unsigned short packet_serialnum[NR_CARDS];
static struct spct6100_cmd_node *spct6100_cmd_node_list;
static int us_access_process;

#if 0 //shengguo
static unsigned short packet_flag_table[PID_NUM] = {
#define PACKET_INFO(a,b,c,d,e,f,g,h,i,j,k,l,m,n) d,
#include "spct6100-packet-info.h"
};
#endif

static unsigned int packet_ack_table[PID_NUM] = {
#define PACKET_INFO(a,b,c,d,e,f,g,h,i,j,k,l,m,n) g,
#include "spct6100-packet-info.h"
};

static unsigned int packet_payloadsize[PID_NUM] = {
#define PACKET_INFO(a,b,c,d,e,f,g,h,i,j,k,l,m,n) c,
#include "spct6100-packet-info.h"
};

unsigned long spct6100_cci_maskarea_addr[4][5] = {
  {
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[0].tParam.tMaskArea[0]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[0].tParam.tMaskArea[1]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[0].tParam.tMaskArea[2]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[0].tParam.tMaskArea[3]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[0].tParam.tMaskArea[4])
  },{
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[1].tParam.tMaskArea[0]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[1].tParam.tMaskArea[1]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[1].tParam.tMaskArea[2]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[1].tParam.tMaskArea[3]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[1].tParam.tMaskArea[4])
  },{
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[2].tParam.tMaskArea[0]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[2].tParam.tMaskArea[1]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[2].tParam.tMaskArea[2]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[2].tParam.tMaskArea[3]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[2].tParam.tMaskArea[4])
  },{
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[3].tParam.tMaskArea[0]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[3].tParam.tMaskArea[1]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[3].tParam.tMaskArea[2]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[3].tParam.tMaskArea[3]),
    (unsigned long)&(((struct tag_SPCT6100_ENV *)(SPCT6100_ENV_ADDR))->chanSet[3].tParam.tMaskArea[4])
  }
};

enum spct6100_reg_types {
  BASE,
  TWIN,
  BLOCK,
  NORMAL,
  UNKNOWN
};


#define ACTIVE   0x1
#define INACTIVE 0x2
#define DIRECT   0x4
#define INDIRECT 0x8

#define WITH_OUT_ACK 0
#define WITH_ACK     1

static unsigned long spct6100_reg_addr[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) b,
#include "spct6100-regs.h"
};

static int spct6100_reg_size[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) c,
#include "spct6100-regs.h"
};


static enum spct6100_reg_types spct6100_reg_type[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) d,
#include "spct6100-regs.h"
};

static int spct6100_reg_step[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) e,
#include "spct6100-regs.h"
};

static int spct6100_reg_status[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) f,
#include "spct6100-regs.h"
};

static int spct6100_reg2cmd[REG_NUM] = {
#define REG_INFO(a,b,c,d,e,f,g,h,i) g,
#include "spct6100-regs.h"
};

int pid_external_to_local(int input)
{
  switch (input) {
    case WIN32_PID_V_CPRS:
      return PID_CDATA;
    case WIN32_PID_V_RAW:
      return PID_LDATA;
    case WIN32_PID_ACK:
      return PID_ACK;
    case WIN32_PID_V_LOST_STATUS:
      return PID_VLOST;
    case WIN32_PID_MDECT_RESULT:
      return PID_MOTION;
    default:
      return input;
  }
}

int pid_local_to_external(int input)
{
  switch (input) {
    case PID_STANDARD:
      return WIN32_PID_V_STD_SET;
    case PID_START:
      return WIN32_PID_S_START;
    case PID_STOP:
      return WIN32_PID_S_STOP;
    case PID_END:
      return WIN32_PID_S_END;
    case PID_REQIFRAME:
      return WIN32_PID_E_V_REQ_I;
    case PID_PING:
      return WIN32_PID_PING;
    default:
      return WIN32_PID_S_END+1;
  }
}

void spct6100_get_reg(void *cur_dev, enum spct6100_reg_index index, unsigned char *data, int channel_id) {

  spct6100_dev *local_ptr = (spct6100_dev *)cur_dev;
  DMAVHEADER *temp_addr;
  switch (spct6100_reg_type[index]) {
    case NORMAL:
    case BLOCK:
      spct6100_readmem(cur_dev, spct6100_reg_addr[index]&0xFFFFFF, data, spct6100_reg_size[index]);
      break;
    case BASE:
      spct6100_readmem(cur_dev, (spct6100_reg_addr[index]+channel_id*spct6100_reg_step[index])&0xFFFFFF, data, spct6100_reg_size[index]);
      break;
    case TWIN:

      temp_addr = (DMAVHEADER *)(fip[local_ptr->index].dq_base + fip[local_ptr->index].dq_len*channel_id + 4*1024);

      switch (index) {
    case REG_AV_RESERVED0:
      *(unsigned int *)data = temp_addr->uReserved0;
      break;
    case REG_AV_FLAGS:
      *(unsigned int *)data = temp_addr->uFlags;
      break;
    case REG_AV_TOTALSIZE:
      *(unsigned int *)data = temp_addr->uTotalDataSize;
      break;
    case REG_AV_AUDIOLEN:
      *(unsigned int *)data = temp_addr->uAudioDataLen;
      break;
    case REG_AV_AUDIOOFF:
      *(unsigned int *)data = temp_addr->uAudioDataAddrOffset;
      break;
    case REG_AV_VIDEOLEN:
      *(unsigned int *)data = temp_addr->uVideoDataLen;
      break;
    case REG_AV_VIDEOOFF:
      *(unsigned int *)data = temp_addr->uVideoDataAddrOffset;
      break;
    case REG_AV_MASKSTATE0:
      *(unsigned int *)data = temp_addr->tMaskArea[0].uMaskState & 0x0000FFFF;
      break;
    case REG_AV_MASKSTATE1:
      *(unsigned int *)data = temp_addr->tMaskArea[1].uMaskState & 0x0000FFFF;
      break;
    case REG_AV_MASKSTATE2:
      *(unsigned int *)data = temp_addr->tMaskArea[2].uMaskState & 0x0000FFFF;
      break;
    case REG_AV_MASKSTATE3:
      *(unsigned int *)data = temp_addr->tMaskArea[3].uMaskState & 0x0000FFFF;
      break;
    case REG_AV_MASKSTATE4:
      *(unsigned int *)data = temp_addr->tMaskArea[4].uMaskState & 0x0000FFFF;
      break;
    default:
      break;
      }
      break;

    default:
      ERROR ("Function spct6100_get_reg failed for register %d\n", index);
      break;
  }
}

int remove_cmd_node(unsigned short serialnum, enum packet_id *ackid, int *ackchannel)
{
  int recordvalue = -1;
  struct spct6100_cmd_node *prev_p, *next_p, *current_p;
  prev_p=current_p=next_p=spct6100_cmd_node_list;

  if (spct6100_cmd_node_list == NULL) {
    ERROR ("Error occured when remove_cmd_node~\n");
    return -1;
  }


  if (spct6100_cmd_node_list->next == NULL) {
    if (spct6100_cmd_node_list->serialnum ==  serialnum) {
      recordvalue = spct6100_cmd_node_list->recordvalue;
      *ackid = spct6100_cmd_node_list->id;
      *ackchannel = spct6100_cmd_node_list->channel;
      set_bit(0, (void *)&(spct6100_cmd_node_list->flag));
    } else {
      INFO ("Removeing a empty command node list !\n");
    }
  } else if (spct6100_cmd_node_list->serialnum ==  serialnum) {
    next_p = spct6100_cmd_node_list->next;
    recordvalue = spct6100_cmd_node_list->recordvalue;
    *ackid = spct6100_cmd_node_list->id;
    *ackchannel = spct6100_cmd_node_list->channel;
    set_bit(0, (void *)&(spct6100_cmd_node_list->flag));
  } else {
    while (current_p != NULL) {
      if (current_p->serialnum == serialnum) {
        recordvalue = current_p->recordvalue;
        *ackid = current_p->id;
        *ackchannel = current_p->channel;
        set_bit(0, (void *)&(current_p->flag));
        break;
      } else {
        current_p = current_p->next;
      }
    }
  }

  /// No user process acess this list, so we can assure
  /// only the irq acess it, so we can do removeing action
  if (test_bit(0, (void *)&us_access_process) == 0) {
    if (spct6100_cmd_node_list->next == NULL) {
      if (test_bit(0, (void *)&(spct6100_cmd_node_list->flag)) == 1) {
        kfree(spct6100_cmd_node_list);
        spct6100_cmd_node_list = NULL;
        return recordvalue;
      }
    } else {
      while (spct6100_cmd_node_list != NULL
             && test_bit(0, (void *)&(spct6100_cmd_node_list->flag)) == 1) {
        next_p = spct6100_cmd_node_list->next;
        kfree(spct6100_cmd_node_list);
        spct6100_cmd_node_list = next_p;
      }

      if (spct6100_cmd_node_list == NULL)
        return recordvalue;
      else {
        prev_p = spct6100_cmd_node_list;
        current_p = spct6100_cmd_node_list->next;
        while (current_p != NULL) {
          next_p = current_p->next;
          if (test_bit(0, (void *)&(current_p->flag)) == 1) {
            kfree(current_p);
            prev_p->next = next_p;
            current_p = next_p;
          } else {
            prev_p = current_p;
            current_p = next_p;
          }
        }
      }
    }
  }

  return recordvalue;
}

/* unsigned int try_counter = 0;
 * unsigned int try_mul = 0;
 */
void insert_cmd_node(void *dev, enum packet_id id,
                     int payloadsize, int channel_id,
                     unsigned char *data, void *record_value)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  int card_id = cur_dev->index;
  unsigned char cpacket[CPACKET_LEN];
  unsigned long hostaddress = 0;
  unsigned long bar_offset;
  int command_queue_full;
  int retry, limit;
  int reset_bar3 = 0;

  if (id != WIN32_PID_V_INIT
      && id != WIN32_PID_E_V_INIT
      && id != WIN32_PID_V_CPRS_SET
      && id != WIN32_PID_V_PRV_SET
      && id != WIN32_PID_V_STD_SET
      && id != WIN32_PID_P_ENAB
      && id != WIN32_PID_A_INIT
      && id != WIN32_PID_A_ENAB_SET
      && id != WIN32_PID_E_A_INIT
      && id != WIN32_PID_E_A_ENAB
      && id != WIN32_PID_E_V_ENAB
      && id != WIN32_PID_E_V_RATE_CTL_TYPE_SET
      && id != WIN32_PID_E_V_RATE_CTL_BR_SET
      && id != WIN32_PID_E_V_RATE_CTL_QP_SET
      && id != WIN32_PID_S_TIMER
      && id != WIN32_PID_E_V_FPS_SET
      && id != WIN32_PID_W_EEP
      && id != WIN32_PID_R_EEP
      && id != WIN32_PID_V_GET_CLR
      && id != WIN32_PID_SET_GPIO_D
      && id != WIN32_PID_W_GPIO
      && id != WIN32_PID_E_V_GOP_SET
      && id != WIN32_PID_E_V_REQ_I
      && id != WIN32_PID_MASK_SET
      && id != WIN32_PID_V_LOST_ENAB
      && id != WIN32_PID_MDECT_SET
      && id != WIN32_PID_MDECT_ENAB
      && id != WIN32_PID_MDECT_RESULT
      && id != WIN32_PID_OSD_SET
      && id != WIN32_PID_MASK_ENAB
      && id != WIN32_PID_OSD_ENAB
      && id != WIN32_PID_V_CLR_SET
      && id != WIN32_PID_PING
      && id != WIN32_PID_ACK // redundant, no need
      && id != WIN32_PID_S_INIT
      && id != WIN32_PID_S_START
      && id != WIN32_PID_S_STOP
      && id != WIN32_PID_S_END
	  && id != WIN32_PID_V_CROPPING)
    return;

#if 0
  if (packet_ack_table[id] != WITH_OUT_ACK) {
    set_bit(0, (void *)&us_access_process);
    if (spct6100_cmd_node_list == NULL) {
      temp_node = (struct spct6100_cmd_node *)
                  kzalloc(sizeof(struct spct6100_cmd_node), GFP_KERNEL);
      temp_node->id = id;
      if (record_value != NULL) {
        temp_node->recordvalue = *((int *)record_value);
      }
      temp_node->serialnum = packet_serialnum[card_id] - 1;
      temp_node->channel = channel_id;
      temp_node->next = NULL;
      spct6100_cmd_node_list = temp_node;
    } else {
      th = spct6100_cmd_node_list;
      while (th->next != NULL)
        th = th->next;

      temp_node = (struct spct6100_cmd_node *)
                  kzalloc(sizeof(struct spct6100_cmd_node), GFP_KERNEL);
      temp_node->id = id;
      if (record_value != NULL) {
        temp_node->recordvalue = *((int *)record_value);
      }
      temp_node->serialnum = packet_serialnum[card_id] - 1;
      temp_node->channel = channel_id;
      temp_node->next = NULL;
      th->next = temp_node;
    }
    clear_bit(0, (void *)&us_access_process);
  }
#endif

  if (cur_dev->stream_status != BOARD_STREAMON && id != WIN32_PID_S_END) {
    ERROR ("[%d]error: insert cmd %d before BOARD_STREAMON\n", card_id, id);
  }

  hostaddress = (unsigned long)cpacket;
  if (fip[card_id].cq_base >= 0x1E000000 + 16*1024*1024
      || fip[card_id].cq_base < 0x1E000000) {
    spct6100_pacdmp_write_bar3(cur_dev, fip[card_id].cq_base);
    bar_offset = fip[card_id].cur_cq_pos - fip[card_id].cq_base;
    reset_bar3 = 1;
  } else {
    bar_offset = fip[card_id].cur_cq_pos - 0x1E000000;
  }

  if (id == WIN32_PID_E_V_REQ_I) { /* REQIFrame. */
    limit = 0;
  } else {
    limit = 20;
  }

  retry = 0;
  command_queue_full = 1;
  while (1) {
    spct6100_readmem(cur_dev, bar_offset, cpacket, 4);
    if (CPACKET_OWNER(hostaddress) == 0x86) {
      command_queue_full = 0;
      break;
    }
    if (retry < limit) {
      retry++;
      msleep(50);
    } else {
      break;
    }
  }

  if (command_queue_full) {
    if (reset_bar3) {
      spct6100_pacdmp_write_bar3(cur_dev, 0x1E000000);
    }
    ERROR ("Command Queue Full, cannot insert cmd: %d\n", id);
    return;
  }

  CPACKET_ID(hostaddress) = id;
  CPACKET_PAYLOADSIZE(hostaddress) = payloadsize;
  CPACKET_SERIAL(hostaddress) = packet_serialnum[card_id]++;
  //CPACKET_FLAG(hostaddress) = packet_flag_table[id];
  CPACKET_COMMON(hostaddress) = channel_id;

#if 1
  if (id == WIN32_PID_W_EEP) {
    CPACKET_RESERVED(hostaddress) = *(unsigned int *)record_value;
  }
#endif
#if 0
  if ((packet_flag_table[id] & PACKET_SCOPE_CHANNEL) && (channel_id != -1))
    CPACKET_COMMON(hostaddress) = channel_id;
#endif

  if (data != NULL && payloadsize > 0)
    memcpy((unsigned char *)(hostaddress+sizeof(struct pheader)), data, payloadsize);

  if (packet_ack_table[id] == WITH_OUT_ACK
        || id == WIN32_PID_S_INIT
        || id == WIN32_PID_A_INIT
        || id == WIN32_PID_A_ENAB_SET
        || id == WIN32_PID_E_A_INIT
        || id == WIN32_PID_E_A_ENAB
        || id == WIN32_PID_V_INIT
        || id == WIN32_PID_V_PRV_SET
        || id == WIN32_PID_V_STD_SET
        || id == WIN32_PID_P_ENAB
        || id == WIN32_PID_E_V_ENAB
        || id == WIN32_PID_V_CPRS_SET
        || id == WIN32_PID_E_V_INIT
        || id == WIN32_PID_E_V_GOP_SET
        || id == WIN32_PID_W_EEP
        || id == WIN32_PID_R_EEP
        || id == WIN32_PID_SET_GPIO_D
        || id == WIN32_PID_V_LOST_ENAB
        || id == WIN32_PID_S_TIMER
        || id == WIN32_PID_E_V_FPS_SET
        || id == WIN32_PID_V_GET_CLR
        || id == WIN32_PID_W_GPIO
        || id == WIN32_PID_E_V_RATE_CTL_TYPE_SET
        || id == WIN32_PID_E_V_RATE_CTL_BR_SET
        || id == WIN32_PID_E_V_RATE_CTL_QP_SET
        || id == WIN32_PID_E_V_GOP_SET
        || id == WIN32_PID_E_V_REQ_I
        || id == WIN32_PID_MASK_SET
        || id == WIN32_PID_OSD_SET
        || id == WIN32_PID_MASK_ENAB
        || id == WIN32_PID_OSD_ENAB
        || id == WIN32_PID_MDECT_RESULT
        || id == WIN32_PID_MDECT_ENAB
        || id == WIN32_PID_MDECT_SET
        || id == WIN32_PID_V_CLR_SET
        || id == WIN32_PID_S_START
        || id == WIN32_PID_S_STOP
        || id == WIN32_PID_S_END
		|| id == WIN32_PID_V_CROPPING) {

    INFO ("[%d]Channel: %d, Insert command: %d\n", card_id, channel_id, id);
    spct6100_writemem(cur_dev, bar_offset, (unsigned char *)cpacket, fip[card_id].cq_len);

    CPACKET_OWNER(hostaddress) = 0x61;
    spct6100_writemem(cur_dev, bar_offset, (unsigned char *)cpacket, 4);

    if (fip[card_id].cur_cq_pos + fip[card_id].cq_len != fip[card_id].cq_end) {
      fip[card_id].cur_cq_pos = fip[card_id].cur_cq_pos + fip[card_id].cq_len;
    } else {
      fip[card_id].cur_cq_pos = fip[card_id].cq_base;
    }
  } else {
    ERROR ("card %d: met an unknown packet id %d\n", card_id, id);
  }
  if (reset_bar3) {
    spct6100_pacdmp_write_bar3(cur_dev, 0x1E000000);
  }
}

#define CP(x) ((unsigned char *)(x))
void spct6100_set_reg(void *cur_dev, enum spct6100_reg_index index,
                      unsigned char *data, int channel_id, ...)
{
  unsigned int mask = 0;
#if 0
  unsigned int has_brightness = 0, brightness = 0;
  unsigned int has_contrast = 0, contrast = 0;
  unsigned int has_saturation = 0, saturation = 0;
  unsigned int has_hue = 0, hue = 0;
  unsigned int has_equality = 0, quality_type = 0;
  unsigned int has_eshowsize = 0, maxbitrate, minbitrate;
  unsigned int has_espeed = 0, qp_i, qp_b, qp_p;
  unsigned int card_id = ((spct6100_dev *)cur_dev)->index;
  unsigned int ch_id = channel_id;
#else
  unsigned int brightness = 0;
  unsigned int contrast = 0;
  unsigned int saturation = 0;
  unsigned int hue = 0;
  unsigned int quality_type = 0;
  unsigned int maxbitrate = 0, minbitrate = 0;
  unsigned int qp_i = 0, qp_b = 0, qp_p = 0;
#endif
  unsigned int payloadsize;
  va_list ap;
  void *record_value = NULL;
  SPCT_PAYLOAD payload, payload_alter;

  memset((void *)&payload, 0, sizeof(SPCT_PAYLOAD));
  memset((void *)&payload_alter, 0, sizeof(SPCT_PAYLOAD));

  if (spct6100_reg_status[index] & INACTIVE)
    return;

  va_start(ap, channel_id);

  if (index == REG_UCOLOR
      || index == REG_UQUALITY
      || packet_ack_table[spct6100_reg2cmd[index]] == WITH_ACK) {
    record_value = va_arg(ap, void *);
  }

  if (index == REG_UCOLOR) {
    mask = va_arg(ap, int);
    brightness = va_arg(ap, int);
    hue = va_arg(ap, int);
    contrast = va_arg(ap, int);
    saturation = va_arg(ap, int);
#if 0//shengguo
    has_brightness = mask & 0x00000001;
    has_contrast = mask & 0x00000002;
    has_saturation = mask & 0x00000004;
    has_hue = mask & 0x00000008;
#endif
  }

  if (index == REG_UQUALITY) {
    mask = va_arg(ap, int);
    quality_type = *(unsigned int *)data & 0x3;
    switch (quality_type) {
      case 0:
        minbitrate = (*(unsigned int *)data >> 2) & 0x3FFF;
        maxbitrate = (*(unsigned int *)data >> 16) & 0x3FFF;
        break;
      default:
        qp_i = (*(unsigned int *)data >> 2) & 0xFF;
        qp_b = (*(unsigned int *)data >> 10) & 0xFF;
        qp_p = (*(unsigned int *)data >> 18) & 0xFF;
        break;
    }
#if 0//shengguo
    has_equality = mask & 0x00000001;
    has_eshowsize = mask & 0x00000002;
    has_espeed = mask & 0x00000004;
#endif
  }
  va_end(ap);

  if (spct6100_reg_status[index] & INDIRECT) {

    if (spct6100_reg2cmd[index] == PID_NULL) {
      switch (index) {
        case REG_UMODE:
          payload.V_STD_SET.ADCTableBaseAddr = 0x03F10000;
          payload.V_STD_SET.format = *(unsigned int *)data;
          if (channel_id == CHANNEL_ALL) {
            for (index = 0; index < CHANNEL_ALL; index++) {
              insert_cmd_node(cur_dev, WIN32_PID_V_STD_SET, 0x8, index, (unsigned char *)&payload, NULL);
            }
          } else {
            insert_cmd_node(cur_dev, WIN32_PID_V_STD_SET, 0x8, channel_id, (unsigned char *)&payload, NULL);
          }
          break;


        case REG_UCOLOR:
#if 1
          payload.V_CLR_SET.hue = hue;//qctl_current_values[card_id][ch_id][HUE_INDEX];
          payload.V_CLR_SET.contrast = contrast; //qctl_current_values[card_id][ch_id][CONTRAST_INDEX];
          payload.V_CLR_SET.saturation = saturation; //qctl_current_values[card_id][ch_id][SATURATION_INDEX];
          payload.V_CLR_SET.brightness = brightness; //qctl_current_values[card_id][ch_id][BRIGHT_INDEX];

          if (channel_id == CHANNEL_ALL) {
            for (index = 0; index < CHANNEL_ALL; index++) {
              insert_cmd_node(cur_dev, WIN32_PID_V_CLR_SET, sizeof(payload.V_CLR_SET),
                              index, (unsigned char *)&payload, NULL);
            }
          } else {
            insert_cmd_node(cur_dev, WIN32_PID_V_CLR_SET, sizeof(payload.V_CLR_SET),
                            channel_id, (unsigned char *)&payload, NULL);
          }

#endif
          break;
        case REG_UQUALITY:

#if 1
          if (quality_type != 0 && quality_type != 2) {
              return;
          }
          payload.E_V_RATE_CTL_TYPE_SET.typesetting = quality_type;
          if (channel_id == CHANNEL_ALL) {
            for (index = 0; index < CHANNEL_ALL; index++)
              insert_cmd_node(cur_dev, WIN32_PID_E_V_RATE_CTL_TYPE_SET,
                  0x4, index, (unsigned char *)&payload, NULL);
          } else
              insert_cmd_node(cur_dev, WIN32_PID_E_V_RATE_CTL_TYPE_SET,
                  0x4, channel_id, (unsigned char *)&payload, NULL);

          if (quality_type == 0) {
            payload.E_V_RATE_CTL_BR_SET.minbitrate = minbitrate;
            payload.E_V_RATE_CTL_BR_SET.maxbitrate = maxbitrate;
            payloadsize = 8;
          } else {
            payload.E_V_RATE_CTL_QP_SET.QP_I = qp_i;
            payload.E_V_RATE_CTL_QP_SET.QP_P = qp_p;
            payload.E_V_RATE_CTL_QP_SET.QP_B = qp_b;
            payloadsize = 12;
          }

          if (channel_id == CHANNEL_ALL) {
            for (index = 0; index < CHANNEL_ALL; index++)
              insert_cmd_node(cur_dev,
                  (quality_type == 0)?WIN32_PID_E_V_RATE_CTL_BR_SET:WIN32_PID_E_V_RATE_CTL_QP_SET,
                  payloadsize, index, (unsigned char *)&payload, NULL);
          } else
            insert_cmd_node(cur_dev,
                (quality_type == 2)?WIN32_PID_E_V_RATE_CTL_QP_SET:WIN32_PID_E_V_RATE_CTL_BR_SET,
                payloadsize, channel_id, (unsigned char *)&payload, NULL);
#endif

          break;
        case REG_USTATUS:

#if 1
          if (*(unsigned int *)data == 3) {
            payload.P_ENAB.enable = 1;
            payload_alter.E_V_ENAB.enable = 1;
          } else if (*(unsigned int *)data == 1) {
            payload.P_ENAB.enable = 0;
            payload_alter.E_V_ENAB.enable = 1;
          } else if (*(unsigned int *)data == 2) {
            payload.P_ENAB.enable = 1;
            payload_alter.E_V_ENAB.enable = 0;
          }

          if (channel_id == CHANNEL_ALL)
            for (index = 0; index < CHANNEL_ALL; index++)
              insert_cmd_node(cur_dev, WIN32_PID_P_ENAB, 0x4, index, (unsigned char *)&payload, NULL);
          else
              insert_cmd_node(cur_dev, WIN32_PID_P_ENAB, 0x4, index, (unsigned char *)&payload, NULL);

          if (channel_id == CHANNEL_ALL)
            for (index = 0; index < CHANNEL_ALL; index++)
              insert_cmd_node(cur_dev, WIN32_PID_E_V_ENAB, 0x4, channel_id, (unsigned char *)&payload_alter, NULL);
          else
              insert_cmd_node(cur_dev, WIN32_PID_E_V_ENAB, 0x4, channel_id, (unsigned char *)&payload_alter, NULL);
#endif
          break;

        default:
          break;
      }
    } else {
      insert_cmd_node(cur_dev,
                      pid_local_to_external(spct6100_reg2cmd[index]),
                      packet_payloadsize[spct6100_reg2cmd[index]],
                      channel_id,
                      data,
                      record_value);
    }
    return;
  }

  switch (spct6100_reg_type[index]) {

    case NORMAL:
    case BLOCK:
      spct6100_writemem(cur_dev, spct6100_reg_addr[index]&0xFFFFFF, data, spct6100_reg_size[index]);
      break;
    case BASE:
      spct6100_writemem(cur_dev, (spct6100_reg_addr[index]+channel_id*spct6100_reg_step[index])&0xFFFFFF, data, spct6100_reg_size[index]);
      break;
    case TWIN:
      ERROR ("You should not write DMAVHeader\n");
      break;
    default:
      ERROR ("Function spct6100_set_reg failed for register %d\n", index);
      break;
  }
}

void
spct6100_calculate_year_month_day(time_t input_time,
                                  unsigned int *arg_year,
                                  unsigned int *arg_month,
                                  unsigned int *arg_day,
                                  unsigned int *arg_hour,
                                  unsigned int *arg_minute,
                                  unsigned int *arg_second)
{
  unsigned int year, month, day, hour, minute, second;
  int leap;
  int is_leap;
  unsigned int value = input_time;
  int month_table[2][12] = {
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
  };

  if (value >= 0x80000000 && value < 0x8017e880) {
    year = 1901;
    month = 12;
    second = value - 0x80000000 + 20*3600 + 45*60 + 52;
    day = second / 3600 / 24 + 13;
    hour = (second / 3600) % 24;
    minute = (second / 60) % 60;
    second = second % 60;
    goto _out;
  }

  second = value - 0x8017e880; // 1902-1-1 00:00:00
  day = second / 3600 / 24 + 1;
  year = (day / 366) + 1902;

  leap = (year - 1901) / 4;
  day -= (year - 1902) * 365 + leap;
  is_leap = !(year % 4);
  if (day > 365 + is_leap) {
    day -= 365 + is_leap;
    year++;
    is_leap = !(year % 4);
  }

  for (month = 0; month < 12; month++) {
    if (day <= month_table[is_leap][month]) {
      break;
    }
    day -= month_table[is_leap][month];
  }
  month++;

  hour = (second / 3600) % 24;
  minute = (second / 60) % 60;
  second = second % 60;

_out:
  *arg_year   = year;
  *arg_month  = month;
  *arg_day    = day;
  *arg_hour   = hour;
  *arg_minute = minute;
  *arg_second = second;
}

static int
spct6100_strlen(const char *input)
{
  int counter = 0;
  while (*input != '\0') {
    input++;
    counter++;
  }
  return counter;
}

void
spct6100_osd_str(char *dst, const char *src)
{
  char tmpstr[20]={0x20, 0x20, 0x20, 0x20,
                   0x20, 0x20, 0x20, 0x20,
                   0x20, 0x20, 0x20, 0x20,
                   0x20, 0x20, 0x20, 0x20,
                   0x30, 0x20, 0x20, 0x20};

  int index;
  int len = spct6100_strlen(src);

  memcpy (dst, tmpstr, sizeof (tmpstr));
  if (len > 15) {
      return;
  }

  memcpy (dst, src, len);
  dst[len] = 0x20;

  for (index = 0; index < len; index++) {
      if (dst[index] >= '0' && dst[index] <= '9') {
          dst[index] -= '0';
      }
  }
}

void
spct6100_enabledevice(void *dev, unsigned int addr)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  unsigned int udata32 = 0;

  spct6100_set_reg(cur_dev, REG_DSP_ICFLUSH, (unsigned char *)&udata32, -1);
  spct6100_set_reg(cur_dev, REG_DSP_ICSA, (unsigned char *)&addr, -1);
  udata32 = 0x100;
  spct6100_set_reg(cur_dev, REG_DSP_ICPS, (unsigned char *)&udata32, -1);
  spct6100_set_reg(cur_dev, REG_DSP_CPSA, (unsigned char *)&addr, -1);
  udata32 = 1;
  spct6100_set_reg(cur_dev, REG_DSP_CPS, (unsigned char *)&udata32, -1);
  udata32 = 0;
  spct6100_set_reg(cur_dev, REG_DSP_ICS, (unsigned char *)&udata32, -1);
  udata32 = 1;
  spct6100_set_reg(cur_dev, REG_DSP_ICDONE, (unsigned char *)&udata32, -1);
}

void spct6100_initenv(void *cur_dev, int compress_only, int compress_quality, int support_tvtime) {
  return;
}

#if 0
void
spct6100_osd_set_next_day(unsigned long arg)
{
  SPCT_PAYLOAD payload;
  struct timeval curymd;
  spct6100_dev *cur_dev = (spct6100_dev *)arg;
  int l_year, l_month, l_day, l_hour, l_minu, l_sec;
  do_gettimeofday(&curymd);
  spct6100_calculate_year_month_day(curymd.tv_sec + 24*3600, &l_year, &l_month, &l_day,
                                    &l_hour, &l_minu, &l_sec);
  l_year = ((l_year / 1000) << 12) + (((l_year / 100) % 10) << 8)
           + (((l_year / 10) % 10) << 4) + (l_year % 10);
  l_month = ((l_month / 10) << 4) + (l_month % 10);
  l_day = ((l_day / 10) << 4) + (l_day % 10);
  payload.S_TIMER.ymd = (l_year << 16) + (l_month << 8) + (l_day);
  insert_cmd_node(cur_dev, WIN32_PID_S_TIMER, sizeof(payload.S_TIMER), -1,
                  (unsigned char *)&payload, NULL);

  cur_dev->update_osd.expires = jiffies + 24 * 3600 * HZ;
  add_timer(&(cur_dev->update_osd));
}
#endif

void
spct6100_init_cmd(void *file, void *dev)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  SPCT_PAYLOAD payload;
  unsigned int index;
  unsigned int card_id = cur_dev->index;
  unsigned int l_year, l_month, l_day;
  unsigned int l_hour, l_minu, l_sec;

#ifdef IPCAMERA_FIRMWARE
  payload.S_INIT.ADC_Table_Size = 0x44;
#else
  payload.S_INIT.ADC_Table_Size = 0x3E0;
#endif
  payload.S_INIT.ADC_Table_Base = 0x03F10000;
  insert_cmd_node(cur_dev, WIN32_PID_S_INIT, sizeof(payload.S_INIT), -1, (unsigned char *)&payload, NULL);

  if (spct6100_gpio_mode) {
    return;
  }

  payload.V_STD_SET.ADCTableBaseAddr = 0x03F10000;
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.V_STD_SET.format = qctl_current_values[card_id][index][MODE_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_V_STD_SET, sizeof(payload.V_STD_SET),
                    index, (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    if (!user_cropset[card_id][index])
      continue;
    payload.V_CROP.crop =   ((g_cropset[card_id][index].start_y & 0x300) >> 2)
                          | ((g_cropset[card_id][index].height & 0x300) >> 4)
                          | ((g_cropset[card_id][index].start_x & 0x300) >> 6)
                          | ((g_cropset[card_id][index].width & 0x300) >> 8);
    payload.V_CROP.v_delay = g_cropset[card_id][index].start_y & 0xff;
    payload.V_CROP.v_active = g_cropset[card_id][index].height & 0xff;
    payload.V_CROP.h_delay = g_cropset[card_id][index].start_x & 0xff;
    payload.V_CROP.h_active = g_cropset[card_id][index].width & 0xff;
    insert_cmd_node(cur_dev, WIN32_PID_V_CROPPING, sizeof(payload.V_CROP), index,
                   (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    if (!user_mosaicset[card_id][index])
      continue;

    set_mosaic_area(&g_mosaic[card_id][index], file, cur_dev, index);
  }

  //0:NTSC, 1:PAL
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.V_INIT.Standard = qctl_current_values[card_id][index][MODE_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_V_INIT, sizeof(payload.V_INIT),
                    index, (unsigned char *)&payload, NULL);
  }

  insert_cmd_node(cur_dev, WIN32_PID_A_INIT, 0, -1, NULL, NULL);

  payload.A_ENAB_SET.enable = 1;
  insert_cmd_node(cur_dev, WIN32_PID_A_ENAB_SET, sizeof(payload.A_ENAB_SET),
                  -1, (unsigned char *)&payload, NULL);

  for (index = 0; index < CHANNEL_ALL; index++) {
    insert_cmd_node(cur_dev, WIN32_PID_E_A_INIT, 0, index, NULL, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.E_A_ENAB.enable = qctl_current_values[card_id][index][AUDIO_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_E_A_ENAB, sizeof(payload.E_A_ENAB),
                    index, (unsigned char *)&payload, NULL);
  }

  //0:4CIFF, 1:CIFF, 2:QCIFF, 3:2CIFF
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.V_CPRS_SET.Resolution =
        qctl_current_values[card_id][index][CSHOWSIZE_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_V_CPRS_SET, sizeof(payload.V_CPRS_SET), index, (unsigned char *)&payload, NULL);
  }

  payload.V_PRV_SET.endian = 0x0; // Endian 0:Little, 1:Big
  payload.V_PRV_SET.DMA_threshold = 0x8;// DMA Threshold
  spct6100_preview_pci = qctl_current_values[card_id][0][PREVIEW_INDEX];
  if (spct6100_preview_pci)
    payload.V_PRV_SET.path = 0x1; // Path 0:DDR, 1:PCI
  else
    payload.V_PRV_SET.path = 0x0; // Path 0:DDR, 1:PCI
  payload.V_PRV_SET.ringbuffer_size = SPCT6100_DVR_ELEM_SZ;// Ring Buffer Element Size
  for (index = 0; index < NR_CH; index++) {
    payload.V_PRV_SET.resolution =
      qctl_current_values[card_id][index][LSHOWSIZE_INDEX];//Resolution 0:4CIFF, 1:CIFF, 2:QCIFF, 3:2CIFF
    if (spct6100_preview_pci)
      payload.V_PRV_SET.ringbuffer_offset = SPCT6100_PRIVATE_SPACE + 4 * 0xA0000 + index * (3*SPCT6100_DVR_ELEM_SZ);// Ring Buffer Base Offset
    insert_cmd_node(cur_dev, WIN32_PID_V_PRV_SET, sizeof(payload.V_PRV_SET), index, (unsigned char *)&payload, NULL);
  }

  // Enable 0:Disable, 1:Enable
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.P_ENAB.enable =
      (qctl_current_values[card_id][index][STATUS_INDEX] & 0x2) == 0 ? 0 : 1;
    insert_cmd_node(cur_dev, WIN32_PID_P_ENAB, sizeof(payload.P_ENAB), index, (unsigned char *)&payload, NULL);
  }

  payload.E_V_INIT.Settings_Base = 0x1E00C900;// VideoEncoder_Settings_Base
  payload.E_V_INIT.paValidVideoEncoderCount = 0x4; // paValidVideoEncoderCount
  payload.E_V_INIT.pnumctrl = spct6100_cpacket_num; // paValidVideoEncoderCount
  insert_cmd_node(cur_dev, WIN32_PID_E_V_INIT, sizeof(payload.E_V_INIT), -1, (unsigned char *)&payload, NULL);

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.E_V_RATE_CTL_TYPE_SET.typesetting = g_eqset[card_id][index].type;
    insert_cmd_node(cur_dev, WIN32_PID_E_V_RATE_CTL_TYPE_SET, sizeof(payload.E_V_RATE_CTL_TYPE_SET), index,
        (unsigned char *)&payload, NULL);

    if (payload.E_V_RATE_CTL_TYPE_SET.typesetting == 0) {
      payload.E_V_RATE_CTL_BR_SET.minbitrate = g_eqset[card_id][index].minbitrate;
      payload.E_V_RATE_CTL_BR_SET.maxbitrate = g_eqset[card_id][index].maxbitrate;
      insert_cmd_node(cur_dev, WIN32_PID_E_V_RATE_CTL_BR_SET, sizeof(payload.E_V_RATE_CTL_BR_SET), index,
          (unsigned char *)&payload, NULL);
    } else if (payload.E_V_RATE_CTL_TYPE_SET.typesetting == 2) {
      payload.E_V_RATE_CTL_QP_SET.QP_I = g_eqset[card_id][index].qpi;
      payload.E_V_RATE_CTL_QP_SET.QP_B = g_eqset[card_id][index].qpb;
      payload.E_V_RATE_CTL_QP_SET.QP_P = g_eqset[card_id][index].qpp;
      insert_cmd_node(cur_dev, WIN32_PID_E_V_RATE_CTL_QP_SET, sizeof(payload.E_V_RATE_CTL_QP_SET), index,
          (unsigned char *)&payload, NULL);
    }
  }

  for (index = 0; index < NR_CH; index++) {
    payload.E_V_GOP_SET.GOP = (g_gopset[card_id][index].gopp & 0xFFFF)
                               | ((g_gopset[card_id][index].gopb & 0xFFFF) << 16);
    insert_cmd_node(cur_dev, WIN32_PID_E_V_GOP_SET,
                    sizeof(payload.E_V_GOP_SET), index,
                    (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.E_V_ENAB.enable =
      (qctl_current_values[card_id][index][STATUS_INDEX] & 0x1) == 0 ? 0 : 1;
    insert_cmd_node(cur_dev, WIN32_PID_E_V_ENAB, sizeof(payload.E_V_ENAB),
                    index, (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    if (g_osdset[card_id][index].timer_enable)
      payload.OSD_ENAB.Osd1Enable = 1;
    else
      payload.OSD_ENAB.Osd1Enable = 0;

    if (g_osdset[card_id][index].osd_enable)
      payload.OSD_ENAB.Osd0Enable = 1;
    else
      payload.OSD_ENAB.Osd0Enable = 1;

    insert_cmd_node(cur_dev, WIN32_PID_OSD_ENAB, sizeof(payload.OSD_ENAB),
                    index, (unsigned char *)&payload, NULL);

    payload.OSD_SET.translucent = osd_color_table[g_osdset[card_id][index].acolor];
    if (g_osdset[card_id][index].timer_enable) {
      payload.OSD_SET.Osd1position = 0x000a000a;// (osd1 position y, osd0 position x)
      payload.OSD_SET.year =   0x8C8D8E8F;//time1
      payload.OSD_SET.month =  0x2F8A8B2F;//time2
      payload.OSD_SET.secday = 0x850C8889;//time4
      payload.OSD_SET.minsec = 0x82833A84;//time5
      payload.OSD_SET.hr =     0x3080813A;
    }

    if (g_osdset[card_id][index].osd_enable) {
      payload.OSD_SET.Osd0position = ((g_osdset[card_id][index].osd_pos_y & 0x0000FFFF) << 16)
                                     | (g_osdset[card_id][index].osd_pos_x & 0x0000FFFF);
      spct6100_osd_str(payload.OSD_SET.str, g_osdset[card_id][index].str);
    } else {
      payload.OSD_SET.Osd0position = 0x000a01a0;//pos_y << 16 | pos_x;
      spct6100_osd_str(payload.OSD_SET.str, "dummydummydummydummy");
    }
    insert_cmd_node(cur_dev, WIN32_PID_OSD_SET, sizeof(payload.OSD_SET),
        index,  (unsigned char *)&payload, NULL);
  }

  spct6100_calculate_year_month_day(qctl_current_values[card_id][0][TIMESET_INDEX],
                                    &l_year, &l_month, &l_day,
                                    &l_hour, &l_minu, &l_sec);
  l_year = ((l_year / 1000) << 12) + (((l_year / 100) % 10) << 8)
           + (((l_year / 10) % 10) << 4) + (l_year % 10);
  l_month = ((l_month / 10) << 4) + (l_month % 10);
  l_day = ((l_day / 10) << 4) + (l_day % 10);
  payload.S_TIMER.ymd = (l_year << 16) + (l_month << 8) + (l_day);
  payload.S_TIMER.hms = (l_hour << 16) + (l_minu << 8) + (l_sec);
  payload.S_TIMER.hms |= 0x07000000;

  insert_cmd_node(cur_dev, WIN32_PID_S_TIMER, sizeof(payload.S_TIMER),
                  0, (unsigned char *)&payload, NULL);

  spct6100_calculate_year_month_day(qctl_current_values[card_id][0][TIMESET_INDEX] + 24*3600,
                                    &l_year, &l_month, &l_day,
                                    &l_hour, &l_minu, &l_sec);

  l_year = ((l_year / 1000) << 12) + (((l_year / 100) % 10) << 8)
           + (((l_year / 10) % 10) << 4) + (l_year % 10);
  l_month = ((l_month / 10) << 4) + (l_month % 10);
  l_day = ((l_day / 10) << 4) + (l_day % 10);
  payload.S_TIMER.ymd = (l_year << 16) + (l_month << 8) + (l_day);

  insert_cmd_node(cur_dev, WIN32_PID_S_TIMER, sizeof(payload.S_TIMER),
                  1, (unsigned char *)&payload, NULL);

#if 0
  init_timer(&(cur_dev->update_osd));
  cur_dev->update_osd.data = (unsigned long)cur_dev;
  cur_dev->update_osd.function = spct6100_osd_set_next_day;
  cur_dev->update_osd.expires = jiffies + (24*3600) * HZ;
  add_timer(&(cur_dev->update_osd));
#endif

  msleep(50);

#if 0
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.V_CLR_SET.hue = qctl_current_values[card_id][index][HUE_INDEX];
    payload.V_CLR_SET.contrast = qctl_current_values[card_id][index][CONTRAST_INDEX];
    payload.V_CLR_SET.saturation = qctl_current_values[card_id][index][SATURATION_INDEX];
    payload.V_CLR_SET.brightness = qctl_current_values[card_id][index][BRIGHT_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_V_CLR_SET, sizeof(payload.V_CLR_SET),
                    index, (unsigned char *)&payload, NULL);
  }
#endif

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.V_LOST_ENAB.enable = qctl_current_values[card_id][index][VLOSTEN_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_V_LOST_ENAB, sizeof(payload.V_LOST_ENAB),
                    index, (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.MD_SETTING.md_thold = qctl_current_values[card_id][index][MTHRESHOLD_INDEX];
    payload.MD_SETTING.md_inter = qctl_current_values[card_id][index][MINTERVAL_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_MDECT_SET, sizeof(payload.MD_SETTING),
                    index, (unsigned char *)&payload, NULL);
  }

  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.MD_ENABLE.enable = qctl_current_values[card_id][index][MOTION_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_MDECT_ENAB, sizeof(payload.MD_ENABLE),
                    index, (unsigned char *)&payload, NULL);
  }


#if 1
  for (index = 0; index < CHANNEL_ALL; index++) {
    payload.E_V_FPS.fps = qctl_current_values[card_id][index][EENSPEED_INDEX];
    insert_cmd_node(cur_dev, WIN32_PID_E_V_FPS_SET, sizeof(payload.E_V_FPS),
                    index, (unsigned char *)&payload, NULL);
  }
#endif

  insert_cmd_node(cur_dev, pid_local_to_external(PID_START), 0, -1, NULL, NULL);
}

int
spct6100_stop_board(void *dev)
{
#if 0 //shengguo
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  unsigned int tmp32 = 0xFFFFFFFF;
  unsigned int channel_index;

  if (cur_dev->status == BOARD_STOP)
    return 0;

  spct6100_set_reg(cur_dev, REG_INT_VIC1_ENABLE,
                   (unsigned char *)&tmp32, -1);
  spct6100_set_reg(cur_dev, REG_INT_VIC2_ENABLE,
                   (unsigned char *)&tmp32, -1);

  tmp32 = 0;
  spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP1,
                   (unsigned char *)&tmp32, -1);
  tmp32 = 0xcafe5477;
  spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP2,
                   (unsigned char *)&tmp32, -1);
  tmp32 = -1;
  msleep(100);

  for (channel_index = 0; channel_index < 4; channel_index++) {

    tmp32 = 0;
    spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP3,
                     (unsigned char *)&tmp32, channel_index);
    spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP4,
                     (unsigned char *)&tmp32, channel_index);
    msleep(10);
    spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP5,
                     (unsigned char *)&tmp32, channel_index);
  }

  tmp32 = 0x00000001;

  spct6100_set_reg(cur_dev, REG_STOPBOARD_STEP6,
                   (unsigned char *)&tmp32, -1);
  spct6100_get_reg(cur_dev, REG_DSP_CPSA,
                   (unsigned char *)&tmp32, -1);

  if(tmp32 != 0x1F001000) {

    ERROR ("Stop SPCT6100 EVB Board Failed!\n");
    return -EIO;
  } else {

    cur_dev->status = BOARD_STOP;
    INFO ("Stop SPCT6100 EVB Board Successed!\n");
  }
  return 0;
#else
  ERROR ("spct6100_stop_board: Not Implemented Yet.\n");
  return -EFAULT;
#endif
}

void
spct6100_dropframe(void *dev)
{
#if 0 //shengguo
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  int channel_index = 0;
  unsigned int threshold[5] = {0, 0, 0, 0, 0};

  threshold[0] = 500;
  threshold[1] = 300;
  threshold[2] = 18;
  threshold[3] = 8;
  threshold[4] = spct6100_drop_frame;

  for (channel_index = 0; channel_index < 4; channel_index++) {
    spct6100_set_reg(cur_dev, REG_VPE_FTHRESHOLD,
                     (unsigned char *)threshold, channel_index);
    spct6100_set_reg(cur_dev, REG_VPE_LTHRESHOLD,
                     (unsigned char *)(threshold+1), channel_index);
    spct6100_set_reg(cur_dev, REG_VPE_CTHRESHOLD,
                     (unsigned char *)(threshold+2), channel_index);
    spct6100_set_reg(cur_dev, REG_VPE_GTHRESHOLD,
                     (unsigned char *)(threshold+4), channel_index);
    spct6100_set_reg(cur_dev, REG_VPE_DTHRESHOLD,
                     (unsigned char *)(threshold+3), channel_index);
  }
#else
  ERROR ("spct6100_dropframe: Not Implemented Yet\n");
#endif
}

void
init_queue_table(void *dev)
{
  spct6100_dev *cur_dev = (spct6100_dev *)dev;
  unsigned int cpacket_len = CPACKET_LEN;
  unsigned int cpacket_default = CPACKET_DEFAULT_QBASE;
  unsigned int dpacket_num = DPACKET_NUM;
  unsigned int dpacket_base;

  dpacket_base = SPCT6100_PRIVATE_SPACE;
  spct6100_set_reg(cur_dev, REG_CQ_BASE,   CP(&cpacket_default), -1);
  spct6100_set_reg(cur_dev, REG_CQ_PKTLEN, CP(&cpacket_len), -1);
  spct6100_set_reg(cur_dev, REG_DQ_BASE,   CP(&dpacket_base), -1);
  spct6100_set_reg(cur_dev, REG_DQ_PKTNUM, CP(&dpacket_num), -1);
}

#define CUR_DEV (g_spct6100_dev[g_device_num])
int
spct6100_dvr_probe_common(void *dev)
{
  struct pci_dev *pdev = (struct pci_dev *)dev;
  u8 data8;
  u32 data32;
  int i, rc;
  void __iomem *mmio_vaddr;

  CUR_DEV.id = pdev->device;
  CUR_DEV.index = g_device_num;
  pci_set_drvdata (pdev, &CUR_DEV);

  /* The following for loop should be removed ? */
  for (i = 1; i < 6; i++) {
    if (pdev->resource[i].start != 0x0
        || (pdev->resource[i].flags & IORESOURCE_IO))
      continue;
    pdev->resource[i].start = pdev->resource[0].start;
    pdev->resource[i].end = pdev->resource[0].end;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
    pdev->resource[i].parent = pdev->resource[0].parent;
#endif
  }

  spin_lock_init(&(CUR_DEV.reg_lock));

  /* pci stuffs (init, irq, mmio, bus mastering... */
  rc = pci_enable_device(pdev);
  if (rc < 0) {

    ERROR ("%s: PCI enable failure\n", CUR_DEV.name);
    rc = -EIO;
    goto err_out;
  }
  CUR_DEV.pci = pdev;

  /* PCI MMIO region (BAR0) */
  rc = pci_request_region (pdev, 0, CUR_DEV.name);
  if (rc < 0) {

    dev_err (&pdev->dev, "%s: could not request PCI region.\n",
                                            CUR_DEV.name);
    goto err_out_disable_pci;
  }

  CUR_DEV.mmio_vaddr = pci_iomap (pdev, 0, 0);

  if (!(CUR_DEV.mmio_vaddr)) {

    dev_err (&pdev->dev, "%s: cannot remap MMIO memory, aborting.\n",
                                            CUR_DEV.name);
    rc = -EIO;
    goto err_out_release_res;
  }

  CUR_DEV.mmio_size = pci_resource_len (pdev, 0);

  //DEBUG ("region 0 size: %d b, %d Kb\n", CUR_DEV.mmio_size, CUR_DEV.mmio_size/1024);

  /* initial MMIO base address to SDRAM start address on EVB */
  spct6100_pacdmp_write_bar3(&CUR_DEV, START_ADDRESS);

  // PCI Bus Mastering stuff...
  pci_set_master (pdev);
  if (pci_set_dma_mask (pdev, DMA_32BIT_MASK)) {

    ERROR ("%s: Oops: no 32bit PCI DMA ???\n", CUR_DEV.name);
    rc = -EIO;
    goto err_out_unmap;
  }

  if (spct6100_reserved_addr == 0) {
    CUR_DEV.master_free_pages = __get_free_pages(GFP_KERNEL|GFP_ATOMIC, SPCT6100_DVR_ORDER);

    if (CUR_DEV.master_free_pages == 0) {
      ERROR ("Alloc dma buffer in kernel failed\n");
      rc = -EIO;
      goto err_out_unmap;
    }

    CUR_DEV.master_buf_phy = (unsigned char *)__pa(CUR_DEV.master_free_pages);
#ifdef __arm__
    CUR_DEV.master_buf_logic =
        ioremap_nocache((unsigned long)CUR_DEV.master_buf_phy,
                        (unsigned long)SPCT6100_DVR_RING_SZ);
    if (CUR_DEV.master_buf_logic == NULL) {
      ERROR ("ioremap_nocache failed\n");
      rc = -EIO;
      goto err_out_unmap;
    }
#else
    CUR_DEV.master_buf_logic = (unsigned char *)CUR_DEV.master_free_pages;
#endif
    memset_io(CUR_DEV.master_buf_logic, 0xCA,
              (unsigned long)SPCT6100_DVR_RING_SZ);
  } else {
    CUR_DEV.master_buf_phy = (unsigned char *)spct6100_reserved_addr
                             + g_device_num*SPCT6100_DVR_RING_SZ;

    CUR_DEV.master_buf_logic =
        ioremap_nocache((unsigned long)CUR_DEV.master_buf_phy,
                        (unsigned long)SPCT6100_DVR_RING_SZ);

    if (CUR_DEV.master_buf_logic == NULL) {
      ERROR ("ioremap_nocache failed\n");
      rc = -EIO;
      goto err_out_unmap;
    }

    memset_io(CUR_DEV.master_buf_logic, 0xCA,
              (unsigned long)SPCT6100_DVR_RING_SZ);
  }

  pci_write_config_dword(pdev, PCI_BASE_ADDRESS_4,
                         (u32)CUR_DEV.master_buf_phy);

  // Disable ALL VIC First
  mmio_vaddr = CUR_DEV.mmio_vaddr;
  data32 = 0xFFFFFFFF;
  spct6100_set_reg(&CUR_DEV, REG_INT_VIC1_ENABLE,
                   (unsigned char *)&data32, -1);
  spct6100_set_reg(&CUR_DEV, REG_INT_VIC2_ENABLE,
                   (unsigned char *)&data32, -1);

#ifdef __arm__
  for (i = 0; i < g_device_num; i++) {
    if (pdev->irq == g_spct6100_dev[i].pci->irq) {
      spct6100_irq_shared = 1;
      INFO ("%s: Share IRQ %d\n", CUR_DEV.name, pdev->irq);
      pci_read_config_byte(pdev, PCI_INTERRUPT_LINE, &data8);
      INFO ("%s: PCI_INTERRUPT_LINE = %d\n", CUR_DEV.name, data8);
      INFO ("Probing success!\n");
      return 0;
    }
  }
#endif

  //Data provider is real firmware through IRQ
  if ((data32 = request_irq(pdev->irq, &spct6100_interrupt,
          IRQF_SHARED, CUR_DEV.name,
          (void *)&CUR_DEV))) {

    ERROR ("%s: Could not reserve IRQ %d\n", CUR_DEV.name, pdev->irq);
    pci_read_config_byte(pdev, PCI_INTERRUPT_PIN, &data8);
    ERROR ("%s: PCI_INTERRUPT_PIN = %d\n", CUR_DEV.name, data8);
    pci_read_config_byte(pdev, PCI_INTERRUPT_LINE, &data8);
    ERROR ("%s: PCI_INTERRUPT_LINE = %d\n", CUR_DEV.name, data8);
    rc = -EIO;
    goto err_out_free_master_buf;
  } else {

    INFO ("%s: Reserve IRQ %d\n", CUR_DEV.name, pdev->irq);
    pci_read_config_byte(pdev, PCI_INTERRUPT_LINE, &data8);
    INFO ("%s: PCI_INTERRUPT_LINE = %d\n", CUR_DEV.name, data8);
  }

  INFO ("Probing success!\n");
  return 0;

err_out_free_master_buf:
  if (spct6100_reserved_addr == 0) {
#ifdef __arm__
    iounmap (CUR_DEV.master_buf_logic);
#endif
    free_pages (CUR_DEV.master_free_pages, SPCT6100_DVR_ORDER);
  } else {
    iounmap (CUR_DEV.master_buf_logic);
  }
err_out_unmap:
  pci_iounmap (pdev, CUR_DEV.mmio_vaddr);
err_out_release_res:
  pci_release_region (pdev, 0);
err_out_disable_pci:
  pci_disable_device (pdev);
err_out:
  return rc;
}

void
spct6100_dvr_free_resources(void *dev)
{
  struct pci_dev *pdev = (struct pci_dev *)dev;

  free_irq(pdev->irq, &CUR_DEV);
  if (spct6100_reserved_addr == 0) {
#ifdef __arm__
    iounmap (CUR_DEV.master_buf_logic);
#endif
    free_pages (CUR_DEV.master_free_pages, SPCT6100_DVR_ORDER);
  } else {
    iounmap (CUR_DEV.master_buf_logic);
  }
  pci_iounmap (pdev, CUR_DEV.mmio_vaddr);
  pci_release_region (pdev, 0);
  pci_disable_device (pdev);
}
#undef CUR_DEV
