#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <fcntl.h>
#include "lib.h"

#define OPTION_NUM 2

static int g_nTrace = 0;
static char *g_szOptions[OPTION_NUM];

static int
trace_log (const char *fmt, ...)
{
    if (g_nTrace) {
        int n;
        va_list vp;

        va_start (vp, fmt);
        n = vprintf (fmt, vp);
        va_end (vp);
        return n;
    }
    return 0;
}

static void
process_command_line (int argc, char *argv[])
{
    int i;
    int show_help = 0;
    int invalid_option = 0;
    int opt_idx = 0;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') {
            if (opt_idx < OPTION_NUM) {
                g_szOptions[opt_idx] = argv[i];
                opt_idx++;
                continue;
            } 
            invalid_option = 1;
            break;
        }
        if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help")) {
            show_help = 1;
            break;
        } else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--trace")) {
            g_nTrace = 1;
        } else {
            invalid_option = 1;
            break;
        }
    }

    if (show_help) {
        printf ("Usage: %s [OPTION] <DEVICE> <ADDRESS>\n", argv[0]);
        printf ("Run code from <DEVICE> <ADDRESS>\n");
        printf ("OPTION:\n");
        printf ("-h,--help        show usage\n");
        printf ("-t,--trace       print trace log\n");
        exit (0);
    } else if (invalid_option) {
        printf ("%s: invalid option -- `%s`\n", argv[0], argv[i]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    } else if (opt_idx != OPTION_NUM) {
        printf ("%s: necessary options missed\n", argv[0]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    }
}

static int
CheckDspStatus (int fd)
{
#define DSP_RUN         0x1
#define DSP_EXCEPTION   0x2
#define DSP_DEBUGGING   0x4
    unsigned int dsp_status;

    while (1) {
        dsp_status = -1;
        if (g_ctrl_DSPSTATUS (fd, (int *)&dsp_status) != 0) {
            trace_log ("g_ctrl_DSPSTATUS fail\n");
            return -1;
        }
        if (dsp_status == 0x0) {
            trace_log( "Program Trapped.\n\n" );
            break;
        } else if (dsp_status == DSP_RUN) {
            usleep (10000);
        } else if (dsp_status == DSP_EXCEPTION) {
            trace_log("DSP Exception!!\n\n");
            return -1;
        } else if (dsp_status == DSP_DEBUGGING) {
            trace_log("DSP in Debugging mode!!\n\n");
            return -1;
        } else {
            trace_log ("unknow DSP status: 0x%x\n", dsp_status);
            return -1;
        }
    }

    return 0;
}

int
main (int argc, char *argv[])
{
    char *device_name;
    unsigned long address;
    int fd;
    int rc;

    process_command_line (argc, argv);
    device_name = g_szOptions[0];
    if (strncmp (g_szOptions[1], "0x", 2) == 0) {
        sscanf (g_szOptions[1], "%lx", &address);
    } else {
        sscanf (g_szOptions[1], "%lu", &address);
    }

    trace_log ("Enable device %s <0x%08lx>\n", device_name, address);
    fd = open (device_name, O_RDWR|O_SYNC);
    if (fd < 0) {
        trace_log ("Open device: %s failed\n", device_name);
        return -1;
    }
    rc = s_ctrl_RUNCODE (fd, address);
    if (rc != 0) {
      trace_log ("s_ctrl_RUNCODE failed\n");
    } else {
      rc = CheckDspStatus (fd);
    }
    close (fd);
    return rc;
}
