#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <fcntl.h>
#include "lib.h"

#define OPTION_NUM 1

static int g_nTrace = 0;
static char *g_szOptions[OPTION_NUM];

static int
trace_log (const char *fmt, ...)
{
    if (g_nTrace) {
        int n;
        va_list vp;

        va_start (vp, fmt);
        n = vprintf (fmt, vp);
        va_end (vp);
        return n;
    }
    return 0;
}

static void
process_command_line (int argc, char *argv[])
{
    int i;
    int show_help = 0;
    int invalid_option = 0;
    int opt_idx = 0;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') {
            if (opt_idx < OPTION_NUM) {
                g_szOptions[opt_idx] = argv[i];
                opt_idx++;
                continue;
            } 
            invalid_option = 1;
            break;
        }
        if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help")) {
            show_help = 1;
            break;
        } else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--trace")) {
            g_nTrace = 1;
        } else {
            invalid_option = 1;
            break;
        }
    }

    if (show_help) {
        printf ("Usage: %s [OPTION] <DEVICE>\n", argv[0]);
        printf ("Init <DEVICE> env\n");
        printf ("OPTION:\n");
        printf ("-h,--help        show usage\n");
        printf ("-t,--trace       print trace log\n");
        exit (0);
    } else if (invalid_option) {
        printf ("%s: invalid option -- `%s`\n", argv[0], argv[i]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    } else if (opt_idx != OPTION_NUM) {
        printf ("%s: necessary options missed\n", argv[0]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    }
}

int
main (int argc, char *argv[])
{
    char *device_name;
    int fd;
    int rc;

    process_command_line (argc, argv);
    device_name = g_szOptions[0];

    trace_log ("Initenv device %s\n", device_name);
    fd = open (device_name, O_RDWR|O_SYNC);
    if (fd < 0) {
        trace_log ("Open device: %s failed\n", device_name);
        return -1;
    }
    rc = s_ctrl_INITENV (fd);
    if (rc != 0) {
        trace_log ("s_ctrl_INITENV failed\n");
    }
    close (fd);
    return rc;
}
