#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <fcntl.h>
#include "lib.h"

#define OPTION_NUM 3

static int g_nTrace = 0;
static char *g_szOptions[OPTION_NUM];

#define BUFFER_SIZE 4096
static char g_strUpBuf[BUFFER_SIZE];

static int
trace_log (const char *fmt, ...)
{
    if (g_nTrace) {
        int n;
        va_list vp;

        va_start (vp, fmt);
        n = vprintf (fmt, vp);
        va_end (vp);
        return n;
    }
    return 0;
}

static void
process_command_line (int argc, char *argv[])
{
    int i;
    int show_help = 0;
    int invalid_option = 0;
    int opt_idx = 0;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') {
            if (opt_idx < OPTION_NUM) {
                g_szOptions[opt_idx] = argv[i];
                opt_idx++;
                continue;
            }
            invalid_option = 1;
            break;
        }
        if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help")) {
            show_help = 1;
            break;
        } else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--trace")) {
            g_nTrace = 1;
        } else {
            invalid_option = 1;
            break;
        }
    }

    if (show_help) {
        printf ("Usage: %s [OPTION] <DEVICE> <ADDRESS> <COUNT>\n", argv[0]);
        printf ("Read data from <DEVICE> <ADDRESS>,<ADDRESS + 4*COUNT - 1>\n");
        printf ("OPTION:\n");
        printf ("-h,--help        show usage\n");
        printf ("-t,--trace       print trace log\n");
        exit (0);
    } else if (invalid_option) {
        printf ("%s: invalid option -- `%s`\n", argv[0], argv[i]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    } else if (opt_idx != OPTION_NUM) {
        printf ("%s: necessary options missed\n", argv[0]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    }
}

static int
upload (int fd, unsigned int memaddr, unsigned int count)
{
    unsigned int up_size, left_size;
    int i;

    if (count == 0) {
        return 0;
    }
    left_size = 4 * count;
    while (left_size > 0) {
        if (left_size < sizeof (g_strUpBuf)) {
            up_size = left_size;
        } else {
            up_size = sizeof (g_strUpBuf);
        }

        if (g_ctrl_UPLOAD (fd, memaddr, g_strUpBuf, up_size) != 0) {
            trace_log ("g_ctrl_UPLOAD fail\n");
            return -1;
        }
        for (i = 0; i < up_size; i += 4) {
            printf ("0x%08x,", *(unsigned int *)&g_strUpBuf[i]);
            if ((i + 4) % 32 == 0) {
                printf ("\n");
            } else {
                printf (" ");
            }
        }

        left_size -= up_size;
        memaddr += up_size;
    }
    printf ("\n");
    return 0;
}

int
main (int argc, char *argv[])
{
    char *device_name;
    unsigned long address;
    unsigned int count;
    int fd;
    int rc;

    process_command_line (argc, argv);
    device_name = g_szOptions[0];
    if (strncmp (g_szOptions[1], "0x", 2) == 0) {
        sscanf (g_szOptions[1], "%lx", &address);
    } else {
        sscanf (g_szOptions[1], "%lu", &address);
    }
    if (strncmp (g_szOptions[2], "0x", 2) == 0) {
        sscanf (g_szOptions[2], "%x", &count);
    } else {
        sscanf (g_szOptions[2], "%u", &count);
    }

    trace_log ("Read dword from %s <0x%08lx>,<0x%08lx>\n",
               device_name, address, address + 4*count - 1);
    fd = open (device_name, O_RDWR|O_SYNC);
    if (fd < 0) {
        trace_log ("Open device: %s failed\n", device_name);
        return -1;
    }
    rc = upload (fd, address, count);
    close (fd);
    return rc;
}
