#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <fcntl.h>
#include "lib.h"

#define OPTION_NUM 4

static int g_nCheck = 0;
static int g_nTrace = 0;
static char *g_szOptions[OPTION_NUM];

#define BUFFER_SIZE 4096
static char g_strDownBuf[BUFFER_SIZE];
static char g_strUpBuf[BUFFER_SIZE];

static int
trace_log (const char *fmt, ...)
{
    if (g_nTrace) {
        int n;
        va_list vp;

        va_start (vp, fmt);
        n = vprintf (fmt, vp);
        va_end (vp);
        return n;
    }
    return 0;
}

static void
process_command_line (int argc, char *argv[])
{
    int i;
    int show_help = 0;
    int invalid_option = 0;
    int opt_idx = 0;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') {
            if (opt_idx < OPTION_NUM) {
                g_szOptions[opt_idx] = argv[i];
                opt_idx++;
                continue;
            }
            invalid_option = 1;
            break;
        }
        if (!strcmp (argv[i], "-h") || !strcmp (argv[i], "--help")) {
            show_help = 1;
            break;
        } else if (!strcmp (argv[i], "-t") || !strcmp (argv[i], "--trace")) {
            g_nTrace = 1;
        } else if (!strcmp (argv[i], "-c") || !strcmp (argv[i], "--check")) {
            g_nCheck = 1;
        } else {
            invalid_option = 1;
            break;
        }
    }

    if (show_help) {
        printf ("Usage: %s [OPTION] <DEVICE> <ADDRESS> <DATA> <COUNT>\n", argv[0]);
        printf ("Write <DATA> to <DEVICE> <ADDRESS>,<ADDRESS + 4*COUNT - 1>\n");
        printf ("OPTION:\n");
        printf ("-h,--help        show usage\n");
        printf ("-t,--trace       print trace log\n");
        printf ("-c,--check       download then upload and comprare them\n");
        exit (0);
    } else if (invalid_option) {
        printf ("%s: invalid option -- `%s`\n", argv[0], argv[i]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    } else if (opt_idx != OPTION_NUM) {
        printf ("%s: necessary options missed\n", argv[0]);
        printf ("Try `%s --help' for more information.\n", argv[0]);
        exit (1);
    }
}

static int
download (int fd, unsigned int memaddr, unsigned int data, unsigned int count)
{
    unsigned int down_size, left_size;
    int i;

    if (count == 0) {
        return 0;
    }
    left_size = 4 * count;
    while (left_size > 0) {
        if (left_size < sizeof (g_strDownBuf)) {
            down_size = left_size;
        } else {
            down_size = sizeof (g_strDownBuf);
        }

        for (i = 0; i < down_size; i += 4) {
            *(unsigned int *)&g_strDownBuf[i] = data;
        }

        if (s_ctrl_DOWNLOAD (fd, memaddr, g_strDownBuf, down_size) != 0) {
            trace_log ("s_ctrl_DOWNLOAD fail\n");
            return -1;
        }

        if (g_nCheck) {
            if (g_ctrl_UPLOAD (fd, memaddr, g_strUpBuf, down_size) != 0) {
                trace_log ("g_ctrl_UPLOAD fail\n");
                return -1;
            }
            if (memcmp (g_strUpBuf, g_strDownBuf, down_size) != 0) {
                trace_log ("download check fail\n");
                return -1;
            }
        }

        left_size -= down_size;
        memaddr += down_size;
    }
    if (g_nCheck) {
        trace_log ("download size %d, check pass\n", 4 * count);
    }
    return 0;
}

int
main (int argc, char *argv[])
{
    char *device_name;
    unsigned long address;
    unsigned int data;
    unsigned int count;
    int fd;
    int rc;

    process_command_line (argc, argv);
    device_name = g_szOptions[0];
    if (strncmp (g_szOptions[1], "0x", 2) == 0) {
        sscanf (g_szOptions[1], "%lx", &address);
    } else {
        sscanf (g_szOptions[1], "%lu", &address);
    }
    if (strncmp (g_szOptions[2], "0x", 2) == 0) {
        sscanf (g_szOptions[2], "%x", &data);
    } else {
        sscanf (g_szOptions[2], "%u", &data);
    }
    if (strncmp (g_szOptions[3], "0x", 2) == 0) {
        sscanf (g_szOptions[3], "%x", &count);
    } else {
        sscanf (g_szOptions[3], "%u", &count);
    }

    trace_log ("Write dword: 0x%08x to %s <0x%08lx>,<0x%08lx>\n",
               data, device_name, address, address + 4*count - 1);
    fd = open (device_name, O_RDWR|O_SYNC);
    if (fd < 0) {
        trace_log ("Open device: %s failed\n", device_name);
        return -1;
    }
    rc = download (fd, address, data, count);
    close (fd);
    return rc;
}
