#!/bin/sh

show_help(){
cat << EOF
Usage: $0 [OPTIONS]

OPTIONS:
  -h, --help            display this help and exit
  -t, --trace           enable trace
  -g, --gpio            force driver work in gpio mode
  -r reserved_address   specify the reserved high mem phy addr
EOF
exit 0
} #show_help()

tf=""
gpio=""
reserved=0
for ac_option do
  case "$ac_option" in
  -h|--help)
    show_help
    ;;
  -t|--trace)
    tf="-t"
    ;;
  -g|--gpio)
    gpio="spct6100_gpio_mode=1"
    ;;
  -r|--reserved)
    reserved=1
    ;;
  *)
    if [ ${reserved} -eq 1 ]
	then
      reserved=$ac_option
	else

      echo "error: unknown option -- $ac_option"
      exit 1
	fi
    ;;
  esac
done

_tmp_path=`dirname $0`
ROOT_DIR=`cd ${_tmp_path}; pwd`
TOOL_DIR=${ROOT_DIR}/firmware
BIN_DIR=${ROOT_DIR}/firmware/bin
DRIVER_FILE=${ROOT_DIR}/driver/spct6100.ko

rmmod spct6100
echo "modprobe videodev"
modprobe videodev
echo "insmod ${DRIVER_FILE} spct6100_process_expire=10000 ${gpio}"
if [ ${reserved} -eq 0 ]
then
insmod ${DRIVER_FILE} spct6100_process_expire=10000 ${gpio}
else
insmod ${DRIVER_FILE} spct6100_process_expire=10000 spct6100_reserved_addr=${reserved} ${gpio}
fi
sleep 1

cf="-c"
devices="/dev/video1 /dev/video3 /dev/video5 /dev/video7"
for dev_name in ${devices}; do
    if [ ! -c ${dev_name} ]; then continue; fi

    echo "Initing device ${dev_name} ......"
    ##--- Reset IC
    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E162004 0x1 0x1 \
		${tf}
    if [ $? -ne 0 ]; then
        echo "Reset IC: failed"
        exit 1
    fi

    ##--- Init DDR
    ${TOOL_DIR}/write_file_to_device ${dev_name} \
		0x1C000000 ${BIN_DIR}/ddr_config_128_sync_16bit.text.bin \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init DDR: step 1 failed"
        exit 1
    fi

    ${TOOL_DIR}/write_file_to_device ${dev_name} \
		0x1E000000 ${BIN_DIR}/ddr_config_128_sync_16bit.data.bin \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init DDR: step 2 failed"
        exit 1
    fi

    ${TOOL_DIR}/enable_device ${dev_name} 0x1C000000 ${tf}
    if [ $? -ne 0 ]; then
        echo "Init DDR: step 3 failed"
        exit 1
    fi

    ##--- Init ICM arbiter
    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E162050 0x00C8 0x1 \
		${tf} ${cf};
    if [ $? -ne 0 ]; then
        echo "Init ICM arbiter: step 1 failed"
        exit 1
    fi

    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E162054 0x012C 0x1 \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init ICM arbiter: step 2 failed"
        exit 1
    fi

    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E162058 0x01F4 0x1 \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init ICM arbiter: step 3 failed"
        exit 1
    fi

    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E150000 0xDDA9961E 0x1 \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init ICM arbiter: step 4 failed"
        exit 1
    fi

    ##--- Init Local Data Memory
    ${TOOL_DIR}/write_dword_to_device ${dev_name} \
		0x1E000000 0x00000000 0x4000 \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Init Local Data Memory: failed"
        exit 1
    fi

    ##--- Download dsp_stub_main.text.bin : 0x01900000
    ${TOOL_DIR}/write_file_to_device ${dev_name} \
		0x01900000 ${BIN_DIR}/dsp_stub_main.text.bin \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Download dsp_stub_main.text.bin: failed"
        exit 1
    fi

    ##--- Download ST6100_TABLE.bin : 0x03E80000
    ${TOOL_DIR}/write_file_to_device ${dev_name} \
		0x03E80000 ${BIN_DIR}/ST6100_TABLE.bin \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Download ST6100_TABLE.bin: failed"
        exit 1
    fi

    ##--- Download TimerISR.bin : 0x1E003E00
    ${TOOL_DIR}/write_file_to_device ${dev_name} \
		0x1E003E00 ${BIN_DIR}/TimerISR.text.bin \
		${tf} ${cf}
    if [ $? -ne 0 ]; then
        echo "Download TimerISR.bin: failed"
        exit 1
    fi

    ##--- Driver init env
    ${TOOL_DIR}/init_env ${dev_name} ${tf}
    if [ $? -ne 0 ]; then
        echo "Driver init env: failed"
        exit 1
    fi

    echo "*********************************************************"
    echo "Initialize ${dev_name} OK"
    echo "*********************************************************"
done
